# Procedures to output xpk and ppm.out files from NMRView to Cyana 2
# Written by Pascal Mercier
# pascalmercier@shaw.ca
# University of Alberta, Edmonton, Canada
# October 2005
#############################################################################################################################


# Version 3.0 b8
# now runs with the latest NMRViewJ versions

# Version 3.0 b7
# rewrote things more intelligently - I hope
# Exports for Cyana 2
# got code from iAssignUwatch for writing up xpk files and calculating folding shifts

# Version 2.6.1
# fixed a bug with lys QD, QG and glu QG
# this scipt is becoming a mess and a shame

# Version 2.6
# July 23 2005
# fixed a bug. it would not convert the HBs properly to QBs in case one of the HA or HB was not assigned
# also added code to deal with aromatic residues

# Version 2.5
# Now handles methyl groups IDed as MD1, MG2, etc from NMRView
# Java compatible

# Version 2.4
# outut files for cyana version 2.  the names are fully compatible with the cyana 2 library (cyanalib).

# Version 2.3.1
# added the option to export volumes or intensities

#Version 2.3
# Added a gui window to give instructions on how the script works

# Version 2.2
# now compatible with NMRViewJ

# The syntax goes as follow:
# nmrviewtocyana name_of_output_prot_file {1st_peaklist heteroatomtype(C or N) cs_folding(1/0) 1/0 1/0 1/0} {2nd_peaklist heteroatomtype(C or N) cs_folding(1/0) 1/0 1/0 1/0} # cyana does not like a peak to be numbered 0.  as a solution, peak 0 is renumber n+1 and put at the end of the list, where n=number of peaks

# **************
# there is a bug in Cyana 1.0.x: the checkpeak commands seems to work only if the indirect proton is in the 1st dimension
# so by default, this script will output things to be in this order: hH(N/C)
# ******************************************************************************************************************************


proc nmrviewtocyana_convert {args} {

global cyana_name_conversion

set listofpeaklist [lrange $args 0 end]
	
set numberofpeaklists [llength $listofpeaklist]
set peaklists ""
for {set i 0} {$i<$numberofpeaklists} {incr i} {
	set current  							[lindex $listofpeaklist $i]
	set peaklistname 						[lindex $current 0]
	lappend peaklists $peaklistname
	#set hetero($peaklistname) [lindex $current 1]
	set needfolding($peaklistname) 			[lindex $current 1]
	set keeponlyid($peaklistname) 			[lindex $current 2]
	set include_ambiguous($peaklistname)  	[lindex $current 3]
	set wipeoutambiguous($peaklistname)  		[lindex $current 4]
	set skip_markedfordeletion($peaklistname) [lindex $current 5]
	set onlyoutputshortrange($peaklistname) [lindex $current 6]
	set onlyoutputlongrange($peaklistname) 	[lindex $current 7]
	set wipeoutlongrange($peaklistname) 	[lindex $current 8]
}

# store current shifts in memory and store their cyana names in an array. this way the conversion does not have to be done each time
puts [string repeat # 50]
puts "nomenclature change for Cyana ... taking current atom shift assignments and stereospecificity into account"
build_Cyana_translation_table
puts [string repeat # 50]

# the next two lines are nessary when dealing with xpk in C or Java
set cyana_name_conversion() ""
set cyana_name_conversion(?) "?"

# write out the peaklists
# the _cyana extension in the name is automatically added

foreach peaklist $peaklists {
	
	#make sure that the peak list exists
	if {[lsearch [nv_peak list] $peaklist]<0} {
		bell
		puts "the peak list $peaklist does not exist"
		continue
	}
	
	set relation [nv_peak relation $peaklist]
	set ndim [llength [nv_peak label $peaklist]]
    
	puts "writting up peak list $peaklist\_cyana.xpk ..."
	update
	
	#save the peak list
	#folding is perfomed automatically in the writecyanapeaks procedure
	writecyanapeaks $peaklist $peaklist\_cyana.xpk $skip_markedfordeletion($peaklist) $keeponlyid($peaklist)  $include_ambiguous($peaklist) $wipeoutambiguous($peaklistname)  $onlyoutputshortrange($peaklistname) $onlyoutputlongrange($peaklistname) $wipeoutlongrange($peaklistname)

	# write out the chemical shift file for that peak list
	puts "writting up peak the chemical shift file $peaklist\_cyana.xpk"
	update
	writecyanappm $peaklist $peaklist\_ppm.out  $needfolding($peaklist)
}

puts ""; puts "Done exporting the peak list $peaklist"
}






###########################################################################
#procedure to write xpk peak lists with cyana/nmrview nomenclature
# derived from iAssignUwatch
#this is also based on writepks.tcl
#however it has a little bit more options
###########################################################################
proc writecyanapeaks {peaklist filename skipdeletedpeaks keeponlyassignedpeaks include_ambiguous wipeoutambiguous onlyoutputshortrange onlyoutputlongrange wipeoutlongrange} {

global Nv_Priv
global Nv_Natoms
global cyana_name_conversion

if {$Nv_Priv(javaMode)} {
	set string_for_no_shift_available ""
	set string_for_no_peak_assignment ""
} else {
	set string_for_no_peak_assignment "?"
	set string_for_no_shift_available -1000000
}


set namedimensions [nv_peak labels $peaklist]

puts ""
# check the consistency of the peak assignments in the peak list with the current atom assignments
foreachpeak i $peaklist {
	foreach dim $namedimensions {
		set currentassignments [nv_peak elem $dim.L $peaklist.$i]
		foreach assignment $currentassignments {
			if {$assignment!=$string_for_no_peak_assignment} {
				# check if the atom is assigned in the shift panel
				if {[nv_atom elem ppm $assignment]==$string_for_no_shift_available} {
					# issue a warning
					puts "Warning: no chemical shift assignment defined for $assignment as assignment for peak $peaklist.$i "
				}
			}
		}
	}
}
puts ""

#check which version of nmrview is running, C or Java
if {$Nv_Priv(javaMode)} {
	set string_for_no_peak_assignment ""
} else {
	set string_for_no_peak_assignment "?"
}

set numberoutput 0
set numb_ambiguous 0 
set numb_unambiguous 0

set good_peak_cyana ""
set cyana_wipe_peak ""


set fileid [open $filename w] 

set labels [nv_peak label $peaklist]
set ndim [llength $labels]
for {set i 0} {$i<$ndim} {incr i} {set namedim($i) [lindex $labels $i]}


set fields "LPWBEJUVISCF"
set newfields [split $fields {}]
puts $fileid "label dataset sw sf"

foreach pf "label dataset sw sf" {puts $fileid [nv_peak $pf $peaklist]}
   
set linetoprint ""
   
for {set j 0} {$j<$ndim} {incr j} {
	set lab [lindex $labels $j]
	foreach pf "L P W B E J U" {
      if {[string match *$pf* $newfields]} {
      	lappend linetoprint $lab.$pf
       }
     }
}
    
set headeradd(V) vol
set headeradd(I) int
set headeradd(S) stat
set headeradd(C) comment
set headeradd(F) flag0
    
foreach pf "V I S C F" {
	if {[string match *$pf* $newfields]} {
		lappend linetoprint $headeradd($pf)
	}
}
    
puts $fileid $linetoprint

foreachpeak peak $peaklist {
	#check if the peak passes test to be ouput
	

	
	#check if it needs to skip peaks that are unassigned
	if {$keeponlyassignedpeaks} {
		set temp  [nv_peak elem $namedim(0).L $peaklist.$peak]
		if {$temp==$string_for_no_peak_assignment} {continue}
		#check if it needs to skip ambiguous peaks
		if {!$include_ambiguous} {
			if {[llength $temp]>1} {continue}
		}
	}

	#skip peaks marked for deletion
	if {$skipdeletedpeaks} {
		if {[nv_peak elem status $peaklist.$peak]==-1} {continue}
	}
	
	set temp1 [nv_peak elem $namedim(0).L $peaklist.$peak]
	set temp2 [nv_peak elem $namedim(1).L $peaklist.$peak]
	set nass  [llength $temp1]


	set gotshort 0
	set gotlong  0
	set needwipe 0
	# analyse all assignments
	
	if {!($nass > 1 && $wipeoutambiguous)} {
		for {set i 0} {$i<$nass} {incr i} {
			set res1 [file root [lindex $temp1 $i]]
			set res2 [file root [lindex $temp2 $i]]
			if {$res1!=$string_for_no_peak_assignment && $res2!=$string_for_no_peak_assignment} {
				if {[expr abs($res1-$res2)]>=5} {set gotlong 1} else {set gotshort 1}
			}
		}
		
		
		#check if it needs to skip peaks that are shortrange or longrange
		if {$onlyoutputlongrange || $onlyoutputshortrange || $wipeoutlongrange} {
			#make sure that all assignments are longrange
			
			if {$onlyoutputlongrange  && $gotshort} {continue}
			if {$onlyoutputshortrange && $gotlong}  {continue}
			if {$wipeoutlongrange && $gotlong} {
				set needwipe 1
			}
		}
	} else {
		# it's an ambiguous peak and the user wants to wipe out the assignments
		set needwipe 1
	}	
	
	
	#the peak passes all tests
	incr numberoutput 
	lappend good_peak_cyana $peak
	
	set linetoprint $peak
	for {set i 0} {$i<$ndim} {incr i} {
		
		# do L
		# unfortunately if I use lappend here I loose the {} if the assignment is "?" (or empty string in Java)
		# must treat linetoprint as a string
		if {$needwipe} {
			lappend cyana_wipe_peak $peak
			set linetoprint [concat $linetoprint "{$string_for_no_peak_assignment}"]
		} else {
			set assignment [nv_peak elem $namedim($i).L $peaklist.$peak]
			
			#need to loop through each assignment
			set newassignment ""
			foreach item $assignment {
				lappend newassignment $cyana_name_conversion($item)
			}
			set linetoprint [concat $linetoprint "{$newassignment}"]
			
		}	
		#do P W B E J
		foreach letter "P W B E J" {
			set linetoprint [concat $linetoprint [nv_peak elem $namedim($i).$letter $peaklist.$peak]]
		}
	
		#do U
		set linetoprint [concat $linetoprint "{[nv_peak elem $namedim($i).U $peaklist.$peak]}"]
	}
	
	# do vol, int and stat
	foreach word "vol int stat"  {
			set linetoprint [concat $linetoprint [nv_peak elem $word $peaklist.$peak]]	
	}
	
	#do comment
	set linetoprint [concat $linetoprint "{[nv_peak elem com $peaklist.$peak]}"]
	
	#do flag0
	set linetoprint [concat $linetoprint [nv_peak elem flag0 $peaklist.$peak]]
	
	#outputs the result
	puts $fileid  $linetoprint
}
	
	
puts "The peak list $peaklist was saved as $filename and contains $numberoutput peaks"

close $fileid	



#end of procedure writecyanapeaks
}



####################################################################################
# procedure to saves a chemical shift file for Cyana 2 with the
# nmrview atom nomenclature
# 
# the shifts are folded according to the sweepwidth of the given peaklist if needed
####################################################################################

proc writecyanappm {xpklist outputfile needfolding} {

global Nv_Priv
global Nv_Natoms
global cyana_name_conversion

#check which version of nmrview is running, C or Java
if {$Nv_Priv(javaMode)} {
	set string_for_no_shift_available ""
} else {
	set string_for_no_shift_available -1000000
}


set FileToWrite [open $outputfile w]

# for 2D support, needfolding needs to be set to 0

if {$needfolding} {
	set dataset 	[nv_peak dataset $xpklist]
	set datasetsf 	[nv_dataset sf $dataset 3]
	set datasetsw 	[nv_dataset sw $dataset 3]
	set datasetsize [nv_dataset size $dataset 3]
	set datasetsf 	[nv_dataset sf $dataset 3]
	set fold 		[expr $datasetsw/$datasetsf]
	
	#very important here: must consider the refpt
	set datasetcenter [nv_dataset ref   $dataset 3]
	set datasetrefpt  [nv_dataset refpt $dataset 3]
	set limitup 	[expr $datasetcenter+$fold/$datasetsize*($datasetrefpt-1)]
	set limitdown 	[expr $datasetcenter-$fold/$datasetsize*($datasetsize-$datasetrefpt)]


	#determine the hetero atom type (C or N)
	#from the name of the 3rd dimension
	if {[string match *C* [lindex [nv_peak label $xpklist] 2]]} {
		set heteroatom C
		} elseif {[string match *N* [lindex [nv_peak label $xpklist] 2]]} {
		set heteroatom N
		} else {
		bell; bell; bell
		puts "iAssignUwatch cannot determine the hetero atom type (C or N) in the 3rd dimension of $xpklist"
		puts "please label the dimension with something like C13, 13C, C,  N15, 15N or N"
		return	
	}
} else {
	set heteroatom ""
}

for {set i 1} {$i<=$Nv_Natoms} {incr i} {
	set currentppm [nv_atom elem ppm $i]
	if {$currentppm==$string_for_no_shift_available} {continue}
	
	set atom_name [nv_atom elem aname $i]
	if {[string match $heteroatom* $atom_name]==1 && $needfolding} {
		set change 0
		# do an unlimited number of folding until the shift fits in the spectral width
		while {$currentppm<=$limitdown || $currentppm>=$limitup} {
			set change 1
			# calculate the right value for the shift
			if {$currentppm > $limitup}   {set currentppm [expr $limitdown-$limitup + $currentppm]}
			if {$currentppm < $limitdown} {set currentppm [expr $limitup-$limitdown + $currentppm]}
		}
	
		if {$change} {puts [format "%-10s currenshift:%12.3f foldedshift:%12.3f" "[nv_atom elem seq $i].[nv_atom elem aname $i]" "[nv_atom elem ppm $i]" "$currentppm"]}
	}
	
	#now output to file
	set residue_number [nv_atom elem seq $i]
	set stereo_value [nv_atom elem stereo $i]
	puts $FileToWrite [format "%-8s %15.3f %5d" $cyana_name_conversion($residue_number.$atom_name) $currentppm $stereo_value]
}

close $FileToWrite
puts "Done creating the shift $outputfile for Cyana 2"

#end of writecyanappm
}





####################################################################################################
# procedure that changes the nomenclature of an atom to the cyana 2 format
# depending on which version is running (Java or C), two different procedures are called
# I kept two seperate procedures because eventually, the C version will disepear
# and it will be easier to clean the code that way
# another reason is because in nmrviewJ 6.2.16, nv_atom elem sstereo does not return the right thing
####################################################################################################
proc nmrviewname_to_cyananame {identifier} {
	global Nv_Priv
	
	if {$Nv_Priv(javaMode)} {
		nmrviewname_to_cyananame_J $identifier
	} else {
		nmrviewname_to_cyananame_C $identifier
	}
}





###########################################################################
# procedure that changes the nomenclature of an atom
# to the cyana 2 format, for the J version of NMRView
###########################################################################
proc nmrviewname_to_cyananame_J {identifier} {

#check if the identifier exists
set error [catch "nv_atom elem equiv $identifier" class]
if {$error} {return -1}

set temp [split $identifier .]
set resnumber [lindex $temp 0]
set atomtype  [lindex $temp 1]
set restype   [string toupper [nv_atom elem rname $identifier]]

set isalreadyspeudo 0
# with the current version of nmrviewJ (6.2.18), nv_atom equiv does not return
# anything for a methyl group Ided with M

if {[string range $atomtype 0 0]=="M"} {
	set isalreadyspeudo 1
	set ismethyl 1
	# artificially set some variables to fool the script below and force it to work with Ms
	if {$restype=="VAL" || $restype=="LEU"} {
		set number_of_relatedH 2
		set last_letter [string range $atomtype 2 2]
		if {$last_letter==1} {set partner_number 2} else {set partner_number 1}
		set equivalence "dummy dummy dummy dummy dummy {M[string range $atomtype 1 1]$partner_number}"
	} else {
		# for all of the other methyl containing residues
		return $identifier
	}
	
} else {
	set equivalence [nv_atom elem equiv $identifier]
	set related_H [lindex $equivalence 3]
	set number_of_relatedH [llength $related_H]
	set ismethyl 0
}


if {$number_of_relatedH==0} {
	# the atom is unique
	return $identifier
} else {

	# there are associated protons
	# check if it's a methyl group or not
	
	if {$number_of_relatedH==1} {
		#not a methyl group, easy case
		#check if the partner is assigned
		set partner_name  [lindex $related_H 0]
		set partner_shift [nv_atom elem ppm $resnumber.$related_H]
	} else {
		# it's a methyl group
		set ismethyl 1
		# check if the methyl group has a partner (like a Val HG, or LEU HD)
		if { [llength $equivalence] ==4 } {
			# it has no partner
			# set the partner_shift to a dummy value, just for the rest
			# of the script
			set partner_shift 1
			set partner_name ""
			set new_name $atomtype
		} else {
			# it has a partner associated with it
			# check if the partner is assigned
			# for something like a val residue, equivalence is string like
			# c 2 2 {HG22 HG23} c 3 4 {HG11 HG12 HG13}
			# will use HG11
			set partner_name  [lindex [lindex $equivalence end] 0]
			set partner_shift [nv_atom elem ppm $resnumber.$partner_name]	
		}
	}
	
	#check if the partner is assigned
	if {$partner_shift !="" && $partner_shift !=[nv_atom elem ppm $identifier]} {
		# the partner is assigned
		# does not need substitution unless it's a methyl group
		set new_name $atomtype
		if {$ismethyl} {
			# replace the H by M and drop the last letter if the name is not M for a methyl already
			if {$isalreadyspeudo} {
				return $identifier
			} else {
				set stringlength [string length $atomtype]
				set new_name [string replace [string range $atomtype 0 [expr $stringlength-2]] 0 0 M]
			}
		}
	} else {
		# the partner is not assigned or the partner is assigned to the same shift
		# substitution is needed
		# if it is a methyl group, then we have special cases (Val and Leu)
		if {$ismethyl} {
			set new_name "QQ[string range $atomtype 1 1]"
		} else {
			# just need to replace things like HB2 to QB
			set stringlength [string length $atomtype]
			set new_name [string replace [string range $atomtype 0 [expr $stringlength-2]] 0 0 Q]
		}
	}
	
	#puts "the related H is $partner_name"
	#puts "its shift is $partner_shift"
	#puts "new name is $new_name"
	return $resnumber.$new_name
}

# end of nmrviewname_to_cyananame_J
}





###########################################################################
# procedure that changes the nomenclature of an atom
# to the cyana 2 format, for the C version of NMRView
# only operates on protons
###########################################################################

proc nmrviewname_to_cyananame_C {identifier} {

set temp [split $identifier .]
set resnumber [lindex $temp 0]
set atomtype  [lindex $temp 1]
set restype   [string toupper [nv_atom elem rname $identifier]]

# no substitution needed if it's a C or N
set temp [string range $atomtype 0 0]
if {$temp=="N" || $temp=="C"} {return $identifier}

# change the N-terminal HT1 to HN
if {$atomtype=="HT1"} {return $resnumber.HN}

set nprotons [nv_atom elem sstereo $identifier]

if {$nprotons==0} {return $identifier}

set sstereo [nv_atom elem sstereo $identifier]
# scenario for sstereo
# 1 2 2 : 2 Hs attached to the same C or N
# 1 1 2 : 1  H attached to the same C or N, but there is another H on another C or N that can have the same name
# 1 3 3 : unique Me group (like Ala, Met)
# 2 3 3 6 : non unique Me group
# 2 2 2 4 : non unique H (like HH12 on Arg)
# 2 0 3 6 : non unique methyl group ided with M (Val, Ile)
# 1 0 3   : unique methyl group (Ala, Met, Thr, Ile)

set length_sstereo [llength $sstereo]
set H_attached [lindex $sstereo 1]

# unique protons, like aromatics, but not PHE or TYR
# these two represent exeptions
if {$H_attached==1 && $restype!="PHE" && $restype!="TYR"} {return $identifier}

set mightneed_substitution 1
set ismethyl 0

set nprotons [lindex $sstereo 2]
if {$nprotons==3} {
	set ismethyl 1
	if {[string range $atomtype 0 0]=="M"} {set methyl_starts_M 1} else {set  methyl_starts_M 0} 
	# check if it's a unique methyl group
	if {$length_sstereo==3} {
	
		# check if we need to replace the H by an M
		# or if an M exists already
		
		if {!$methyl_starts_M} {
			set atomtype [string replace $atomtype end end]
			set atomtype [string replace $atomtype 0 0 M]
		}
		return $resnumber.$atomtype
	}
}


# if it might need substitution, check if the partner is assigned

if {$mightneed_substitution} {
	
	# need to get the right name for the partner
	if {$nprotons!=3} {
		set last_letter [string range $atomtype end end]
		if {$last_letter==1} {set partner_number 2} else {set partner_number 1}
		set name_partner [string replace $atomtype end end $partner_number]
	} else {
		# it's a methyl group
		set last_letter [string range $atomtype 2 2]
		if {$last_letter==1} {set partner_number 2} else {set partner_number 1}
		set name_partner [string replace $atomtype 2 2 $partner_number]
	}
	
	#puts "identifier is $identifier"
	#puts "partner is $name_partner"
	set ppm_partner [nv_atom elem ppm $resnumber.$name_partner]
	#puts "ppm partner is $ppm_partner"
	
	if {$ppm_partner==-1000000.000} {
		# partner is not assigned
		# substitution is needed
		# need Q for normal protons, M for methyl groups as first letter
		# need to drop the last letter
		set atomtype [string replace $atomtype end end]
		if {$ismethyl} { 
			# we're in presence of a Val or Leu
			# need to drop the last letter again if it starts with an H
			if {!$methyl_starts_M} {set atomtype [string replace $atomtype end end]}
			# now change H or M to QQ
			set atomtype [string replace $atomtype 0 0 QQ]	
		} else {
			set atomtype [string replace $atomtype 0 0 Q]
		}
	
	} else {
		# the partner is assigned
		if {$ismethyl && !$methyl_starts_M} {
			set atomtype [string replace $atomtype end end]
			set atomtype [string replace $atomtype 0 0 M]
		}
	}
} 

return $resnumber.$atomtype

# end of nmrviewname_to_cyananame_C
}





#######################################################
# procedure that translates a cyana xpk list 
# using pseudo atom naming like QB, etc, back to nmrview
#########################################################

proc translate_cyana_xpk {peaklist} {

global Nv_Priv Nv_Natoms nmrview_name_conversion

#check which version of nmrview is running, C or Java
if {$Nv_Priv(javaMode)} {
	set string_for_no_shift_available ""
} else {
	set string_for_no_shift_available -1000000
}

# build a name translation map
# I will just use the nmrviewname_to_cyananame procedure
# and apply the reverse mapping. way to save time and code.
# lazy way to do it - but most intelligent way too probably

for {set i 1} {$i<=$Nv_Natoms} {incr i} {
	set resnum [nv_atom elem seq $i]
	set aname  [nv_atom elem aname $i]
	set nmrviewname $resnum.$aname
	if { [nv_atom elem ppm $i] == $string_for_no_shift_available } {
		# there is no chemical shift assignment for this atom
		set cyananame $nmrviewname
	} else {
		set cyananame [nmrviewname_to_cyananame $resnum.$aname]
	}
	set nmrview_name_conversion($cyananame) $nmrviewname
	#if {$cyananame != $nmrviewname} {puts "$cyananame will be changed to $nmrviewname"}
	update
}

set nmrview_name_conversion(?) "?"
set nmrview_name_conversion() ""

puts [string repeat = 50]	
puts "now translating atom names in $peaklist to the NMRView nomenclature"
update

set dimlabels [nv_peak label $peaklist]

foreachpeak i $peaklist {
	foreach dim $dimlabels {
		set newassignment ""
		set currentassignments [nv_peak elem $dim.L $peaklist.$i]
		foreach assignment $currentassignments {
			lappend newassignment $nmrview_name_conversion($assignment)
		}
		# assign the peak with the new name
		nv_peak elem $dim.L $peaklist.$i $newassignment
	}
}

puts "done"
}




####################################################################################
proc build_Cyana_translation_table {} {

global Nv_Priv Nv_Natoms cyana_name_conversion


#check which version of nmrview is running, C or Java
if {$Nv_Priv(javaMode)} {
	set string_for_no_shift_available ""
} else {
	set string_for_no_shift_available -1000000
}


for {set i 1} {$i<=$Nv_Natoms} {incr i} {
	
	set resnum [nv_atom elem seq $i]
	set aname  [nv_atom elem aname $i]
	set nmrviewname $resnum.$aname
	if { [nv_atom elem ppm $i] != $string_for_no_shift_available } {
		set cyana_name_conversion($resnum.$aname) [nmrviewname_to_cyananame $resnum.$aname]
	} else {
		set cyana_name_conversion($resnum.$aname) $nmrviewname
	}
	#if {$cyana_name_conversion($resnum.$aname) != "$resnum.$aname"} {puts "$resnum.$aname changed to $cyana_name_conversion($resnum.$aname) "}
	update
}
}



proc export_to_cyana_help {} {
	global Nv_Priv
	
	# code taken from fill shifts v4.0
	set f .nview_to_cyana_about
	destroy $f
	toplevel $f

	wm title $f "About NMRView/Cyana converter"

	pack [frame $f.fr] 
	
	if {$Nv_Priv(javaMode)} {
		
		jscrollpane $f.fr.javascroll
		text $f.fr.tx -height 40 -width 90 -font {Helvetica -12}
		$f.fr.javascroll add $f.fr.tx 
		pack $f.fr.javascroll -in $f -side top -expand 1 -fill x 
		
	} else {
		pack [text $f.fr.tx -font "Helvetica 12" -wrap word  \
			-yscrollcommand {.nview_to_cyana_about.fr.sb set} ] -side left 
		pack [scrollbar $f.fr.sb -background lightblue \
			-activebackground lightblue \
        	-width 10 -borderwidth 1 -command {.nview_to_cyana_about.fr.tx yview}  \
			 ] -fill y -side right 
	}
	
	

	
	pack [button $f.b1 -text "Close" -default active -activebackground red \
		-command {destroy .nview_to_cyana_about}] -side bottom

	
	$f.fr.tx insert end {
NMRView to Cyana conversion utility - Version 3.0
------------------------------------------------------------------------------------------------------------

This scipt converts the current atom assignments (shifts) and the selected peak list for use in Cyana 2.1.   Proper pseudo-substitution for atom naming is performed.

Two files will be created in the current working directory:
peaklist_ppm.out and  peaklist_cyana.xpk 


Options:

Need 13C or 15N shift folding: 
your chemical shift assignments will be folded according to the sweepwidth of the selected peak list.

Keep only identified peaks:
Only assigned peaks will be exported

Include peaks with ambiguous assignments:
If not checked, only non-ambiguous peaks are exported

Skip peaks marked for deletion:
will not keep the peaks marked for deletion (those peaks have the skull icon activated in the nmrview analysis panel)



Written by Pascal Mercier
pascalmercier@shaw.ca
University of Alberta, Edmonton, Canada

}

}








proc export_to_cyana {} {

if {[winfo exist .nmrviewtocyana]} {destroy .nmrviewtocyana}

toplevel .nmrviewtocyana
wm withdraw .nmrviewtocyana
wm title .nmrviewtocyana "NMRView to Cyana 2 Utility"

set top .nmrviewtocyana

    frame $top.fra79 \
        -borderwidth 2 -relief flat  -bg white
    set site_3_0 $top.fra79
    frame $site_3_0.cpd94 \
        -borderwidth 2 -height 75 -relief groove -width 125 
    set site_4_0 $site_3_0.cpd94

    label $site_4_0.lab67 \
        -font {Helvetica -12} -text "Peak list"

    menubutton $site_4_0.men68 \
        -font {Helvetica -12 bold} -menu "$site_4_0.men68.m" -padx 4 -pady 3 \
        -text {Choose} -width 8 -borderwidth 2 -relief raised 

 	entry $site_4_0.01 \
    	-font {Helvetica -12 bold} -background #ffffda -width 10 -state disabled
     	
    menu $site_4_0.men68.m \
        -tearoff 0 
   
    pack $site_4_0.lab67 \
        -in $site_4_0 -anchor center -expand 0 -fill none -ipadx 10 \
        -side left
    pack $site_4_0.01 \
        -in $site_4_0 -anchor center -expand 1 -fill x -padx 2 -pady 2 \
        -side left 
             
    pack $site_4_0.men68 \
        -in $site_4_0 -anchor center -expand 0 -fill none -side left 

pack $site_4_0 -in $site_3_0 -anchor center -expand 0 -fill x -side top 
pack $top.fra79 -in $top -anchor center -expand 0 -fill x -side top 


frame $top.fra86 \
        -borderwidth 0 -height 190 -relief flat -width 425  -bg white

set site_3_0 $top.fra86
    
frame $site_3_0.cpd90 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd90
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Need 13C or 15N shift folding} 
    	
    checkbutton $site_4_0.che104 -variable needfolding
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd90 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 
   	

frame $site_3_0.cpd91 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd91
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Keep only identified peaks} 
    	
    checkbutton $site_4_0.che104 -variable keeponlyid
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd91 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 


frame $site_3_0.cpd92 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd92
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Include peaks with ambiguous assignments} 
    	
    checkbutton $site_4_0.che104 -variable include_ambiguous
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd92 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 

frame $site_3_0.cpd97 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd97
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Wipe out ambiguous assignments} 
    	
    checkbutton $site_4_0.che104 -variable wipeoutambiguous
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd97 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 
        
        

frame $site_3_0.cpd93 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd93
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Skip peaks marked for deletion} 
    	
    checkbutton $site_4_0.che104 -variable skip_markedfordeletion
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd93 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 



frame $site_3_0.cpd94 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd94
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Only output short range} 
    	
    checkbutton $site_4_0.che104 -variable onlyoutputshortrange
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd94 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 
        
pack $top.fra86 \
    	-in $top -anchor center -expand 0 -fill x -side top   




frame $site_3_0.cpd95 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd95
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Only output long range} 
    	
    checkbutton $site_4_0.che104 -variable onlyoutputlongrange
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd95 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 
        
pack $top.fra86 \
    	-in $top -anchor center -expand 0 -fill x -side top   




frame $site_3_0.cpd96 \
        -borderwidth 2 -relief groove -width 149 
    set site_4_0 $site_3_0.cpd96
    label $site_4_0.lab103 \
        -font {Helvetica -12} -text {Wipe out long range assignments} 
    	
    checkbutton $site_4_0.che104 -variable wipeoutlongrange
    
    pack $site_4_0.lab103 \
        -in $site_4_0 -anchor w -expand 0 -fill none -side left 
    pack $site_4_0.che104 \
        -in $site_4_0 -anchor w -expand 0 -fill none -padx 10 -side top 
    
    pack $site_3_0.cpd96 \
        -in $site_3_0 -anchor center -expand 0 -fill x -side top 
        
pack $top.fra86 \
    	-in $top -anchor center -expand 0 -fill x -side top   



frame $top.fra87 \
        -borderwidth 2 -height 190 -relief flat -width 425  -bg white

set site_3_0 $top.fra87

button $site_3_0.b1 -text "Convert" -width 15 -command { \
		set choosenpeaklist [.nmrviewtocyana.fra79.cpd94.01 get]
		#make sure that the peak list exists
		if {[lsearch [nv_peak list] $choosenpeaklist]<0} {
			bell
			puts "the peak list $choosenpeaklist does not exist"
			return
		}

	set toexec "nmrviewtocyana_convert { $choosenpeaklist $needfolding $keeponlyid $include_ambiguous $wipeoutambiguous $skip_markedfordeletion $onlyoutputshortrange $onlyoutputlongrange $wipeoutlongrange}"
	eval $toexec
}

button $site_3_0.b2 -text "Close"   -width 15 -command {destroy .nmrviewtocyana}
button $site_3_0.b3 -text "?"   -width 3 -command {export_to_cyana_help}

pack  $site_3_0.b1 -in  $site_3_0 -side left -expand 1 -fill x
pack  $site_3_0.b2 -in  $site_3_0 -side left -expand 1 -fill x
pack  $site_3_0.b3 -in  $site_3_0 -side right 

pack $top.fra87 \
    	-in $top -anchor center -expand 0 -fill x -side top   





set temp .nmrviewtocyana.fra79.cpd94.men68.m
	   .nmrviewtocyana.fra79.cpd94.01  configure -textvariable .nmrviewtocyana.fra79.cpd94.men68.m(list)
	   
$temp conf -postcommand [list NvFillPeakMenu $temp .nmrviewtocyana.fra79.cpd94.men68.m(list) {} 0]



wm deiconify $top
$top conf  -bg white

}
