############################################################################################################
#
# Noe intensities calibration of 15N-edited-3D-Noesy
#  based on Stephane M. Gagne script
#
#
# Original Author:Vincent
#
#  last modifications: Oct 31, 2001
#
# modified by Pascal Mercier.  Includes fixes:
# v1.5
# looks at symmetric DNN peaks and keeps the lowest/highest limit

# v1.4
# the program now correctly evaluate the noise level (was 5 times to large)
# the script now correctly handles stereospecific assignement of Me groups (were automatically treated as stereospecically assigned in the orignal version)
# the program can now handle non-Me stereospecific assignements (were automatically treated as non stereospecically assigned in the orignal version)
# the program now correctly determines the number of protons attached to a Carbon
# the script can be sourced at startup without brining an interface automatically
############################################################################################################




################
#
#  Procedures
#
################



proc calibrationN15_smg {} {

if {[winfo exists .nnoe_cal]} {
        wm deiconify .nnoe_cal
	} else {
	makecalibN15interface
   }
}





proc bds_show_pipp_wind { } {

        if {![winfo exists .pipp_convert]} {
                set script_name ~/nmrview/scripts/noe_cal/pipp_convert.tcl
                source $script_name
        }
        
        wm deiconify .pipp_convert
        raise .pipp_convert
        
}



###########################################
#
# procedure to activate the entry boxes
#   according to the checkbutton values
#
###########################################

proc activate_entry { name value } {
	
	if {$value=="n"} {
		set forecolor grey
		set act_state disabled
	} elseif {$value=="y"} {
		set forecolor black
		set act_state normal		
	}
		
	if {$name=="sum"} {
			.nnoe_cal.label#12 configure -foreground $forecolor
			.nnoe_cal.label#13 configure -foreground $forecolor
			.nnoe_cal.label#14 configure -foreground $forecolor
			.nnoe_cal.corr_CH3_ent configure -state $act_state -foreground $forecolor 
			.nnoe_cal.corr_CH2_ent configure -state $act_state -foreground $forecolor
			.nnoe_cal.corr_MET_ent configure -state $act_state -foreground $forecolor	
	} elseif {$name=="phe"} {
			.nnoe_cal.label#15 configure -foreground $forecolor
			.nnoe_cal.phe_corr_ent configure -state $act_state -foreground $forecolor 	
	} elseif {$name=="log"} {
			.nnoe_cal.logfile_ent configure -state $act_state -foreground $forecolor 
	} elseif {$name=="xplor"} {
			.nnoe_cal.xplorfile_ent configure -state $act_state -foreground $forecolor 	
	} elseif {$name=="rstrnt"} {
			.nnoe_cal.rstrntfile_ent configure -state $act_state -foreground $forecolor 	
	}
	
}


###############################################################
#
# procedure to determinate the noise level in the spectrum
#
#    might not work with all spectra !
#    be cautious !!
#
###############################################################

proc bds_noise_lvl {} {
	
	global noise_value
	global noise_var
	global NOISE

	if {$noise_value=="1"} {
		set initial_value [nv_win lvl]
		set noise_tmp [NvSpectrumSetLvl]
		nv_win lvl $initial_value
		set noise_var [expr $noise_tmp/5.0]
	}
	
	set NOISE $noise_var
	.nnoe_cal.noise_ent configure -textvariable noise_var

}



##################################################
#
# procedure to calibrate the noe intensities
#
##################################################


proc bds_nnoe_calib {} {


## declare global variables

	global curr_list
	global nb_of_residue
	global xpklist
	global Nv_Nresidues
	global NOISE
	global r_up_Na_var
	global r_low_Na_var
	global r_up_aN_var
	global r_low_aN_var
	global nq_up_var
	global nq_low_var
	global eCAL_var
	global eNOE_var
	global dmin_var
	global dmax_var
	global dmaxCH3_var
	global nq_corr_value
	global up_fix_value
	global low_fix_value
	global sum_corr_value
	global corr_CH3_var
	global corr_CH2_var
	global corr_MET_var
	global phe_corr_value
	global logfile_value
	global xplorfile_value
	global logfile_var
	global xplorfile_var
	global rstrntfile_var
	global rstrntfile_value


##  create and set original values for some variables  ##

	set dNa_max 0
	set dNa_min 1000000
	set dNa(0) 0
	set daN(0) 0
	set up_cal(0) 0
	set low_cal(0) 0
	set nq_cal(0) 0
	set nq_cal_nb(0) 0


##  reset the arrays  ##

	unset dNa
	unset daN
	unset up_cal
	unset low_cal
	unset nq_cal
	unset nq_cal_nb


## read values from interface

	set low_min $dmin_var
	set up_max $dmax_var
	set up_max_CH3 $dmaxCH3_var
	set ru1 $r_up_Na_var
	set rl1 $r_low_Na_var
	set ru2 $r_up_aN_var
	set rl2 $r_low_aN_var
	set nq_ru $nq_up_var
	set nq_rl $nq_low_var
	set Ecal $eCAL_var
	set Enoe $eNOE_var
	set nq_state $nq_corr_value
	set up_fixed $up_fix_value
	set low_fixed $low_fix_value
	set SUM $sum_corr_value
	set CH3_corr $corr_CH3_var
	set CH2_corr $corr_CH2_var
	set MET_corr $corr_MET_var
	set PHE_pseudo $phe_corr_value
	set savelog $logfile_value
	set logfilename $logfile_var
	set saverstrnt $rstrntfile_value
	set rstrntfilename $rstrntfile_var
	set savexplor $xplorfile_value
	set xplorfilename $xplorfile_var


## determine noise level in the spectrum

	bds_noise_lvl


## set peak list name 

	set xpklist $curr_list
	if {$xpklist == ""} {
#		puts "you have to load a peak list in peaks window!"
		tk_messageBox -title "Error: Peak List?" -icon error \
			-parent .nnoe_cal \
			-type ok \
			-message "no peak list loaded in peaks windows"
	}

	
## read number of residues in sequence

	set nb_of_residue $Nv_Nresidues
	if {$nb_of_residue == "0" } {
#		puts "you have to load the sequence file"
		tk_messageBox -title "Error: Protein Sequence?" -icon error \
			-parent .nnoe_cal \
			-type ok \
			-message "no protein sequence file loaded"
	}


## comment out next line for testing!
#set nb_of_residue 73 


## read peak list data in arrays and find dNa_max and dNa_min values  ###

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
	#puts "doing residue $i"
	
		set result [nv_peak match $xpklist $i.HN $i.N]
		set nb_of_peak_per_residue [llength $result]
		set res_tmp [nv_residue $i toto]
		set res_type $toto(name)
		for {set j 0} {$j <= [expr $nb_of_peak_per_residue - 1]} {incr j} {
			set peak_nb [lindex $result $j]
			
			set proton_label1 [nv_peak element HN.L [lindex $result $j]]	
			set dot_place1 [string first "." $proton_label1]
			set aa_nb1 [string range $proton_label1 0 [expr $dot_place1 - 1]]
			set aa_label1 [string range $proton_label1 [expr $dot_place1 +1] end]
			set aa_tmp [nv_residue $aa_nb1 toto]
			set aa_type1 $toto(name)
			set proton_label [nv_peak element H.L [lindex $result $j]]
			set dot_place [string first "." $proton_label]
			set aa_nb [string range $proton_label 0 [expr $dot_place - 1]]
			set aa_label [string range $proton_label [expr $dot_place +1] end]
			set aa_tmp [nv_residue $aa_nb toto]
			set aa_type $toto(name)
			set peak_int [nv_peak elem int [lindex $result $j]]
			set atom_stereo [nv_atom element stereo $aa_nb.$aa_label]
			set atom_sstereo [nv_atom element sstereo $aa_nb.$aa_label]
			set nb_of_proton [lindex $atom_sstereo 1]
			#puts "peak_nb $peak_nb $aa_nb.$aa_label $nb_of_proton"
			if {$nb_of_proton == ""} {
				set nb_of_proton 1
			}
			
			set new_aa_label $aa_label
			set ambig_state 0
			set other_label 0


		## put a "#" at the end of methyl group labels
		
			if {$nb_of_proton == "3"} {
				#puts "3 protons for $aa_nb.$aa_label"
				set new_aa_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]*
				#now check the ambiguity code.  If it's not unique find the proper label for the partner
				# Ala and Met have only one Me group, so there is no need to treat them
				# Leu and Val have Me groups attached to the same carbon, and Ile to two different carbons
				if {$atom_stereo!=1} {
					if {$aa_type=="VAL"} {
						set ambig_state or;
						if {$new_aa_label=="HG1*"} {set other_label "HG2*"}
						if {$new_aa_label=="HG2*"} {set other_label "HG1*"}
					}
					if {$aa_type=="LEU"} {
						set ambig_state or;
						if {$new_aa_label=="HD1*"} {set other_label "HD2*"}
						if {$new_aa_label=="HD2*"} {set other_label "HD1*"}
					}
				
					if {$aa_type=="ILE"} {
						set ambig_state or;
						if {$new_aa_label=="HD1*"} {set other_label "HG2*"}
						if {$new_aa_label=="HG2*"} {set other_label "HD1*"}
					}
				}
			}
			
		
			
		## put a "*" at the end of CH2 group if second proton chemical shift not found!
		#   otherwise put an "or" as ambig_state
			
			if {$nb_of_proton == "2"} {
				set label_length [string length $aa_label]
				set last_charac [string range $aa_label [expr $label_length -1] end]
				if { $last_charac == "1"} {
					set other_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]2
					} else {
					set other_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]1
				}
#				set ambig_state or
				set other_label_ppm [nv_atom elem ppm $aa_nb.$other_label]
				if { $other_label_ppm > 1000 | $other_label_ppm < -1000 } {
					set new_aa_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]*
					set ambig_state 0
					set other_label 0
					} else {
					set ambig_state or
				}
			}	
			
		## note: when trying to insert peak_nb in peak($i,$j) array there's some problem
		## seems that the value end with a return and it's impossible to add some other
		## value at the end after peak_nb ! then I had to use 'regexp' to extract the number

			regexp {[0-9]*} $peak_nb peak_nb2


		## write array_list with ambiguous state and corrected label (after replacing by a *)
		
			set peak($i,$j) "$peak_nb2 $aa_type1 $aa_nb1 $aa_label1 $aa_type $aa_nb $aa_label \
									$peak_int $atom_stereo \
									$nb_of_proton $new_aa_label $ambig_state $other_label"
			
			
		##  find dNa or daN for each residue
		#   special case: GLY always calibrated using dNa_max and dNa_min  ###
		
			if {$res_type != "GLY"} {
				if {$aa_nb == $i & $aa_label == "HA"} {
					set dNa($i) $peak_int
					if {$dNa($i) > $dNa_max} {
						set dNa_max $dNa($i)
					}
					if {$dNa($i) < $dNa_min} {
						set dNa_min $dNa($i)
					}
				} elseif {$aa_nb == [expr $i - 1] & $aa_label == "HA"} {
					set daN($i) $peak_int
				}
			}
		}
	}


	##  calculating lower and upper calibration for each residue  ###

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		if {[info exists dNa($i)] == "1"} {
			set up_cal($i) [expr $ru1*pow(($dNa($i)*(1+$Ecal)+$NOISE),0.16666667)]
			set low_cal($i) [expr $rl1*pow(($dNa($i)*(1-$Ecal)-$NOISE),0.16666667)]
			set method($i) dNa		
		} elseif {[info exists daN($i)] == "1"} {
			set up_cal($i) [expr $ru2*pow(($daN($i)*(1+$Ecal)+$NOISE),0.16666667)]
			set low_cal($i) [expr $rl2*pow(($daN($i)*(1-$Ecal)-$NOISE),0.16666667)]
			set method($i) daN		
		} else {
			set up_cal($i) [expr $ru1*pow(($dNa_max*(1+$Ecal)+$NOISE),0.16666667)]
			set low_cal($i) [expr $rl1*pow(($dNa_min*(1-$Ecal)-$NOISE),0.16666667)]
			set method($i) dNa_max		
		}
	}


	## start logfile

	if {$savelog == "y"} {
		set log_fileid [open $logfilename "w+"]
	} else {
		set log_fileid stdout
	}


	## write headers of logfile

	puts $log_fileid "log file created by nnoe_calib.tcl (on [exec date])"
	puts $log_fileid "\npeak list used: $xpklist"
	puts $log_fileid "Noise level: $NOISE"
	puts $log_fileid "dNa_max: $dNa_max"
	puts $log_fileid "dNa_min: $dNa_min"
	
#	puts $log_fileid "\npeak# aa_type aa# H_type Int. stereo proton# newH_type \
#								corr_Int max_dist corr_max_dist min_dist final_max final_min"  


	for {set i 1} {$i <= $nb_of_residue} {incr i} {

		if {[info exists peak($i,0)] == "1"} {
			#puts $log_fileid "[format "\nresidue %s (...; %s.HN,%s.N)  calibration used: %s  up_cal: %3.5f \
								low_cal: %3.5f" $i $i $i $method($i) $up_cal($i) $low_cal($i)]"
		}
		set j 0
		while {[info exists peak($i,$j)] == "1"} {
			set noe_int_tmp [lindex $peak($i,$j) 7]		
		
			##  "SUM" correction for methyl - intensity / 3  #
		
			if {$SUM == "y"} {
				if {[string range [lindex $peak($i,$j) 10] end end] =="*"} {
					if {[lindex $peak($i,$j) 4] == "ALA" & [lindex $peak($i,$j) 10] == "HB*" \
						| [lindex $peak($i,$j) 4] == "MET" & [lindex $peak($i,$j) 10] == "HE*" \
						| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HG2*" \
						| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HD1*" \
						| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD1*" \
						| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD2*" \
						| [lindex $peak($i,$j) 4] == "THR" & [lindex $peak($i,$j) 10] == "HG2*" \
						| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG1*" \
						| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG2*" } {
						set noe_int_tmp [expr $noe_int_tmp / 3]				
					} else {
						set noe_int_tmp [expr $noe_int_tmp / 2]
					}
				}
			}
	
	
			##  set temporary lower restraint after intensity SUM correction  #
		
			set low_tmp [expr $low_cal($i)/pow($noe_int_tmp*(1+$Enoe)+$NOISE,0.16666667)]
			if {$low_tmp < $low_min} {
				set low_tmp $low_min
			}
		
		
			##  set temporary upper restraint after intensity SUM correction  #
		
			if {[expr $noe_int_tmp*(1-$Enoe)-$NOISE] > 0} {
				set up_tmp [expr $up_cal($i)/pow($noe_int_tmp*(1-$Enoe)-$NOISE,0.16666667)]
				if {$up_tmp > $up_max} {
					set up_tmp $up_max
				}
			} else {
				set up_tmp $up_max
			}
		
			set up_orig $up_tmp
		
		
			##  additional correction for methyl groups: CH3_corr (and MET_corr)  #

			if {$SUM == "y"} {
			if {[string range [lindex $peak($i,$j) 10] end end] =="*"} {	
				if {[lindex $peak($i,$j) 4] == "ALA" & [lindex $peak($i,$j) 10] == "HB*" \
					| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HG2*" \
					| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HD1*" \
					| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD1*" \
					| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD2*" \
					| [lindex $peak($i,$j) 4] == "THR" & [lindex $peak($i,$j) 10] == "HG2*" \
					| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG1*" \
					| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG2*" } {
					set up_tmp [expr $up_tmp + $CH3_corr]
					if {$up_tmp > $up_max_CH3} {
						set up_tmp $up_max_CH3
					}
				} elseif {[lindex $peak($i,$j) 4] == "MET" & [lindex $peak($i,$j) 10] == "HE1" } {
					set up_tmp [expr $up_tmp + $CH3_corr + $MET_corr]
					if {$up_tmp > $up_max_CH3} {
						set up_tmp $up_max_CH3
					}
				} else {
					set up_tmp [expr $up_tmp + $CH2_corr]
					if {$up_tmp > $up_max} {
						set up_tmp $up_max
					}
				}
			}	
			}	
			###  correction for PHE pseudo atoms  ###
		
#			if {$PHE_pseudo == "y"} {
#				if {[lindex $peak($i,$j) 4] == "PHE" & [string range [lindex $peak($i,$j) 10] 0 1] == "HD" } {
#					set up_tmp [expr $up_tmp + $PHE_ps_corr]
#				}
#			}
		
		
			##  correction if fixed lower and upper restraints  #
		
			if {$low_fixed =="y"} {
				set low_tmp $low_min
			}
			if {$up_fixed =="y"} {
				set up_tmp $up_max
				if {[lindex $peak($i,$j) 4] == "ALA" & [lindex $peak($i,$j) 10] == "HB*" \
					| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HG2*" \
					| [lindex $peak($i,$j) 4] == "ILE" & [lindex $peak($i,$j) 10] == "HD1*" \
					| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD1*" \
					| [lindex $peak($i,$j) 4] == "LEU" & [lindex $peak($i,$j) 10] == "HD2*" \
					| [lindex $peak($i,$j) 4] == "THR" & [lindex $peak($i,$j) 10] == "HG2*" \
					| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG1*" \
					| [lindex $peak($i,$j) 4] == "VAL" & [lindex $peak($i,$j) 10] == "HG2*" } {
						set up_tmp $up_max_CH3
				}				
			}

				set peak_tmp [format "%3.4f %3.4f %3.4f %3.4f" $noe_int_tmp $up_orig $up_tmp $low_tmp]
				set peak($i,$j) "$peak($i,$j) $peak_tmp"

			incr j
		}


		## special labeling of CH2 and CH3 groups
		#    if ambig_state = "or", keeps the extrema for the restraint for both protons on the same group

		set k 0
		while {[info exists peak($i,$k)] == "1"} {
			set final_max [lindex $peak($i,$k) 15]
			set final_min [lindex $peak($i,$k) 16]
			set new_label2 [lindex $peak($i,$k) 10]
			if {[lindex $peak($i,$k) 11] == "or"} {
				set other_label2 [lindex $peak($i,$k) 12]

				set other_label_found 0
				set m 0
				while {[info exists peak($i,$m)] == "1"} {
		
					if {[lindex $peak($i,$m) 10] == $other_label2 & [lindex $peak($i,$m) 5] == [lindex $peak($i,$k) 5]} {
						set other_label_found 1
						if {[lindex $peak($i,$m) 15] > $final_max } {
							set final_max [lindex $peak($i,$m) 15]
						}
						if {[lindex $peak($i,$m) 16] < $final_min } {
							set final_min [lindex $peak($i,$m) 16]
						}	
					}
					
				incr m
				}
				#if {$other_label_found == "0"} 
				# set new_label3 [string range $new_label2 0 [expr [string length $new_label2] - 2] ]*
				# set peak($i,$k) [lreplace $peak($i,$k) 10 10 $new_label3]
				# set up_tmp2 [expr $final_max + $CH2_corr]
				#if {$up_tmp2 > $up_max} 
					#set final_max $up_max
					# else
					#set final_max $up_tmp2
				
				
			} 
			
			
			
						
				
			set peak_tmp [format "%3.4f %3.4f" $final_max $final_min]
			
			set peak($i,$k) "$peak($i,$k) $peak_tmp"
			#puts $log_fileid $peak($i,$k)
									
		incr k
		}


	}
	
	
	# another fix by Pascal since v.1.5
	# the program also needs to look at symmetric DNNs
	# could not put this code in a loop above
	
	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		if {[info exists peak($i,0)] == "1"} {
			puts $log_fileid "[format "\nresidue %s (...; %s.HN,%s.N)  calibration used: %s  up_cal: %3.5f \
								low_cal: %3.5f" $i $i $i $method($i) $up_cal($i) $low_cal($i)]"
			set k 0
			while {[info exists peak($i,$k)] == "1"} {
				if {[lindex $peak($i,$k) 6] == "HN"} {
					set final_max [lindex $peak($i,$k) 15]
					set final_min [lindex $peak($i,$k) 16]
					# need to find the symmetric DNN, if any
					set residsym [lindex $peak($i,$k) 5]
					puts "sym peak [lindex $peak($i,$k) 2] [lindex $peak($i,$k) 5] $i"
					set w 0
					while {[info exists peak($residsym,$w)] == "1"} {
						if {[lindex $peak($residsym,$w) 5] == $i && [lindex $peak($residsym,$w) 6]=="HN"} {
							# found the sym dnn peak
							# now check which one has the largest calibration
							#puts "found sym peak at $w final_max is $final_max final min is $final_min"
							
							#puts "with the new peak the final_max would be [lindex $peak($residsym,$w) 15] and the final_min [lindex $peak($residsym,$w) 16]"
							
							if {[lindex $peak($residsym,$w) 15] > $final_max } {
								set final_max [lindex $peak($residsym,$w) 15]
								puts "new final max is $final_max"
							}
							if {[lindex $peak($residsym,$w) 16] < $final_min } {
								set final_min [lindex $peak($residsym,$w) 16]
								puts "new final max is $final_min"
							}	
							
							set peak($i,$k) [concat [lrange [set peak($i,$k)] 0 16] [format "%3.4f %3.4f" $final_max $final_min]]
							
								
						}
						incr w
					}
				}
				puts $log_fileid $peak($i,$k)
				incr k
			}
			
		}
	}
	
	if {$savelog == "y"} {
		close $log_fileid
	} 



	###  writing Xplor table  
	
	
	if {$savexplor == "y"} {
		set xplor_fileid [open $xplorfilename "w+"] 
	} else {
		set xplor_fileid stdout
	}

	puts $xplor_fileid "! Xplor table file created by nnoe_calib.tcl (on [exec date])"
	puts $xplor_fileid "! peak list used: $xpklist"

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		
	if {[info exists peak($i,0)] == "1"} {
			puts $xplor_fileid "\n! residue $i (i.e.  ...;$i.HN,$i.N)"
		}
		set j 0
		while {[info exists peak($i,$j)] == "1"} {
			set resid1_nb [lindex $peak($i,$j) 5]
			set resid1_lbl [lindex $peak($i,$j) 10]
			set resid2_nb [lindex $peak($i,$j) 2]
			set resid2_lbl [lindex $peak($i,$j) 3]
			set max_dist [lindex $peak($i,$j) 17]
			set min_dist [lindex $peak($i,$j) 18]
			
			set ecart [expr ($max_dist-$min_dist)*0.5]
			set avg [expr $min_dist+$ecart]
			
			#set delta_min [expr [lindex $peak($i,$j) 14] - [lindex $peak($i,$j) 18]]
			#set delta_plus [expr [lindex $peak($i,$j) 17] - [lindex $peak($i,$j) 14]]
			if {$resid1_nb == $resid2_nb & $resid1_lbl == $resid2_lbl } {
			} else {
				if {[lindex $peak($i,$j) 11] == "or"} {
					set other_label1 [lindex $peak($i,$j) 12]
					puts $xplor_fileid "[format "assign ((resid %s and name %s) or (resid %s and name %s))\
										 (resid %s and name %s)\
											\t %3.3f %3.3f %3.3f" \
								$resid1_nb $resid1_lbl $resid1_nb $other_label1 $resid2_nb $resid2_lbl \
								$avg $ecart $ecart]"
				
				} else {
					puts $xplor_fileid "[format "assign (resid %s and name %s) (resid %s and name %s)\
						\t %3.3f %3.3f %3.3f" $resid1_nb $resid1_lbl $resid2_nb $resid2_lbl $avg $ecart $ecart]"
				}
			}
		incr j
		}
	}

	if {$savexplor == "y"} {
		close $xplor_fileid
	}


	if {$nq_state == "y"} {
		bds_nq_calib
	}

}


################################################
#
#  Procedure to calibrate N & Q sidechains
#
################################################


proc bds_nq_calib {} {

## declare global variables

	global curr_list
	global nb_of_residue
	global xpklist
	global Nv_Nresidues
	global NOISE
	global r_up_Na_var
	global r_low_Na_var
	global r_up_aN_var
	global r_low_aN_var
	global nq_up_var
	global nq_low_var
	global eCAL_var
	global eNOE_var
	global dmin_var
	global dmax_var
	global dmaxCH3_var
	global up_fix_value
	global low_fix_value
	global sum_corr_value
	global corr_CH3_var
	global corr_CH2_var
	global corr_MET_var
	global phe_corr_value
	global logfile_value
	global xplorfile_value
	global logfile_var
	global xplorfile_var
	global rstrntfile_var
	global rstrntfile_value


## read values from interface

	set low_min $dmin_var
	set up_max $dmax_var
	set up_max_CH3 $dmaxCH3_var
	set nq_ru $nq_up_var
	set nq_rl $nq_low_var
	set Ecal $eCAL_var
	set Enoe $eNOE_var
	set up_fixed $up_fix_value
	set low_fixed $low_fix_value
	set SUM $sum_corr_value
	set CH3_corr $corr_CH3_var
	set CH2_corr $corr_CH2_var
	set MET_corr $corr_MET_var
	set PHE_pseudo $phe_corr_value
	set savelog $logfile_value
	set logfilename $logfile_var
	set savexplor $xplorfile_value
	set xplorfilename $xplorfile_var
	set saverstrnt $rstrntfile_value
	set rstrntfilename $rstrntfile_var



	## read peak list data in arrays and find calibration values  ###

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		
		set nq_cal($i) 0
		set nq_cal_nb($i) 0
		
		set result [nv_peak match $xpklist $i.HD2* $i.ND2]
		append result " [nv_peak match $xpklist $i.HE2* $i.NE2]"
		set nb_of_peak_per_residue [llength $result]
		set res_tmp [nv_residue $i toto]
		set res_type $toto(name)
		for {set j 0} {$j <= [expr $nb_of_peak_per_residue - 1]} {incr j} {
			set peak_nb [lindex $result $j]
			set proton_label1 [nv_peak element HN.L [lindex $result $j]]
			set dot_place1 [string first "." $proton_label1]
			set aa_nb1 [string range $proton_label1 0 [expr $dot_place1 - 1]]
			set aa_label1 [string range $proton_label1 [expr $dot_place1 +1] end]
			set aa_tmp [nv_residue $aa_nb1 toto]
			set aa_type1 $toto(name)
			set proton_label [nv_peak element 1H.L [lindex $result $j]]
			set dot_place [string first "." $proton_label]
			set aa_nb [string range $proton_label 0 [expr $dot_place - 1]]
			set aa_label [string range $proton_label [expr $dot_place +1] end]
			set aa_tmp [nv_residue $aa_nb toto]
			set aa_type $toto(name)
			set peak_int [nv_peak elem int [lindex $result $j]]
			set atom_stereo [nv_atom element stereo $aa_nb.$aa_label]
			set atom_sstereo [nv_atom element sstereo $aa_nb.$aa_label]
			set nb_of_proton [lindex $atom_sstereo $atom_stereo]
			
			if {$nb_of_proton == ""} {
				set nb_of_proton 1
			}
			
			set new_aa_label $aa_label
			set ambig_state 0
			set other_label 0


		## put a "*" at the end of methyl group labels
		
			if {$nb_of_proton == "3"} {
				set new_aa_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]*
			}
		
			
		## put a "*" at the end of CH2 group if second proton chemical shift not found!
			
			if {$nb_of_proton == "2"} {
				set label_length [string length $aa_label]
				set last_charac [string range $aa_label [expr $label_length -1] end]
				if { $last_charac == "1"} {
					set other_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]2
					} else {
					set other_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]1
				}
				set other_label_ppm [nv_atom elem ppm $aa_nb.$other_label]
				if { $other_label_ppm > 1000 | $other_label_ppm < -1000 } {
					set new_aa_label [string range $aa_label 0 [expr [string length $aa_label] - 2] ]*
					set ambig_state 0
					set other_label 0
					} else {
					set ambig_state or
				}
			}	
			
		## note: when trying to insert peak_nb in nq_peak($i,$j) array there's some problem
		## seems that the value end with a return and it's impossible to add some other
		## value at the end after peak_nb ! then I had to use 'regexp' to extract the number

			regexp {[0-9]*} $peak_nb peak_nb2

			set nq_peak($i,$j) "$peak_nb2 $aa_type1 $aa_nb1 $aa_label1 $aa_type $aa_nb $aa_label $peak_int $atom_stereo \
									$nb_of_proton $new_aa_label $ambig_state $other_label"
#			puts $nq_peak($i,$j)
		
	###  N & Q calibration  ###
		
		
#			puts $proton_label1
			set last_charac1 [string range $proton_label1 [expr [string length $proton_label1] -1] end]
			if { $last_charac1 == "1"} {
				set other_label1 [string range $proton_label1 0 [expr [string length $proton_label1] - 2] ]2
				} else {
				set other_label1 [string range $proton_label1 0 [expr [string length $proton_label1] - 2] ]1
			}
		
				if { $other_label1 == "${aa_nb}.${aa_label}" } {
					set nq_cal($i) [expr $nq_cal($i) + $peak_int]
					incr nq_cal_nb($i) 
#					puts "$nq_cal($i) $nq_cal_nb($i)"
				}
		}
	}


	##  calculating lower and upper calibration for each residue  ###

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		if { $nq_cal($i) != "0"} {
			set nq_cal($i) [expr $nq_cal($i) / $nq_cal_nb($i)]
			set up_cal($i) [expr $nq_ru*pow(($nq_cal($i)*(1+$Ecal)+$NOISE),0.16666667)]
			set low_cal($i) [expr $nq_rl*pow(($nq_cal($i)*(1-$Ecal)-$NOISE),0.16666667)]
			set method($i) "N/Q calibration"		
		} else {
			set method($i) "toto"
			set up_cal($i) 0
			set low_cal($i) 0
		}	
	}


	## start logfile

	if {$savelog == "y"} {
		set log_fileid [open $logfilename "a+"]
	} else {
		set log_fileid stdout
	}

#	puts $log_fileid "log file created by nnoe_calib.tcl (on [exec date])"
#	puts $log_fileid "\npeak list used: $xpklist"
#	puts $log_fileid "Noise level: $NOISE"
	
#	puts $log_fileid "\npeak# aa_type aa# H_type Int. stereo proton# newH_type \
#								corr_Int max_dist corr_max_dist min_dist final_max final_min"  


	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		if {[info exists nq_peak($i,0)] == "1"} {
			puts $log_fileid "[format "\nresidue %s   calibration used: %s  up_cal: %3.5f \
									low_cal: %3.5f" $i $method($i) $up_cal($i) $low_cal($i)]"
		}
		
		set j 0
		while {[info exists nq_peak($i,$j)] == "1"} {
			set noe_int_tmp [lindex $nq_peak($i,$j) 7]		
		
			###  "SUM" correction for methyl - intensity / 3  ###

#			if {$SUM == "y"} {
#				set nb_of_proton_temp [lindex $nq_peak($i,$j) 10]
#				set noe_int_tmp [expr $noe_int_tmp / $nb_of_proton_temp ]
#			}


		
			if {$SUM == "y"} {
				if {[string range [lindex $nq_peak($i,$j) 10] end end] =="*"} {
					if {[lindex $nq_peak($i,$j) 4] == "ALA" & [lindex $nq_peak($i,$j) 10] == "HB*" \
						| [lindex $nq_peak($i,$j) 4] == "MET" & [lindex $nq_peak($i,$j) 10] == "HE*" \
						| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
						| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
						| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
						| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD2*" \
						| [lindex $nq_peak($i,$j) 4] == "THR" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
						| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG1*" \
						| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG2*" } {
						set noe_int_tmp [expr $noe_int_tmp / 3]				
					} else {
						set noe_int_tmp [expr $noe_int_tmp / 2]
					}
				}
			}
	
	
			###  set temporary lower restraint after intensity SUM correction  ###
		
			set low_tmp [expr $low_cal($i)/pow($noe_int_tmp*(1+$Enoe)+$NOISE,0.16666667)]
			if {$low_tmp < $low_min} {
				set low_tmp $low_min
			}
		
		
			###  set temporary upper restraint after intensity SUM correction  ###
		
			if {[expr $noe_int_tmp*(1-$Enoe)-$NOISE] > 0} {
				set up_tmp [expr $up_cal($i)/pow($noe_int_tmp*(1-$Enoe)-$NOISE,0.16666667)]
				if {$up_tmp > $up_max} {
					set up_tmp $up_max
				}
			} else {
				set up_tmp $up_max
			}
		
			set up_orig $up_tmp
		
		
			###  corrections for methyl groups: CH3_corr (and MET_corr)  ###

			if {$SUM == "y"} {
			if {[string range [lindex $nq_peak($i,$j) 10] end end] =="*"} {	
				if {[lindex $nq_peak($i,$j) 4] == "ALA" & [lindex $nq_peak($i,$j) 10] == "HB*" \
					| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
					| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
					| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
					| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD2*" \
					| [lindex $nq_peak($i,$j) 4] == "THR" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
					| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG1*" \
					| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG2*" } {
					set up_tmp [expr $up_tmp + $CH3_corr]
					if {$up_tmp > $up_max_CH3} {
						set up_tmp $up_max_CH3
					}
				} elseif {[lindex $nq_peak($i,$j) 4] == "MET" & [lindex $nq_peak($i,$j) 10] == "HE1" } {
					set up_tmp [expr $up_tmp + $CH3_corr + $MET_corr]
					if {$up_tmp > $up_max_CH3} {
						set up_tmp $up_max_CH3
					}
				} else {
					set up_tmp [expr $up_tmp + $CH2_corr]
					if {$up_tmp > $up_max} {
						set up_tmp $up_max
					}
				}
			}	
			}	
			###  correction for PHE pseudo atoms  ###
		
#			if {$PHE_pseudo == "y"} {
#				if {[lindex $nq_peak($i,$j) 1] == "PHE" & [string range [lindex $nq_peak($i,$j) 10] 0 1] == "HD" } {
#					set up_tmp [expr $up_tmp + $PHE_ps_corr]
#				}
#			}
		
		
			###  correction if fixed lower and upper restraints  ###
		
			if {$low_fixed =="y"} {
				set low_tmp $low_min
			}
			if {$up_fixed =="y"} {
				set up_tmp $up_max
				if {[lindex $nq_peak($i,$j) 4] == "ALA" & [lindex $nq_peak($i,$j) 10] == "HB*" \
					| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
					| [lindex $nq_peak($i,$j) 4] == "ILE" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
					| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD1*" \
					| [lindex $nq_peak($i,$j) 4] == "LEU" & [lindex $nq_peak($i,$j) 10] == "HD2*" \
					| [lindex $nq_peak($i,$j) 4] == "THR" & [lindex $nq_peak($i,$j) 10] == "HG2*" \
					| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG1*" \
					| [lindex $nq_peak($i,$j) 4] == "VAL" & [lindex $nq_peak($i,$j) 10] == "HG2*" } {
						set up_tmp $up_max_CH3
				}				
			}

				set peak_tmp [format "%3.4f %3.4f %3.4f %3.4f" $noe_int_tmp $up_orig $up_tmp $low_tmp]
				set nq_peak($i,$j) "$nq_peak($i,$j) $peak_tmp"

			incr j
		}


		### special labeling for CH2 group ###

		set k 0
		while {[info exists nq_peak($i,$k)] == "1"} {
			set final_max [lindex $nq_peak($i,$k) 15]
			set final_min [lindex $nq_peak($i,$k) 16]
			set new_label2 [lindex $nq_peak($i,$k) 10]
			if {[lindex $nq_peak($i,$k) 11] == "or"} {
				set other_label2 [lindex $nq_peak($i,$k) 12]

				set other_label_found 0
				set m 0
				while {[info exists nq_peak($i,$m)] == "1"} {
		
					if {[lindex $nq_peak($i,$m) 10] == $other_label2 & [lindex $nq_peak($i,$m) 5] == [lindex $nq_peak($i,$k) 5]} {
						set other_label_found 1
						if {[lindex $nq_peak($i,$m) 15] > $final_max } {
							set final_max [lindex $nq_peak($i,$m) 15]
						}
						if {[lindex $nq_peak($i,$m) 16] < $final_min } {
							set final_min [lindex $nq_peak($i,$m) 16]
						}	
					}
					
				incr m
				}
				if {$other_label_found == "0"} {
#					set new_label3 [string range $new_label2 0 [expr [string length $new_label2] - 2] ]*
#					set nq_peak($i,$k) [lreplace $nq_peak($i,$k) 10 10 $new_label3]
					set up_tmp2 [expr $final_max + $CH2_corr]
					if {$up_tmp2 > $up_max} {
						set final_max $up_max
						} else {
						set final_max $up_tmp2
					}
				}
			}			
				
			set peak_tmp [format "%3.4f %3.4f" $final_max $final_min]
			set nq_peak($i,$k) "$nq_peak($i,$k) $peak_tmp"
			puts $log_fileid $nq_peak($i,$k)
									
		incr k
		}


	}
	if {$savelog == "y"} {
		close $log_fileid
	} 



	

	###  writing X-PLOR table  

	if {$savexplor == "y"} {
		set xplor_fileid [open $xplorfilename "a+"] 
	} else {
		set xplor_fileid stdout
	}

#	puts $xplor_fileid "! Xplor table file created by nnoe_calib.tcl (on [exec date])"
#	puts $xplor_fileid "! peak list used: $xpklist"

	for {set i 1} {$i <= $nb_of_residue} {incr i} {
		
		if {[info exists nq_peak($i,0)] == "1"} {
			puts $xplor_fileid "\n! residue $i"
		}
		
		set j 0
		while {[info exists nq_peak($i,$j)] == "1"} {
#			puts $nq_peak($i,$j)
			set resid1_nb [lindex $nq_peak($i,$j) 5]
			set resid1_lbl [lindex $nq_peak($i,$j) 10]
			set resid2_nb [lindex $nq_peak($i,$j) 2]
			set resid2_lbl [lindex $nq_peak($i,$j) 3]
			set max_dist [lindex $nq_peak($i,$j) 14]
			set delta_min [expr [lindex $nq_peak($i,$j) 14] - [lindex $nq_peak($i,$j) 18]]
			set delta_plus [expr [lindex $nq_peak($i,$j) 17] - [lindex $nq_peak($i,$j) 14]]
			if {$resid1_nb == $resid2_nb & $resid1_lbl == $resid2_lbl } {
			} else {
				puts $xplor_fileid "[format "assign (resid %s and name %s) (resid %s and name %s)\
					\t %3.3f %3.3f %3.3f" $resid1_nb $resid1_lbl $resid2_nb $resid2_lbl $max_dist $delta_min $delta_plus]"
			}
		incr j
		}
	}

	if {$savexplor == "y"} {
		close $xplor_fileid
	}


}





proc makecalibN15interface {} {

## declare global variables

	global curr_list
	global nb_of_residue
	global xpklist
	global Nv_Nresidues
	global NOISE
	global r_up_Na_var
	global r_low_Na_var
	global r_up_aN_var
	global r_low_aN_var
	global nq_up_var
	global nq_low_var
	global eCAL_var
	global eNOE_var
	global dmin_var
	global dmax_var
	global dmaxCH3_var
	global nq_corr_value
	global up_fix_value
	global low_fix_value
	global sum_corr_value
	global corr_CH3_var
	global corr_CH2_var
	global corr_MET_var
	global phe_corr_value
	global logfile_value
	global xplorfile_value
	global logfile_var
	global xplorfile_var
	global rstrntfile_var
	global rstrntfile_value


##  create and set original values for some variables  ##

	set dNa_max 0
	set dNa_min 1000000
	set dNa(0) 0
	set daN(0) 0
	set up_cal(0) 0
	set low_cal(0) 0
	set nq_cal(0) 0
	set nq_cal_nb(0) 0


##  reset the arrays  ##

	unset dNa
	unset daN
	unset up_cal
	unset low_cal
	unset nq_cal
	unset nq_cal_nb



################################################
#
# creation of the toplevel window "Calib"
#
################################################

toplevel .nnoe_cal
wm withdraw .nnoe_cal
wm title .nnoe_cal {N15 edited 3D noesy Calibration}


################################################
#
# set the initial values for parameters
#
################################################

set base .nnoe_cal

	set dmin_var 1.70
	set dmax_var 6.00
	set dmaxCH3_var 6.00
	set r_up_Na_var 3.05
	set r_low_Na_var 2.70
	set r_up_aN_var 3.60
	set r_low_aN_var 1.70
	set nq_up_var 2.20
	set nq_low_var 1.60
	set eNOE_var 0.050
	set eCAL_var 0.050
	set noise_var 0.05
	set noise_value y
	set nq_corr_value n
	set up_fix_value n
	set low_fix_value y
	set sum_corr_value n
	set corr_CH3_var 0.50 
	set corr_CH2_var 0.50 
	set corr_MET_var 0.50
	set phe_corr_value n
	set phe_corr_var 0.00
	set logfile_value n
	set logfile_var nnoe_calib.log
	set xplorfile_value n
	set xplorfile_var calib.xplor
	set rstrntfile_value n
	set rstrntfile_var calib.rstrnt


################################################
#
# creation of the widgets for the interface
#
################################################
  
	label $base.label#25 -foreground blue -padx 5 -text Distances
	label $base.label#9 -text minimum
	entry $base.dmin_ent -cursor {} -textvariable dmin_var -width 6
	label $base.label#10 -text maximum
	entry $base.dmax_ent -cursor {} -textvariable dmax_var -width 6
	label $base.label#11 -text maximum_CH3
	entry $base.dmaxCH3_ent -cursor {} -textvariable dmaxCH3_var -width 6
	label $base.label#26 -foreground blue -padx 5 -text Restraints
	label $base.label#1 -text {dNa upper}
	entry $base.r_up_Na_ent -cursor {} -textvariable r_up_Na_var -width 6
	label $base.label#2 -text {daN upper}
	entry $base.r_up_aN_ent -cursor {} -textvariable r_up_aN_var -width 6
	label $base.label#3 -text {dNa lower}
	entry $base.r_low_Na_ent -cursor {} -textvariable r_low_Na_var -width 6
	label $base.label#4 -text {daN lower}
	entry $base.r_low_aN_ent -cursor {} -textvariable r_low_aN_var -width 6
	label $base.label#31 -text {N/Q upper}
	entry $base.nq_up_ent -cursor {} -textvariable nq_up_var -width 6
	label $base.label#32 -text {N/Q lower}
	entry $base.nq_low_ent -cursor {} -textvariable nq_low_var -width 6
	label $base.label#27 -foreground blue -padx 5 -text Errors
	label $base.label#6 -text {on NOE int.}
	entry $base.eNOE_ent -cursor {} -textvariable eNOE_var -width 6
	label $base.label#5 -text {on calibration}
	entry $base.eCAL_ent -cursor {} -textvariable eCAL_var -width 6
	label $base.label#29 -foreground blue -padx 5 -text {Noise level}
	label $base.label#30 -text noise
	entry $base.noise_ent -cursor {} -textvariable noise_var -width 6
	checkbutton $base.noise_chk -text {Auto lvl} -variable noise_value
	label $base.label#23 -foreground blue -padx 5 -text Corrections
	checkbutton $base.nq_corr_chk -text {N and Q corr.} -variable nq_corr_value \
			-onvalue y \
			-offvalue n
	checkbutton $base.up_fix_chk -text {up fixed} -variable up_fix_value \
			-onvalue y \
			-offvalue n
	checkbutton $base.low_fix_chk -text {low fixed} -variable low_fix_value \
			-onvalue y \
			-offvalue n
	checkbutton $base.sum_corr_chk -text {SUM corr.} -variable sum_corr_value \
			-command { activate_entry sum $sum_corr_value } \
			-onvalue y \
			-offvalue n
	label $base.label#12 -text {CH3 corr.} -foreground grey
	entry $base.corr_CH3_ent -cursor {} -textvariable corr_CH3_var -state disabled -foreground grey -width 6
	label $base.label#13 -text {CH2 corr.} -foreground grey
	entry $base.corr_CH2_ent -cursor {} -textvariable corr_CH2_var -state disabled -foreground grey -width 6
	label $base.label#14 -text {MET corr.} -foreground grey
	entry $base.corr_MET_ent -cursor {} -textvariable corr_MET_var -state disabled -foreground grey -width 6
	checkbutton $base.phe_corr_chk -text {PHE corr.} -state disabled -variable phe_corr_value \
			-command { activate_entry phe $phe_corr_value } \
			-onvalue y \
			-offvalue n
	label $base.label#15 -text {PHE_ps corr.} -foreground grey
	entry $base.phe_corr_ent -cursor {} -state disabled -foreground grey \
			-textvariable phe_corr_var -width 6
	label $base.label#28 -foreground blue -padx 5 -text {Save options}
	checkbutton $base.logfile_chk -text {Log file} -variable logfile_value \
			-command { activate_entry log $logfile_value } \
			-onvalue y \
			-offvalue n			
	entry $base.logfile_ent -cursor {} -foreground grey -state disabled \
			-textvariable logfile_var -width 11
	checkbutton $base.xplorfile_chk -text {Xplor file} -variable xplorfile_value \
			-command { activate_entry xplor $xplorfile_value } \
			-onvalue y \
			-offvalue n
	entry $base.xplorfile_ent -cursor {} -foreground grey -state disabled \
			-textvariable xplorfile_var -width 11
	checkbutton $base.rstrntfile_chk -text {Restraint file} -state normal -variable rstrntfile_value \
			-command { activate_entry rstrnt $rstrntfile_value } \
			-onvalue y \
			-offvalue n
	entry $base.rstrntfile_ent -cursor {} -foreground grey -state disabled \
			-textvariable rstrntfile_var -width 11
	button $base.calib_but -text Calib -width 6 \
			-command bds_nnoe_calib
	button $base.pipp_but  -text "Pipp fans" -state disabled -width 6 \
			-command bds_show_pipp_wind
	button $base.help_but -text Help -state disabled -width 6 \
			-command bell			
	button $base.close_but  -text Close -width 6 \
			-command {wm withdraw .nnoe_cal}
			


####################################
#
# Grid filling
#
####################################

	grid $base.label#25 -row 2 -column 1  -sticky w
	grid $base.label#9 -row 2 -column 2  -sticky e
	grid $base.dmin_ent -row 2 -column 3  -sticky w
	grid $base.label#10 -row 2 -column 4  -sticky e
	grid $base.dmax_ent -row 2 -column 5  -sticky w
	grid $base.label#11 -row 2 -column 6  -sticky e
	grid $base.dmaxCH3_ent -row 2 -column 7  -sticky w
	grid $base.label#26 -row 4 -column 1  -sticky w
	grid $base.label#1 -row 4 -column 2  -sticky e
	grid $base.r_up_Na_ent -row 4 -column 3  -sticky w
	grid $base.label#2 -row 4 -column 4  -sticky e
	grid $base.r_up_aN_ent -row 4 -column 5  -sticky w
	grid $base.label#3 -row 5 -column 2  -sticky e
	grid $base.r_low_Na_ent -row 5 -column 3  -sticky w
	grid $base.label#4 -row 5 -column 4  -sticky e
	grid $base.r_low_aN_ent -row 5 -column 5  -sticky w
	grid $base.label#31 -row 4 -column 6  -sticky e
	grid $base.nq_up_ent -row 4 -column 7  -sticky w
	grid $base.label#32 -row 5 -column 6  -sticky e
	grid $base.nq_low_ent -row 5 -column 7  -sticky w
	grid $base.label#27 -row 7 -column 1  -sticky w
	grid $base.label#6 -row 7 -column 2  -sticky e
	grid $base.eNOE_ent -row 7 -column 3  -sticky w
	grid $base.label#5 -row 7 -column 4  -sticky e
	grid $base.eCAL_ent -row 7 -column 5  -sticky w
	grid $base.label#29 -row 9 -column 1  -sticky w
	grid $base.label#30 -row 9 -column 2  -sticky e
	grid $base.noise_ent -row 9 -column 3  -sticky w
	grid $base.noise_chk -row 9 -column 4  -sticky w
	grid $base.label#23 -row 11 -column 1  -sticky w
	grid $base.nq_corr_chk -row 11 -column 2  -sticky w
	grid $base.low_fix_chk -row 12 -column 2  -sticky w
	grid $base.up_fix_chk -row 12 -column 3  -sticky w
	grid $base.sum_corr_chk -row 13 -column 2  -sticky w
	grid $base.label#12 -row 13 -column 3  -sticky e
	grid $base.corr_CH3_ent -row 13 -column 4  -sticky w
	grid $base.label#13 -row 13 -column 5  -sticky e
	grid $base.corr_CH2_ent -row 13 -column 6  -sticky w
	grid $base.label#14 -row 13 -column 7  -sticky e
	grid $base.corr_MET_ent -row 13 -column 8  -sticky w
	grid $base.phe_corr_chk -row 14 -column 2  -sticky w
	grid $base.label#15 -row 14 -column 3  -sticky e
	grid $base.phe_corr_ent -row 14 -column 4  -sticky w
	grid $base.label#28 -row 16 -column 1  -sticky w
	grid $base.logfile_chk -row 16 -column 2  -sticky w
	grid $base.logfile_ent -row 16 -column 3  -sticky w
	grid $base.xplorfile_chk -row 17 -column 5  -sticky w
	grid $base.xplorfile_ent -row 17 -column 6  -sticky w
	grid $base.rstrntfile_chk -row 17 -column 2 
	grid $base.rstrntfile_ent -row 17 -column 3  -sticky w
	grid $base.calib_but -row 19 -column 2
	grid $base.pipp_but -row 19 -column 4
	grid $base.help_but -row 19 -column 6
	grid $base.close_but -row 19 -column 8 
	

###################################
#	
# Grid size management
#
###################################

	grid rowconfigure $base 1 -weight 0 -minsize 20
	grid rowconfigure $base 2 -weight 0 -minsize 5
	grid rowconfigure $base 3 -weight 0 -minsize 15
	grid rowconfigure $base 4 -weight 0 -minsize 5
	grid rowconfigure $base 5 -weight 0 -minsize 5
	grid rowconfigure $base 6 -weight 0 -minsize 15
	grid rowconfigure $base 7 -weight 0 -minsize 5
	grid rowconfigure $base 8 -weight 0 -minsize 15
	grid rowconfigure $base 9 -weight 0 -minsize 5
	grid rowconfigure $base 10 -weight 0 -minsize 15
	grid rowconfigure $base 11 -weight 0 -minsize 5
	grid rowconfigure $base 12 -weight 0 -minsize 5
	grid rowconfigure $base 13 -weight 0 -minsize 5
	grid rowconfigure $base 14 -weight 0 -minsize 5
	grid rowconfigure $base 15 -weight 0 -minsize 15
	grid rowconfigure $base 16 -weight 0 -minsize 5
	grid rowconfigure $base 17 -weight 0 -minsize 5
	grid rowconfigure $base 18 -weight 0 -minsize 15
	grid rowconfigure $base 19 -weight 0 -minsize 55
	grid columnconfigure $base 1 -weight 0 -minsize 30
	grid columnconfigure $base 2 -weight 0 -minsize 69
	grid columnconfigure $base 3 -weight 0 -minsize 36
	grid columnconfigure $base 4 -weight 0 -minsize 87
	grid columnconfigure $base 5 -weight 0 -minsize 15
	grid columnconfigure $base 6 -weight 0 -minsize 5
	grid columnconfigure $base 7 -weight 0 -minsize 10
	grid columnconfigure $base 8 -weight 0 -minsize 64
	grid columnconfigure $base 9 -weight 0 -minsize 50


###########################
#
# show Calib window
#
###########################




 wm deiconify .nnoe_cal

}

