# script history: part of Merck distribution
# menu interface modified 8/99 c hastings
# access to pattern checking added 9/99 c hastings
# edited 9/99 c hastings to add access to user prefs

# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved


############################################################################
#modified by Pascal Mercier on July 16, 2001 to include the following changes:
#
#adds a Go button in the toolbar
#adds a Search Peak function which has more features
#adds a Filter function in the edit menu to get rid of peaks according to different criteria
#replaces the command "Search Peaks" with "Search Peaks (label) in the Check menu
#adds the command "Search Peaks (ppm) in the check menu

#patch on the original script from nmrview 5.0.4

#Brian Sykes lab
#CIHR group on protein and structure
#University of Alberta
#Canada
###########################################################################



proc Nv_ShowPeakAnal {args} {
    global Nv_Priv curr_list curr_peak
    set narg [llength $args]
    if {$narg>0} { set nm [lindex $args 0] } else {set nm $Nv_Priv(peakAnal)}
    if {$narg>1} {
        set listName [lindex $args 1]
        set listIdx [lsearch [nv_peak lists] $listName]
        set curr_list ""
        if {$listIdx >= 0} {
            set curr_list $listName
            set curr_peak 0
        }
        
    } else {
        set nm $Nv_Priv(peakAnal)
    }
    if {![winfo exists $nm]} {
        Nv_MkPeakAnal $nm
    }
    NvSetPeakAnal $nm
    wm deiconify $nm
    raise $nm
    update
    NvUpdatePeakAnal $nm
}

proc Nv_MkPeakAnal {nm} {
    
    if {[string range $nm 0 0]!="."} {
        errmsg "Window must start with ."
        return
    }
    
    set varnm [NvVarnm $nm]
    global $varnm Nv_Priv
    
    if {[winfo exists $nm]} {
        raise $nm
        return
    }
    toplevel $nm
    wm title $nm peakPanel
    wm withdraw $nm
    global $varnm Nv_Priv
    upvar #0 $varnm panvar
    
    set panvar(start) 0
    if {[llength [nv_peak lists]] < 1} {
        set panvar(curr_list) ""
        set panvar(curr_peak) -1
    } else {
        set panvar(curr_list) [lindex [nv_peak lists] 0]
        set panvar(curr_peak) 0
    }
    
    frame $nm.mbar -relief raised -bd 2
    frame $nm.fr1 -border 0 -relief flat
    frame $nm.fr2
    frame $nm.fr3
    menubutton $nm.mbar.file -text File -underline 0 \
    -menu $nm.mbar.file.menu
    
    add_tip $nm.mbar.file peakAnal.file "File Menu:\n\n\
    WriteList: Write peak list to an ascii text file.\n\n\
    ReadList: Read a peak list from an ascii text file.\n\n\
    Close: Close Peak Analysis Panel."
    
    
    menubutton $nm.mbar.move -text Move -underline 0 -menu $nm.mbar.move.menu
    
    add_tip $nm.mbar.move peakAnal.move "Move Menu:\n\n First: Jump to first peak in list.\n\n Last: Jump to last peak in list."
    
    menubutton $nm.mbar.edit -text Edit -underline 0 -menu $nm.mbar.edit.menu
    
    add_tip $nm.mbar.edit peakAnal.edit "Edit Menu:\n\n Compress: Compress peak list by permanently removing deleted peaks.\n\n DeleteList: Delete the\
    currently selected peak list.\n\n Reference: Display and edit reference data for this peak list.\n\n Couple: Join peaks that are split by a specified\
    coupling.\n\n Identify: List possible assignments for the current peak.\n\n Get Volumes: Calculate peak volumes for all peaks in the current list.\n\n\
    Get Intensities: Calculate peak intensities for all peaks in the current list.\n\n Plot Volumes vs. Intensites: Show an XY plot of volume vs.\
    intensities for all the peaks in current list."
    
    menubutton $nm.mbar.check -text Check -underline 0 -menu $nm.mbar.check.menu
    
    
    menubutton $nm.mbar.integrate -text Int -underline 0 -menu $nm.mbar.integrate.menu
    
    menubutton $nm.mbar.list -text List -underline 0 -menu $nm.mbar.list.menu
    
    add_tip $nm.mbar.list peakAnal.list "List Menu:\n\n Use this menu to select a peak list to be displayed in this panel."
    
    
    pack $nm.mbar.file $nm.mbar.move $nm.mbar.edit $nm.mbar.check $nm.mbar.integrate $nm.mbar.list -side left -padx 1m -pady 1m
    
    menu $nm.mbar.file.menu -tearoff 0
    $nm.mbar.file.menu add command -label "Write List" -command "NvPeaksWriteList"
    $nm.mbar.file.menu add command -label "Read List" -command "NvGetFile rdpeaks"
    $nm.mbar.file.menu add command -label "Make New List" -command "MkNewlist"
    $nm.mbar.file.menu add command -label "Read All Lists" -command "readallpks"
    $nm.mbar.file.menu add command -label "Write All Lists" -command "writeallpeaks"
    $nm.mbar.file.menu add command -label Close -command "wm withdraw $nm"
    menu $nm.mbar.move.menu -tearoff 0
    $nm.mbar.move.menu add command -label First -command "
    set ${varnm}(curr_peak) 0
    NvUpdatePeakAnal $nm
    "
    $nm.mbar.move.menu add command -label "First Unassigned" -command "gfirstua"
    $nm.mbar.move.menu add command -label "Next Unassigned" -command "gnextua"
    $nm.mbar.move.menu add command -label "First Unlocked" -command "gfirstul"
    $nm.mbar.move.menu add command -label "Next Unlocked" -command "gnextul"
    $nm.mbar.move.menu add command -label Last -command "
    set ${varnm}(curr_peak) 100000
    NvUpdatePeakAnal $nm
    "
    menu $nm.mbar.edit.menu -tearoff 0
    $nm.mbar.edit.menu add command -label Compress -command NvPeakCompress
    $nm.mbar.edit.menu add command -label Degap -command "compact"
    $nm.mbar.edit.menu add command -label "Compress & Degap" -command "compactdel"
    $nm.mbar.edit.menu add command -label DeleteList -command "NvPeakDeleteList \$${varnm}(curr_list)"

    #added by Pascal Mercier
    $nm.mbar.edit.menu add command -label Filter -command "Nv_MkFilter"
    #

    $nm.mbar.edit.menu add command -label Reference -command "Nv_ShowPeakRef $nm"
    $nm.mbar.edit.menu add command -label Couple -command "Nv_ShowPeakCouple $Nv_Priv(peakCouple)"
    $nm.mbar.edit.menu add command -label Cluster -command "nv_peak cluster \$${varnm}(curr_list)"
    $nm.mbar.edit.menu add command -label Fit -command "nv_peak fit \[nv_peak dataset \$${varnm}(curr_list)\] \$${varnm}(curr_list).\$${varnm}(curr_peak)"
    $nm.mbar.edit.menu add command -label Partners -command "NvPeakPartners"
    $nm.mbar.edit.menu add command -label Identify -command "Nv_ShowPeakId $Nv_Priv(peakId)"
    
    menu $nm.mbar.check.menu -tearoff 0
    
    $nm.mbar.check.menu add command -label "Check Pattern" -command "MkCheckPeakPattern .checkPattern"
    $nm.mbar.check.menu add command -label "Check Assignments" -command "MkCheckPeakShifts .checkPeakShifts"
    $nm.mbar.check.menu add command -label "Search Peaks (label)" -command "MkPeakSearch .peakSearch"
    
    #added by Pascal Mercier
    $nm.mbar.check.menu add command -label "Search Peaks (ppm)" -command "searchpeaks"
    #

    menu $nm.mbar.integrate.menu -tearoff 0
    $nm.mbar.integrate.menu add command -label "Associate Dataset" -command "NvDatasetAssoc"
    $nm.mbar.integrate.menu add command -label "Get Volumes" -command "
    Nv_PeakVolumes volume
    NvUpdatePeakAnal $nm
    "
    $nm.mbar.integrate.menu add command -label "Get eVolumes" -command "
    Nv_PeakVolumes evolume
    NvUpdatePeakAnal $nm
    "
    $nm.mbar.integrate.menu add command -label "Get Intensities" -command "
    Nv_PeakIntensities center
    NvUpdatePeakAnal $nm
    "
    $nm.mbar.integrate.menu add command -label "Plot Vol vs. Intensities" -command "NvPeakPlotVolInt"
    
    set menu $nm.mbar.list.menu
    menu $menu -postcommand [list NvFillPeakAnalMenu $nm $menu ${varnm}(curr_list) ]
    
    #    scrollbar $nm.fr1.scroll -command "NvUpdatePeakAnal $nm"
    pack $nm.mbar  -fill x -padx 1m -pady 1m -side top
    button $nm.mbar.prev -bitmap @$Nv_Priv(image,down)  -width 20 -command "NvPeakIncr $varnm -1;NvUpdatePeakAnal $nm"
    add_tip $nm.mbar.prev peakAnal.prev "Click this button to go to the previous peak in list."
    
    button $nm.mbar.next -bitmap @$Nv_Priv(image,up) -width 20 -command "NvPeakIncr $varnm 1;NvUpdatePeakAnal $nm"
    add_tip $nm.mbar.next peakAnal.next "Click this button to go to the next peak in list."
    
    
    button $nm.mbar.delstatus -bitmap @$Nv_Priv(image,pirate) -width 20 -command "NvPeakFlipStatus $nm"
    add_tip $nm.mbar.delstatus peakAnal.delstatus "Click this button to delete or undelete the current peak.
    The button will be red if the status of the peak is set to DELETE."
    

     #added by Pascal Mercier
        button $nm.mbar.gobutton -text "Go" -width 20 -command "NvUpdatePeakAnal $nm;GoPeakPlane $nm"
     # 

    button $nm.mbar.locked -bitmap @$Nv_Priv(image,unlock) -width 20 -command "NvPeakFlipLock $nm"
    add_tip $nm.mbar.locked peakAnal.locked "Click this button to lock or unlock the current peak.
    The button will be red if the status of the peak is set to DELETE."
    
    entry $nm.mbar.peak -width 6  -relief sunken  -textvariable ${varnm}(curr_peak)
    add_tip $nm.mbar.peak peakAnal.peak "The current peak.\n\n  Type in a value and
    hit the Enter key to change it."
    
    bind $nm.mbar.peak <Return> "NvUpdatePeakAnal $nm"
    
   
 #first pack command modified by Pascal to include the new go button

    pack $nm.mbar.peak $nm.mbar.prev $nm.mbar.next $nm.mbar.delstatus $nm.mbar.locked $nm.mbar.gobutton -side left
    

    if {!$Nv_Priv(javaMode)} {
        pack $nm.mbar  -fill x  -side top
        
        pack $nm.fr1  -fill both -side top -expand y
        pack $nm.fr2 -side bottom -fill x
        pack $nm.fr3 -side bottom -fill x
    } else {
        grid $nm.mbar -gridx 0 -gridy 0 -fill x -weightx 1.0 -weighty 0.0
        grid $nm.fr1 -gridx 0 -gridy 1 -fill both -weightx 1.0 -weighty 1.0
        grid $nm.fr2 -gridx 0 -gridy 2 -fill x -weightx 1.0 -weighty 0.0
        grid $nm.fr3 -gridx 0 -gridy 3 -fill x -weightx 1.0 -weighty 0.0
    }
    
    if {!$Nv_Priv(javaMode)} {
        set canvas $nm.fr1.canvas
        canvas $canvas -width 10 -height 10 \
        -yscrollcommand [list $nm.fr1.yscroll set]
        scrollbar $nm.fr1.yscroll -orient vertical \
        -command [list $canvas yview]
        pack $nm.fr1.yscroll -side right -fill y
        pack $canvas -side left -fill both -expand true
        
        set fr $canvas.data
        frame $fr
        $canvas create window 0 0 -anchor nw -window $fr
        
        
    } else {
        set spane $nm.fr1.spane
        #		jscrollpane $spane
        frame $spane
        pack $spane -fill both -expand true
        set fr $nm.fr1.spane.panel
        frame $fr -relief raised -bd 2
        #		$spane jset $fr
        pack $fr -fill both -expand yes
    }
    
    
    
    set  RowNames "Label RName Center Width Bounds J LinkA LinkB Cluster User Error"
    set  RowTypes {.L .R .P .WH .BH .J .a .b .c .U .E}
    set panvar(nRows) [llength $RowNames]
    set nfields 4
    set panvar(nLabels) 1
    set panvar(labelIndex) 0
    for {set row 0 } {$row < [llength $RowNames]} {incr row} {
        set i $row
        set  panvar(Label,$row) [lindex $RowNames $row]
        set  panvar(Type,$row) [lindex $RowTypes $row]
        if {[lindex $RowNames $row]=="Label"} {
            frame $fr.labframe
            label $fr.labframe.lab -text Lab -width 3
            pack $fr.labframe.lab -side left
            label $fr.labframe.labI -textvariable ${varnm}(labelIndex) -width 2
            pack $fr.labframe.labI -side left
            label $fr.labframe.labN -textvariable ${varnm}(nLabels)  -width 2
            pack $fr.labframe.labN -side left
            button $fr.labframe.down -text >  \
            -width 1 -command "NvPeakShiftLabel $nm $fr.$row.0 1"
            button $fr.labframe.up -text <  \
            -width 1 -command "NvPeakShiftLabel $nm $fr.$row.0 -1"
            pack $fr.labframe.up $fr.labframe.down -side left
            if {!$Nv_Priv(javaMode)} {
                grid columnconfigure $fr 0 -weight 100
                grid rowconfigure $fr $i -weight 100
                grid $fr.labframe -in $fr -column 0 -row $i -sticky nsew
            } else {
                grid $fr.labframe -column 0 -row $i -sticky nsew -weightx 1.0
            }
        } else {
            label $fr.lab$row -text [set panvar(Label,$row)] -width 8
            if {!$Nv_Priv(javaMode)} {
                grid columnconfigure $fr 0 -weight 100
                grid rowconfigure $fr $i -weight 100
                grid $fr.lab$row -in $fr -column 0 -row $i -sticky nsew
            } else {
                grid $fr.lab$row -column 0 -row $i -sticky nsew -weightx 1
            }
        }
        for {set field 0 } {$field < $nfields} {incr field} {
            set j [expr {$field+1}]
            entry $fr.entry${row}_$field -width 8  -bd 2 -relief sunken  -textvariable ${varnm}($row,$field)
            if {!$Nv_Priv(javaMode)} {
                grid columnconfigure $fr $j -weight 100
                grid $fr.entry${row}_$field -in $fr -column $j -row $i -sticky nsew
            } else {
                grid $fr.entry${row}_$field -column $j -row $i -sticky nsew -weightx 1
            }
            set panvar($row,$field) ""
            bind $fr.entry${row}_$field <Return> "NvGetPeakAnal $nm"
            if {!$Nv_Priv(javaMode)} {
                bind $fr.entry${row}_$field <KeyPress> "$nm.mbar.peak configure -bg Red"
            }
            if {[lindex $RowNames $row]=="Center"} {
                bind $fr.entry${row}_$field <Down> "nv_peak fold \$${varnm}(curr_list).\$${varnm}(curr_peak).[expr $field+1] down;NvUpdatePeakAnal $nm;nv_database sync"
                bind $fr.entry${row}_$field <Up> "nv_peak fold \$${varnm}(curr_list).\$${varnm}(curr_peak).[expr $field+1] up;NvUpdatePeakAnal $nm;nv_database sync"
            }
        }
    }
    label $nm.fr2.intlab -text Int. -width 4
    label $nm.fr2.vollab -text Vol. -width 4
    entry $nm.fr2.int -width 8  -relief sunken  -textvariable ${varnm}(int)
    add_tip $nm.fr2.int peakAnal.int "The peak intensity (maximum value)."
    entry $nm.fr2.vol -width 8  -relief sunken  -textvariable ${varnm}(vol)
    add_tip $nm.fr2.vol peakAnal.vol "The peak volume (This is not calculated during peak picking)."
    
    label $nm.fr2.currlist -textvariable ${varnm}(curr_list) -relief sunken
    
    pack $nm.fr2.intlab $nm.fr2.int $nm.fr2.vollab $nm.fr2.vol -side left
    pack $nm.fr2.currlist -side right
    
    bind $nm.fr2.int <Return> "NvUpdatePeakAnal $nm"
    bind $nm.fr2.vol <Return> "NvUpdatePeakAnal $nm"
    
    
    label $nm.fr3.commentLabel -text Comment -width 8
    entry $nm.fr3.comment -width 46 -relief sunken -textvariable ${varnm}(comment)
    bind $nm.fr3.comment <Return> "NvGetPeakAnal $nm"
    bind $nm.fr3.comment <KeyPress> "$nm.mbar.peak configure -bg Red"
    
    add_tip $nm.fr3.comment peakAnal.comment "The peak comment."
    pack $nm.fr3.commentLabel $nm.fr3.comment -side left
    bind $nm.fr3.comment <Return> "NvGetPeakAnal $nm"
    if {!$Nv_Priv(javaMode)} {
        
        set child [lindex [grid slaves $fr] 0]
        #	tkwait visibility $child
        update
        set incr [winfo height $child]
        set height [winfo height $fr]
        set width [winfo width $fr]
        $canvas config -scrollregion "0 0 $width $height"
        $canvas config -yscrollincrement $incr
        if {$height > 4 * $incr } {
            set height [expr 4 * $incr]
        }
        $canvas config -width $width -height $height
        wm maxsize $nm 600 700
        wm minsize $nm 180 160
        
    }
}

proc NvPeakIncr {varnm incrVal} {
    upvar #0 $varnm panvar
    
    set iList $panvar(curr_list)
    set iPeak [expr {$panvar(curr_peak)+$incrVal}]
    set index [NvPeakCheckIndex $iList $iPeak $incrVal]
    if {$index >= 0} {
        set panvar(curr_peak) [eval nv_peak idnum $iList.$index]
    }
}

proc NvPeakCheckIndex {iList iPeak incrVal} {
    set idlast [nv_peak idlast $iList]
    if {$iPeak < 0} {return {} }
    while {1} {
        if {$iPeak > $idlast} {
            return -1
        }
        if {![catch "nv_peak elem index $iList.$iPeak" index]} {
            break
        }
        incr iPeak $incrVal
        if {$iPeak == 0} {return -1}
    }
    return $index
}

proc NvPeakShiftLabel {args} {
    global Nv_Priv
    
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(peakAnal)
    }
    if {$narg>1} {
        set labelField [lindex $args 1]
    } else {
        set labelField $nm.fr1.data.0.0
    }
    if {$narg>2} {
        set dir [lindex $args 2]
    } else {
        set dir 1
    }
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    set curr_list $panvar(curr_list)
    set labels [nv_peak label $curr_list]
    set ndim [llength $labels]
    incr panvar(labelIndex) $dir
    if {$panvar(labelIndex) < 0} {
        set panvar(labelIndex) 0
    }
    set nVals [llength $panvar(labelVal,0)]
    if {$panvar(labelIndex) >= $nVals} {
        set panvar(labelIndex) $nVals
    }
    for {set field 0} {$field < $ndim} {incr field} {
        set fldnm [lindex $labels $field]
        set panvar(0,$field) [lindex $panvar(labelVal,$field) $panvar(labelIndex)]
        
    }
    
    
}
proc NvPeakDelete {nm} {
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    nv_peak delete $panvar(curr_list).$panvar(curr_peak)
    $nm.mbar.delstatus config -fg red
    $nm.mbar.delstatus config -highlightcolor red
}
proc NvPeakUnDelete {nm} {
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    nv_peak undelete $panvar(curr_list).$panvar(curr_peak)
    $nm.mbar.delstatus config -fg black
    $nm.mbar.delstatus config -highlightcolor black
}

proc NvPeakFlipStatus {nm} {
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    set status [nv_peak status $panvar(curr_list).$panvar(curr_peak)]
    if {$status>=0} {
        nv_peak delete $panvar(curr_list).$panvar(curr_peak)
        $nm.mbar.delstatus config -fg red
        $nm.mbar.delstatus config -highlightcolor red
    } else {
        nv_peak undelete $panvar(curr_list).$panvar(curr_peak)
        $nm.mbar.delstatus config -fg black
        $nm.mbar.delstatus config -highlightcolor black
    }
    
}

proc NvPeakFlipLock {nm} {
    global Nv_Priv
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    set status [nv_peak elem flag0 $panvar(curr_list).$panvar(curr_peak)]
    if {$status==0} {
        nv_peak elem flag0 $panvar(curr_list).$panvar(curr_peak) 1
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,lock)
    } else {
        nv_peak elem flag0 $panvar(curr_list).$panvar(curr_peak) 0
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,unlock)
    }
    
}

proc ShowPeakAnal args {
    global Nv_Priv
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(peakAnal)
    }
    wm deiconify $nm
    update
}

proc NvSetPeakAnal {args} {
    global Nv_Priv
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(peakAnal)
    }
    set varnm [NvVarnm $nm]
    
    global curr_list curr_peak
    upvar #0 $varnm panvar
    set panvar(curr_list) $curr_list
    set panvar(curr_peak) $curr_peak
    NvUpdatePeakAnal
}



proc NvUpdatePeakAnal args {
    global Nv_Priv
    
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(peakAnal)
    }
    set varnm [NvVarnm $nm]
    
    global curr_list curr_peak
    upvar #0 $varnm panvar
    
    if {$panvar(curr_list)==""} {
        return
    }
    if {$panvar(curr_list)=="NULL"} {
        return
    }
    
    if {[llength [nv_peak lists]] < 1} {
        return
    }
    
    
    if {$panvar(curr_peak)<0} {
        set panvar(curr_peak) 0
    }
    wm title $nm   $panvar(curr_list)
    
    if {$panvar(curr_peak)<0} {set panvar(curr_peak) 0}
    
    set idlast [nv_peak idlast $panvar(curr_list)]
    
    if {$idlast <= 0} {return}
    
    if {$panvar(curr_peak)>$idlast} {set panvar(curr_peak) [expr $idlast]}
    
    set curr_peak $panvar(curr_peak)
    set curr_list $panvar(curr_list)
    
    if {[catch "eval nv_peak status $curr_list.$curr_peak"]} {
        set blankMode 1
    } else {
        set blankMode 0
    }
    
    
    set nrows $panvar(nRows)
    set start $panvar(start)
    
    if {$narg>2} {
        set cmd [lindex $args 1]
        set fstart [lindex $args 2]
        
        switch $cmd {
            scroll {
                incr start $fstart
            }
            moveto {
                set start [expr round($fstart*$nrows)]
            }
        }
    }
    
    set nfields 4
    set labels [nv_peak label $curr_list]
    set ndim [llength $labels]
    if {$blankMode} {set ndim 0}
    
    for {set i 0 } {$i < $panvar(nRows)} {incr i} {
        set lab  $panvar(Label,[expr $i+$start])
        set typ  $panvar(Type,[expr $i+$start])
        for {set field 0 } {$field < $nfields} {incr field} {
            if {$field<$ndim} {
                set fldnm [lindex $labels $field]
                if {$typ==".L"} {
                    set panvar(labelVal,$field) [nv_peak elem $fldnm$typ $panvar(curr_list).$panvar(curr_peak)]
                    set panvar(labelIndex) 0
                    set panvar($i,$field) [lindex $panvar(labelVal,$field) 0]
                } elseif {$typ==".R"} {
                    if {![catch "nv_atom elem rname $panvar([expr {$i-1}],$field)" result]} {
                        set panvar($i,$field) $result
                    } else {
                        set panvar($i,$field) ""
                    }
                } else {
                    set panvar($i,$field) [nv_peak elem $fldnm$typ $panvar(curr_list).$panvar(curr_peak)]
                    
                }
                
            } else  {
                set panvar($i,$field) ""
            }
        }
    }
    if {$blankMode} {
        set panvar(nLabels) 1
        foreach field "int vol comment" {
            set panvar($field) ""
        }
    } else {
        
        # begin edited code to correctly represent the number of labels
        # when the field containing the multiple labels is not the first
        # field
        set panvar(nLabels) 1
        for {set field 0} {$field < $nfields} {incr field} {
            if {($field<$ndim) &&([set nlabels [llength $panvar(labelVal,$field)]] > $panvar(nLabels))} {
                set panvar(nLabels) $nlabels
            }
        }
        
        # end edited code
        
        
        set panvar(int) [nv_peak elem int $panvar(curr_list).$panvar(curr_peak)]
        set panvar(vol) [nv_peak elem vol $panvar(curr_list).$panvar(curr_peak)]
        set panvar(comment) [nv_peak elem comment $panvar(curr_list).$panvar(curr_peak)]
        set status [nv_peak status $panvar(curr_list).$panvar(curr_peak)]
        if {$status>=0} {
            $nm.mbar.delstatus config -fg black
            $nm.mbar.delstatus config -highlightcolor black
        } else {
            $nm.mbar.delstatus config -fg red
            $nm.mbar.delstatus config -highlightcolor red
        }
        set locked [nv_peak elem flag0 $panvar(curr_list).$panvar(curr_peak)]
        if {$locked} {
            $nm.mbar.locked config -bitmap @$Nv_Priv(image,lock)
        } else {
            $nm.mbar.locked config -bitmap @$Nv_Priv(image,unlock)
        }
        if {!$Nv_Priv(javaMode)} {
            nv_peak sho $panvar(curr_list).$panvar(curr_peak)
        }
        if {[winfo exists $Nv_Priv(peakId)]} {
            NvIdPeak
        }
    }
    $nm.mbar.peak configure -bg LightGray

	#added by Pascal Mercier
	if {[winfo exists .filter]} {
	Nv_UpdateFilter
        }
        #

    return 0
}

proc NvGetPeakAnal {nm} {
    global curr_list curr_peak PDIM_MAX
    wm title $nm  $curr_list
    if {[catch "eval nv_peak status $curr_list.$curr_peak"]} {return}
    set varnm [NvVarnm $nm]
    
    global $varnm c
    upvar #0 $varnm panvar
    
    set nrows [set ${varnm}(nRows)]
    
    set nfields $PDIM_MAX
    for {set i 0 } {$i < $nrows} {incr i} {
        set typ  [set panvar(Type,$i)]
        set labels [nv_peak label $panvar(curr_list)]
        set ndim [llength $labels]
        for {set field 0 } {$field < $ndim} {incr field} {
            set fldnm [lindex $labels $field]
            if {$typ==".L"} {
                if {$panvar(labelIndex) >= [llength $panvar(labelVal,$field)]} {
                    lappend panvar(labelVal,$field) $panvar($i,$field)
                } else {
                    set panvar(labelVal,$field) \
                    [lreplace $panvar(labelVal,$field) $panvar(labelIndex) $panvar(labelIndex) $panvar($i,$field)]
                }
                set newLabels ""
                foreach labelVal $panvar(labelVal,$field) {
                    if {$labelVal != {}} {
                        lappend newLabels $labelVal
                        
                    }
                    
                }
                
                nv_peak elem $fldnm$typ $panvar(curr_list).$panvar(curr_peak) $newLabels
                
            } elseif {($typ != ".R") } {
                nv_peak elem $fldnm$typ $panvar(curr_list).$panvar(curr_peak) [set panvar($i,$field)]
            }
        }
    }
    
    # begin edited code to correctly represent the number of labels
    # when the field containing the multiple labels is not the first
    # field
    
    set panvar(nLabels) 1
    for {set field 0} {$field < $nfields} {incr field} {
        if {($field<$ndim) &&([set nlabels [llength $panvar(labelVal,$field)]] > $panvar(nLabels))} {
            set panvar(nLabels) $nlabels
        }
    }
    
    
    # end edited code
    
    nv_peak elem int $panvar(curr_list).$panvar(curr_peak) [set panvar(int)]
    nv_peak elem vol $panvar(curr_list).$panvar(curr_peak) [set panvar(int)]
    nv_peak elem comment $panvar(curr_list).$panvar(curr_peak) [set panvar(comment)]
    $nm.mbar.peak configure -bg LightGray
    nv_database sync
}

proc NvPeakGetList {peak} {
    set dot [string first "." $peak]
    if {$dot<0} {
        return $peak
    } else {
        set list [string range $peak 0 [expr $dot-1]]
    }
    return $list
}

proc NvFillPeakAnalMenu {nm menu var} {
    $menu delete 0 end
    foreach list [nv_peak list] {
        if {$list != ""} {
            $menu add radiobutton -label $list -variable $var -value $list -command "NvPeakAnalSelect $nm"
        }
    }
}

proc NvPeakAnalSelect {nm} {
    set varnm [NvVarnm $nm]
    global  Nv_Priv
    upvar #0 $varnm panvar
    
    set panvar(curr_peak) 0
    NvUpdatePeakRef $nm
    NvUpdatePeakAnal $nm
}

proc NvPeakPlotVolInt {} {
    global xyGraph
    Nv_PeakXYVolInt
    set xyGraph(xAxisBottom,label) Vol.
    set xyGraph(yAxisLeft,label)  Int.
    NvGraphDisplay xyGraph
}

proc NvPeakNavigate args {
    global Nv_Priv
    set narg [llength $args]
    if {$narg>0} { set nm [lindex $args 0] } else {set nm $Nv_Priv(peakAnal)}
}

proc NvPeakCompress {} {
    global Nv_Prefs
    if {!$Nv_Prefs(override,compact)} {
        set result [tk_messageBox -icon question -type yesno -message "Compress peak list? This will permanently remove any peaks marked for deletion."]
        if {$result=="no"} {
            return
        }
    }
    nv_peak compress
}

proc NvPeakDeleteList {list} {
    set result [tk_messageBox -icon question -type yesno -message "Delete the peak list\n named ${list}?\n  Once you do another peakpick this will be permanent!"]
    if {$result=="no"} {return}
    nv_peak name $list ""
}

proc NvPeaksWriteList {} {
    global curr_list Nv_Prefs
    if {$curr_list==""} {error "NvPeaksWriteList: Select a peak list first"}
    set lst [nv_peak name $curr_list]
    if {$Nv_Prefs(usedirs)} {
        set idir $Nv_Prefs(dir,rdpeaks)
    } else {
        set idir [pwd]
    }
    set filename [tk_getSaveFile -title "NMRView PeakList" -initialdir $idir]
    if {$filename=={}} {
        return
    }
    
    if {[lindex [split $filename .] 1] == "str"} {error "Attempting to save a peaklist to a file of type STAR"}
    writepks $lst $filename
}


proc NvPeakRegister {widget} {
    global Nv_Priv
    if {[info exists Nv_Priv(peakWidgets)]} {
        if {[lsearch $Nv_Priv(peakWidgets) $widget] >= 0} {
            return
        }
    }
    lappend Nv_Priv(peakWidgets) $widget
    
}

proc NvPeakPartners {} {
    global curr_list curr_peak
    clear
    foreach list [nv_peak lists] {
        set nmatch 0
        set template ""
        set search ""
        foreach label [nv_peak labels $curr_list] tol [nv_peak idtol $curr_list] {
            if {[string match *C* $label]} {
                continue
            }
            if {[lsearch [nv_peak labels $list] $label] >= 0} {
                incr nmatch
                lappend template $label
                lappend template $tol
                lappend search [eval nv_peak elem $label.P $curr_list.$curr_peak]
            }
        }
        if {$nmatch > 1} {
            eval nv_peak template $list $template
            foreach peak [eval nv_peak find $list $search] {
                puts -nonewline [format "%-18s " $peak]
                foreach label [nv_peak labels $list] {
                    set assignment [eval nv_peak elem $label.L $peak]
                    puts -nonewline [format "%-8s " $assignment]
                }
                puts ""
            }
        }
    }
    tkConPrompt
}


# procedure responding to the added Go button
# procedure added by Pascal Mercier, April 22, 1999

proc GoPeakPlane {nm} {
global curr_list 
global curr_peak
global Nv_Priv


set varnm [NvVarnm $nm]
upvar #0 $varnm panvar


set labels [nv_peak label $curr_list]
set ndim [llength $labels] 

set name_win [nv_win act]

     set tl [winfo toplevel $name_win]
     set children [info commands $tl.*]
     
       foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}

        nv_win active $child
        
        #check if the list is associated with the spectrum
          if {[nv_win dispeaks]==$curr_list} {
           
        
             #check if it's a 3D expiment or not
             # important : assumes that z slices are displayed (2D of x,y)

                if {$ndim ==3} {
		set dim3peaklist [lindex [nv_peak label] 2]
                set zplane [nv_peak elem $dim3peaklist.P $curr_list.$curr_peak]       
                nv_win z $zplane
                nv_win draw
	        } 

              
              # check first if the peak is contained within the limit of the spectrum.  If not, make the window go full
	      
	     set dim1peaklist [lindex [nv_peak label] 0]
             set dim2peaklist [lindex [nv_peak label] 1]
	     set xmax [lindex [nv_win ppm x] 0]
	     set xmin [lindex [nv_win ppm x] 1]
             set ymin [lindex [nv_win ppm y] 0]
	     set ymax [lindex [nv_win ppm y] 1]
	
		#looks at the chemical shift in the x and y dimension

	     set CSdim1 [nv_peak elem $dim1peaklist.P $curr_list.$curr_peak]
	     set CSdim2 [nv_peak elem $dim2peaklist.P $curr_list.$curr_peak]

 	      	# check if the orientation of the spectum is the same as the peak list
	      if {[nv_win lab x]==$dim1peaklist} {
	       if {$CSdim1 > $xmax || $CSdim1 < $xmin || $CSdim2 > $ymax || $CSdim2 < $ymin} {
		nv_win full
		nv_win draw
		}

		} else {
		 if {$CSdim1 > $ymax || $CSdim1 < $ymin || $CSdim2 > $xmax || $CSdim2 < $xmin} {
		nv_win full
		nv_win draw
		}
	       }

	      #puts the black cursor on the the peak
	      # check if the orientation of the spectum is the same as the peak list

             if {[nv_win lab x]==$dim1peaklist} {
              nv_win cross1 [nv_peak elem $dim1peaklist.P $curr_list.$curr_peak] [nv_peak elem $dim2peaklist.P $curr_list.$curr_peak]
             } else {
              nv_win cross1 [nv_peak elem $dim2peaklist.P $curr_list.$curr_peak] [nv_peak elem $dim1peaklist.P $curr_list.$curr_peak]
             }
        }
   }
     nv_win active $name_win
 }


proc NvPeakShowFilter {} {
global Nv_Priv curr_list curr_peak nm
if {![winfo exists .Filter]} {
        Nv_MkFilter
    }
}




################################################
#
# creation of the toplevel window "Filter"
#
################################################

proc Nv_MkFilter {} {

global Nv_Priv curr_list curr_peak nm 

if {[winfo exists .filter]} {
        wm deiconify .filter
	Nv_UpdateFilter
	return
    }



toplevel .filter
wm withdraw .filter

wm geometry .filter 567x227+357+453

frame .filter.frame2 -bd 1 -relief flat 
label .filter.frame2.activelist -text "List to filter out" -font *-Helvetica-bold-*-normal--12-*
label .filter.frame2.nameactivelist   -font *-Helvetica-r-*-normal--12-*

label .filter.frame2.number -text "Number of peaks:"  -font *-Helvetica-bold-*-normal--12-*
label .filter.frame2.numpeaks -width 4 -font *-Helvetica-r-*-normal--12-*

frame .filter.frame3 -bd 1 -relief flat
label .filter.frame3.firstdim  -width 8 -font *-Helvetica-r-*-normal--12-* 
label .filter.frame3.seconddim -width 8 -font *-Helvetica-r-*-normal--12-*


frame .filter.frame4 -bd 1 -relief flat
checkbutton .filter.frame4.firstdim -text "Positive peaks"  -variable positivefirstcheck -width 12 -font *-Helvetica-r-*-normal--12-* 
entry .filter.frame4.firstdimpos -textvariable FirstDimPos  -width 20 -font *-Helvetica-r-*-normal--12-* 
checkbutton .filter.frame4.seconddim -text "Positive peaks" -variable positivesecondcheck -width 12 -font *-Helvetica-r-*-normal--12-* 
entry .filter.frame4.seconddimpos -textvariable SecondDimPos  -width 20 -font *-Helvetica-r-*-normal--12-*

frame .filter.frame5 -bd 1 -relief flat
checkbutton .filter.frame5.firstdim -text "Negative peaks" -variable negativefirstcheck -width 12 -font *-Helvetica-r-*-normal--12-*
entry .filter.frame5.firstdimneg -textvariable FirstDimNeg  -width 20 -font *-Helvetica-r-*-normal--12-*

checkbutton .filter.frame5.seconddim -text "Negative peaks" -variable negativesecondcheck -width 12 -font *-Helvetica-r-*-normal--12-*
entry .filter.frame5.seconddimneg -textvariable SecondDimNeg  -width 20 -font *-Helvetica-r-*-normal--12-*

frame .filter.frame6 -bd 1 -relief flat
checkbutton .filter.frame6.diagonal -text "Delete diagonal peaks"  -variable diagonalcheck -font *-Helvetica-r-*-normal--12-*
entry .filter.frame6.diagonalvalue -textvariable DiagonalThreshold  -width 5 -font *-Helvetica-r-*-normal--12-*

frame .filter.frame7 -bd 1 -relief flat
checkbutton .filter.frame7.posintensity -text "Postive Intensity"  -variable posintensitycheck -font *-Helvetica-r-*-normal--12-*
entry .filter.frame7.posintensityvalue -textvariable IntensityThresholdPOS  -width 10 -font *-Helvetica-r-*-normal--12-*
checkbutton .filter.frame7.negintensity -text "Negative Intensity"  -variable negintensitycheck -font *-Helvetica-r-*-normal--12-*
entry .filter.frame7.negintensityvalue -textvariable IntensityThresholdNEG  -width 10 -font *-Helvetica-r-*-normal--12-*

frame .filter.frame8 -bd 1 -relief flat
entry .filter.frame8.firstpeaknum -textvariable FirstPeakNum  -width 20 -font *-Helvetica-r-*-normal--12-* 
entry .filter.frame8.firstlastnum -textvariable LasttPeakNum  -width 20 -font *-Helvetica-r-*-normal--12-* 

frame .filter.frame9 -bd 1 -relief flat
button .filter.frame9.proceed -text "Proceed" -command {Nv_ProceedFilter}
button .filter.frame9.exit_bt -text "Exit" -command {destroy .filter}

pack .filter.frame2
pack .filter.frame2.activelist 	-padx 1 -ipadx 1 -side left
pack .filter.frame2.nameactivelist  	-padx 1 -ipadx 1 -side left
pack .filter.frame2.numpeaks  	-padx 1 -ipadx 1 -side right
pack .filter.frame2.number -padx 1 -ipadx 1 -side right

pack .filter.frame3 
pack .filter.frame3.firstdim -padx 1 -ipadx 1 -side left
pack .filter.frame3.seconddim -padx 1 -ipadx 1 -side left

pack .filter.frame4
pack .filter.frame4.firstdim  -padx 1 -ipadx 1 -side left
pack .filter.frame4.firstdimpos  -padx 1 -ipadx 1 -side left
pack .filter.frame4.seconddim  -padx 1 -ipadx 1 -side left
pack .filter.frame4.seconddimpos  -padx 1 -ipadx 1 -side left

pack .filter.frame5
pack .filter.frame5.firstdim  -padx 1 -ipadx 1 -side left
pack .filter.frame5.firstdimneg  -padx 1 -ipadx 1 -side left
pack .filter.frame5.seconddim  -padx 1 -ipadx 1 -side left
pack .filter.frame5.seconddimneg  -padx 1 -ipadx 1 -side left

pack .filter.frame6  -fill both -padx 8m -pady 4m
pack .filter.frame6.diagonal -padx 1 -ipadx 1 -side left
pack .filter.frame6.diagonalvalue -padx 10 -ipadx 1 -side left

pack .filter.frame7  -fill both -padx 8m -pady 1m
pack .filter.frame7.posintensity -padx 1 -ipadx 1 -side left
pack .filter.frame7.posintensityvalue -padx 10 -ipadx 1 -side left
pack .filter.frame7.negintensity -padx 1 -ipadx 1 -side left
pack .filter.frame7.negintensityvalue -padx 10 -ipadx 1 -side left


pack .filter.frame9  -fill both -padx 8m -pady 1m
pack .filter.frame9.proceed -padx 1 -ipadx 1 -side left
pack .filter.frame9.exit_bt -padx 1 -ipadx 1 -side left


wm deiconify .filter
Nv_UpdateFilter
}			



###############################################################
#
# Filter window upgrade
# this procedure is also called from the NvUpdatePeakAnal
# procedure in case the Filter window exists
###############################################################

proc Nv_UpdateFilter {} {
global curr_list
global namefirstdim
global nameseconddim


set namefirstdim [lindex [nv_peak lab $curr_list] 0]
set nameseconddim [lindex [nv_peak lab $curr_list] 1]

wm title .filter "Filter $curr_list peak list"
.filter.frame2.numpeaks configure -text [nv_peak n]
.filter.frame2.nameactivelist configure -text $curr_list
.filter.frame3.firstdim configure -text $namefirstdim
.filter.frame3.seconddim configure -text $nameseconddim
}




###############################################################
#
# Filter the list according to the options chosen
###############################################################

proc Nv_ProceedFilter {} {
global curr_list 
global positivefirstcheck negativefirstcheck positivesecondcheck negativesecondcheck
global diagonalcheck posintensitycheck negintensitycheck
global namefirstdim nameseconddim

set IntervalsFirstDimPos [.filter.frame4.firstdimpos get]
set IntervalsFirstDimNeg [.filter.frame5.firstdimneg get]
set IntervalsSecondDimPOS [.filter.frame4.seconddimpos get]
set IntervalsSecondDimNEG [.filter.frame5.seconddimneg get]
set ThresholdDiagonal [.filter.frame6.diagonalvalue get]
set POSThresholdIntensity [.filter.frame7.posintensityvalue get]
set NEGThresholdIntensity [.filter.frame7.negintensityvalue get]

if {$IntervalsFirstDimPos==""} {set positivefirstcheck 0}
if {$IntervalsFirstDimNeg==""} {set negativefirstcheck 0}
if {$IntervalsSecondDimPOS==""} {set positivesecondcheck 0}
if {$IntervalsSecondDimNEG==""} {set negativesecondcheck 0}
if {$ThresholdDiagonal==""}  {set diagonalcheck 0}
if {$POSThresholdIntensity==""} {set posintensitycheck 0}
if {$NEGThresholdIntensity==""} {set negintensitycheck 0}

	if {$positivefirstcheck} {	 
	puts "executing Nv_FilterPosNeg $curr_list POS $namefirstdim $IntervalsFirstDimPos"
 	Nv_FilterPosNeg $curr_list POS $namefirstdim $IntervalsFirstDimPos
	}

	if {$negativefirstcheck} {	 
	puts "executing Nv_FilterPosNeg $curr_list NEG $namefirstdim $IntervalsFirstDimNeg"
	Nv_FilterPosNeg $curr_list NEG $namefirstdim $IntervalsFirstDimNeg 
	}

	if {$positivesecondcheck} {	 
	puts "executing Nv_FilterPosNeg $curr_list POS $nameseconddim $IntervalsSecondDimPOS" 
	Nv_FilterPosNeg $curr_list POS $nameseconddim $IntervalsSecondDimPOS
	}

	if {$negativesecondcheck} {	 
	puts "executing Nv_FilterPosNeg $curr_list NEG $nameseconddim $IntervalsSecondDimNEG" 
	Nv_FilterPosNeg $curr_list NEG $nameseconddim $IntervalsSecondDimNEG
	}

	if {$diagonalcheck} {
	puts "executing Nv_FilterDiagonal $curr_list $namefirstdim $nameseconddim $ThresholdDiagonal"
	Nv_FilterDiagonal $curr_list $namefirstdim $nameseconddim $ThresholdDiagonal	
	}

	if {$posintensitycheck} {
	puts "executing Nv_FilterTooWeak $curr_list POS  $POSThresholdIntensity"
	Nv_FilterTooWeak $curr_list POS  $POSThresholdIntensity	
	}

	if {$negintensitycheck} {
	puts "executing Nv_FilterTooWeak $curr_list NEG  $NEGThresholdIntensity"
	Nv_FilterTooWeak $curr_list NEG  $POSThresholdIntensity 	
	}


}
#############################################################
#
# General Proc to filter out pos or neg peaks
#
# The proc needs these args
# PeakList: the peak list to filter out
# Sign: POS or NEG (can pass pos or neg as well)
# DimToLook: the name of the dimension
# IntervalsToProcess: a text line containing limits in () - ppm units.
#                     the intervals have to be seperated by spaces
# 		      an unlimited number of () can be specified
# 
# Nv_FilterPosNeg hncacb pos C (44,46) (50,63)
# would keep all the peaks in the hncacb peak list that
# have a positive intensity and whose chemical shift in the C
# dimension are in the intervals (44,66) or (50,63)
# the peaks not satisfying these conditions in the C dimension
# will be marked as to be deleted 
#
# written by Pascal Mercier
# MRC group on protein structure
# Department of Biochemistry
# University of Alberta
# May 8, 1999
#############################################################

proc Nv_FilterPosNeg args {

set PeakList [lindex $args 0]
set NumberOfPeaks [nv_peak n $PeakList]
set Sign [lindex $args 1]
set DimToLook [lindex $args 2]
set IntervalsToProcess [lrange $args 3 [llength $args]]

set NumOfIntervals [llength $IntervalsToProcess]

set adjustment 0
puts "Sign is $Sign"

if {[string toupper $Sign]=="POS"} {
set adjustment 1
} else {
set adjustment -1
}

puts "adjustment is $adjustment"

for {set j 0} {$j < $NumOfIntervals} {incr j} {
	set Interval [lindex $IntervalsToProcess $j]
	set Interval [string trim $Interval (]
	set Interval [string trim $Interval )]
	#look for the comma
	set indexcoma [string first , $Interval]
	set Borneinf [string range $Interval 0 [expr $indexcoma-1]]
	set Bornesup [string range $Interval [expr $indexcoma+1] [string length $Interval]]

	#Filter the peaklist

	puts "Interval to process: $IntervalsToProcess"
        puts "Borneinf is $Borneinf and bornesup is $Bornesup"
	puts "Peaklist $PeakList  DimToLook $DimToLook"

       
          #loop in the peaklist (check each peak one by one)
	  #this routine was updated on August 6, 1999 so that
	  #it works with nmrview 4.1.0
	  #older version should start the loop with "for {set k 0} {$k < $NumberOfPeaks} {incr k}"

	 foreachpeak k $PeakList {

		set Intensity [nv_peak elem int $PeakList.$k]
		set ppm [nv_peak elem $DimToLook $PeakList.$k]
		set CorrIntensity [expr $Intensity*$adjustment]
	

	   	#check first if the peak has the right intensity sign
	    		if {$CorrIntensity > 0} {
			
			# the peak has an intensity that matches what was passed in the args
			# if ppm is within the interval, mark it as good, if not, delete it
				if {$ppm>=$Borneinf && $ppm<=$Bornesup} {
				nv_peak delete $PeakList.$k
				#puts "peak $k deleted"
				#incr NumPeaksKept
			} else {
			nv_peak undelete  $PeakList.$k
			}
	    }
	 }
	}
bell
puts "Done"
}


#############################################################
#Proc to filter out Diagonal Peaks
# The proc needs these args
# PeakList: the peak list to filter out
# Dim1ToLook: the name of the first dimension
# Dim2ToLook: the name of the second dimension\
# Treshold: the treshold limit in ppm to consider it's a diagonal peak
#
# adapted from Bruce's procedure included in nmrview4
# Pascal Mercier
# MRC group on protein structure
# Department of Biochemistry
# University of Alberta
# May 8, 1999
#############################################################

proc Nv_FilterDiagonal args {

global NumPeaksKept

set PeakList [lindex $args 0]
set Dim1ToLook [lindex $args 1]
set Dim2ToLook [lindex $args 2]
set Treshold [lindex $args 3]

set NumberOfPeaks [nv_peak n $PeakList]

    
   
	  #this loop was updated on August 6, 1999 so that
	  #it works with nmrview 4.1.0
	  #older version should start the loop with " for {set j 0} {$j < $NumberOfPeaks} {incr j}"

	foreachpeak j $PeakList {

        set ppm1 [eval nv_peak elem $Dim1ToLook $PeakList.$j]
        set ppm2 [eval nv_peak elem $Dim2ToLook $PeakList.$j]
        set offset [ expr {abs($ppm1-$ppm2)} ]
        if {[ expr {$offset<$Treshold} ] } {
            eval nv_peak delete $PeakList.$j
        }
 }
}



#############################################################
#Proc to filter out peaks that are too weak
# The proc needs these args
# PeakList: the peak list to filter out
# Sign: pos or neg (POS/NEG accepted too): positive or negative peaks
# Treshold: the treshold limit in ppm 
#
# adapted from Bruce's procedure included in nmrview4
# Pascal Mercier
# MRC group on protein structure
# Department of Biochemistry
# University of Alberta
# May 8, 1999
#############################################################

proc Nv_FilterTooWeak args {

set PeakList [lindex $args 0]
set Sign [lindex $args 1]
set Threshold [lindex $args 2]
set NumberOfPeaks [nv_peak n $PeakList]

set Sign [string toupper $Sign]


  if {$Sign!= "POS" && $Sign!= "NEG"} {error "Nv_FilterTooWeak: need to specify positive or negative"}

  



	  #this loop was updated on August 6, 1999 so that
	  #it works with nmrview 4.1.0
	  #older version should start the loop with " for {set j 0} {$j < $NumberOfPeaks} {incr j}"


	
if {$Sign=="POS"} {
set adjustment 1
} else {
set adjustment -1
}


	foreachpeak j $PeakList {
        set lvl [nv_peak elem int $PeakList.$j]
          	if {$lvl*$adjustment>0 && $lvl*$adjustment<$Threshold} {
            	nv_peak delete $PeakList.$j
          	}
 
 	}


}


#############################################################
#Proc to filter out peaks that are too narrow or too wide
# The proc needs these args
# PeakList: the peak list to filter out
# Size: narrow or wide (capital letters also accepted)
# args: The tolerances (in ppm) for each dimension
# 
# for exemple: Nv_PeakCutThinOrFat noesy narrow 0.1 0.2
#
# adapted from Bruce's procedure included in nmrview4
# Pascal Mercier
# MRC group on protein structure
# Department of Biochemistry
# University of Alberta
# May 8, 1999
#############################################################


proc Nv_PeakCutThinOrFat {PeakList Size args} {

    set Size [string upper $Size]

    if {$Size!= "NARROW" && $Size!= "WIDE"} {error "Nv_PeakCutThinOrFat: need to specify narrow or wide"}

    if {$Size= "NARROW"} {
    set adjustment 1
    } else {
    set ajustement -1
    }
    set pklabel [nv_peak label $PeakList]
    set ndim [llength $pklabel]
    
    if {$ndim != $nargs} {error "Nv_PeakCutThinorFat: Number of arguments != number of dimensions"}
    
    set ncut 0
    set n [nv_peak n $PeakList]
    for {set j 0} {$j < $n} {incr j } {
        set i 0
        foreach dimname $pklabel {
            set width [nv_peak elem $dimname.W $PeakList.$j]
            set width [expr {abs($width)}]
            set tol [lindex $args $i]
            incr i
            if {[ expr {$adjustment*$width<$adjustment*$tol} ] } {
                nv_peak delete $PeakList.$j
                incr ncut
            }
        }
    }
    puts "Cut $ncut peaks"
}
