# Script to automatically fit T1 or T2 relaxation data
# Version 3.0
# Written by Pascal Mercier, May 29, 2001
# Original version by Pascal Mercier, Oct 23, 1999
#


# Comments for version 3.2
# adapted to nmrview 5.0.4 by changing nv_peak -elem to nv_peak elem (no hyphen)

# Comments for version 3
# Major updates/new features for this release
# Added intensity to the program and the graphical interface to go with it
# Took out volume measurement
# The fits are done by taking the natural log of the peak intensities and a linear regression analysis is performed
# The fits are done with weithing the data according to the noise level in the datasets (neat, isn't it?)
# The source code for linear fitting was taken from Numerical Receipes in C and adapted to Tcl
# The noise level is automatically estimated
# The contour level at witch the datasets are displayed is automatically set by nmrview with a built-in procedure and no longer depend on the related parameter in the par file.
# The delays are automatically written at the top of the intensity file, so that the output file can be used right away with lmquick or xcrvfit.

# Comments for version 2.2
# Fixed a bug (well not really one) that would cause the first dataset to always remain as part of the active window according to the attribute panel.  The "nv_win dataset" command used in previous version was not sufficient to clear the original dataset.
# As a new dataset is processed, no peak now remains displayed prior to automatic peak picking, so the user can better see which peaks the computer is missing.
# Got rid of useless graphical interface elements that we not used since version 1 (!!!!).  I had been too lazy to do so before. 
# For peaks that could not be picked because their intensity was too low (were not drawn because the peak levels were too high), the "-" sign that was used was replaced by "#" so that the output file can be looked at directly using xcrvfit, but don't forget to add the delays at the begining of the file.


# Comments for version 2.1
# version 2.1 fix an important bug that would prevent the labeling of all the peaks following a given one that could not be picked because its intensity was too low.
#
# 
# Comments for version 2.0
# This original script (v1) was later modified so that nmrview would pick peaks
# one by one based on the chemical shift of peaks in the reference peak list
# version 1.0 would simply take the intensities at the exact same chemical shifts
# found in the reference list.  In cases where peaks shift a bit during relaxation 
# measurements, this way of getting intensities is totally inapropriate.  This version
# corrects this.
# Nov 27, 2000

proc relax {} {

if {![winfo exists .relax]} {
       Nv_Mkrelax
    } else {
     wm deiconify .relax
    }
}

################################################
#
# creation of the toplevel window "relax"
#
################################################

proc Nv_Mkrelax {} {

global Nv_Priv


if {[winfo exists .relax]} {
        wm deiconify .relax
	Nv_Updaterelax
	return
    }

set base .relax

    toplevel $base -class Toplevel \
         -menu .relax.023 
    wm focusmodel $base passive
    wm geometry $base 440x483+214+195
    wm maxsize $base 1265 994
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "T1 and T2 analysis tool"
    
    label $base.01 \
        -background #68d488 -borderwidth 1 -font {Times -10} -relief raised \
        -text {Written by Pascal Mercier
v3. May 2001} 
    label $base.02 \
        -background #d9d974 -borderwidth 1 -font {Helvetica -12 bold italic} \
        -foreground #000000 -highlightcolor #000000 -relief raised \
        -text {Automatic peak picking for T1/T2 relaxation data} 
    label $base.03 \
        -borderwidth 1 -font {Helvetica -12} -text {Experiment generic name} 
    label $base.04 \
        -borderwidth 1 -font {Helvetica -12} -text {File index} 

    label $base.05 \
        -borderwidth 1 -font {Helvetica -12} -text {Delays} 
    label $base.06 \
        -borderwidth 1 -font {Helvetica -12} \
        -text {Copy .par file from first to all} 
   
    label $base.07 \
        -borderwidth 1 -font {Helvetica -12} \
        -text {Automatically save xpk peak lists} 
    label $base.08 \
        -borderwidth 1 -font {Helvetica -12} \
        -text {Output results for residue} 
    label $base.09 \
        -borderwidth 1 -font {Helvetica -12} \
        -text {Intensity file output name} 
    label $base.010 \
        -borderwidth 1 -font {Helvetica -12} \
        -text {Clear dataset from memory when done} 
    label $base.011 \
        -borderwidth 1 -font {Helvetica -12} -text {Decay fitting file output name} 
    
    entry $base.015 \
         -textvariable t1generic  
    entry $base.016 \
        -font {Helvetica -10} -textvariable t1fileindex    
      entry $base.017 \
        -font {Helvetica -10} -textvariable expdelays
    checkbutton $base.018 \
        -variable t1copypar 
    frame $base.022 \
        -borderwidth 2 -height 75 -relief groove -width 125 
    menu $base.023 \
         
    menu $base.024 \
        -type menubar 
    label $base.lab75 \
        -background #d9d974 -borderwidth 1 -font {Helvetica -12 bold italic} \
        -relief raised -text {Post treatment options} 
    entry $base.ent77 \
        -textvariable t1firstres  
    label $base.lab78 \
        -borderwidth 1 -font {Helvetica -12} -text to 
    entry $base.ent79 \
        -textvariable t1lastres  
  entry $base.ent80 \
        -textvariable t1intfile  
    entry $base.ent81 \
        -textvariable t1decayfile  

    checkbutton $base.che83 \
        -variable t1savexpk 
    checkbutton $base.che84 \
         -variable t1clearnv 
      button $base.but17 \
        -background #ff2420 -command Nv_ProceedRelaxation -padx 11 -pady 4 \
        -text Proceed 
    button $base.but18 \
        -background #d96640 -command {destroy .relax} -padx 11 -pady 4 \
        -text Dismiss 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.01 \
        -x 20 -y 437 -width 156 -height 38 -anchor nw -bordermode ignore 
    place $base.02 \
        -x 25 -y 5 -width 400 -height 18 -anchor nw -bordermode ignore 
    place $base.03 \
        -x 20 -y 90 -anchor nw -bordermode ignore 
    place $base.04 \
        -x 20 -y 115 -anchor nw -bordermode ignore 
    place $base.05 \
        -x 20 -y 150 -anchor nw -bordermode ignore
    place $base.06 \
        -x 20 -y 180 -anchor nw -bordermode ignore 
   
    place $base.07 \
        -x 20 -y 375 -anchor nw -bordermode ignore 
    place $base.08 \
        -x 20 -y 255 -anchor nw -bordermode ignore 
    place $base.09 \
        -x 20 -y 295 -anchor nw -bordermode ignore 
    place $base.010 \
        -x 20 -y 405 -anchor nw -bordermode ignore 
    place $base.011 \
        -x 20 -y 335 -anchor nw -bordermode ignore 
     
    place $base.015 \
        -x 225 -y 85 -width 143 -height 22 -anchor nw -bordermode ignore 
    place $base.016 \
        -x 225 -y 115 -width 143 -height 22 -anchor nw -bordermode ignore
     place $base.017 \
        -x 105 -y 145 -width 263 -height 22 -anchor nw -bordermode ignore
    place $base.018 \
        -x 348 -y 180 -width 26 -height 22 -anchor nw -bordermode ignore 
    place $base.022 \
        -x 20 -y 255 -width 505 -height -10 -anchor nw -bordermode ignore 
    place $base.lab75 \
        -x 20 -y 220 -width 400 -height 18 -anchor nw -bordermode ignore 
    place $base.ent77 \
        -x 235 -y 255 -width 33 -height 22 -anchor nw -bordermode ignore 
    place $base.lab78 \
        -x 280 -y 255 -anchor nw -bordermode ignore 
    place $base.ent79 \
        -x 308 -y 255 -width 33 -height 22 -anchor nw -bordermode ignore 
    place $base.ent80 \
        -x 230 -y 295 -width 113 -height 22 -anchor nw -bordermode ignore 
    place $base.ent81 \
        -x 230 -y 335 -width 113 -height 22 -anchor nw -bordermode ignore 
    place $base.che83 \
        -x 320 -y 370 -anchor nw -bordermode ignore 
    place $base.che84 \
        -x 320 -y 400 -anchor nw -bordermode ignore 
      place $base.but17 \
        -x 180 -y 440 -width 111 -height 28 -anchor nw -bordermode ignore 
    place $base.but18 \
        -x 300 -y 440 -width 111 -height 28 -anchor nw -bordermode ignore 
}





#############################################################
#
#     Procedure responding to the Proceed button
#
##############################################################

proc Nv_ProceedRelaxation  {} {

global t1dataset t1fileindex t1copypar expdelays
global t1generic
global t1clearnv
global t1savexpk
global t1intfile t1decayfile
global t1firstres t1lastres


Nv_PickPeakRelaxation $t1generic $t1fileindex $expdelays $t1copypar $t1clearnv $t1savexpk $t1intfile $t1decayfile $t1firstres $t1lastres

}


#############################################################
#
#	General procedure for both T1 and T2
#     Peak picking according to the options chosen
#
##############################################################

proc Nv_PickPeakRelaxation args {

global Nv_Priv
global NvSpectrum
global peaktableint


set generic [lindex $args 0]
set fileindex [lindex $args 1]
set expdelays [lindex $args 2]
set copypar [lindex $args 3]
set closenv [lindex $args 4]
set savexpk [lindex $args 5]
set intensityfile [lindex $args 6]
set decayfile [lindex $args 7]
set firstresidue [lindex $args 8]
set lastresidue [lindex $args 9]

set startdataset [nv_win dataset]
set win [nv_win act]
set noiseindatasets ""
set rmode 0




# fileindex is seperated by commas
# have to seperate the numbers by a space instead of a comma
# and count the number of elements

set newfileindex ""

while {[string first , $fileindex] > -1} {
	set temp [string first , $fileindex]

	set newfileindex [concat $newfileindex [string range $fileindex 0 [incr temp -1]]]
	set fileindex [string range $fileindex [expr $temp+2] [string length $fileindex]]
	}

set newfileindex [concat $newfileindex $fileindex]


# check what format the user chose to enter the delays
# it might consist of a list of numbers separated by a coma, or simply a number followed
# by a star sign, meaning the the delay used is constant.  In that case, the delays are
# taken to be delay=constant*file index


if {[string first "*" $expdelays] ==-1} {
 # the experimental delays are seperated by commas
 # have to seperate the numbers by a space instead of a comma
 # and count the number of elements

 set newexpdelays ""

 while {[string first , $expdelays] > -1} {
	set temp [string first , $expdelays]

	set newexpdelays [concat $newexpdelays [string range $expdelays 0 [incr temp -1]]]
	set expdelays [string range $expdelays [expr $temp+2] [string length $expdelays]]
 	}

 set newexpdelays [concat $newexpdelays $expdelays]
 # now store the delays in an array
 set temp 0
 foreach timedelay $newexpdelays {
 	set delays{$temp} $timedelay
 	incr temp
 	}
} else {
# the delay is constant from increment to the other
 set constantdelay [string range $expdelays 0 [expr [string first * $expdelays] -1]]
 set temp 0
 set newexpdelays ""
 foreach index $newfileindex {
	set delays{$temp} [expr $constantdelay*$index]
	set newexpdelays [concat $newexpdelays [set delays{$temp}] ]
	puts [set delays{$temp}]
 	incr temp
	}
}




##########################################
# beginning of the peak picking routine
#########################################

set refname [lindex $newfileindex 0]


for {set i 0} {$i<=[expr [llength $newfileindex] -1 ]} {incr i 1} {

        set basename [lindex $newfileindex $i]

puts "doing dataset $basename"

	#check if the user wants to copy the .par file from the 1st experiment to the others
 	if {$copypar && $i!=0} {
		file copy $generic$refname.par $generic$basename.par
		}



	# now open the dataset if it's not already opened
        if {[string first $generic$basename.nv [nv_dataset open]]== -1} {
	nv_dataset open $generic$basename.nv
	}


	# now read the peak list if it's not already opened
	#if {[string first $generic$basename [nv_peak list]] == -1} {
	#readpks $generic$basename.xpk
	#}

	# get the noise level in the dataset
	set noise{$i} [estimatenoisefromdataset $generic$basename.nv]
	



  if {$i!=0} {
	# puts the right dataset in the active window
	puts "putting $generic$basename.nv in window [nv_win act]"
	nv_win act $win
	# the next two lines replace the current dataset in the window by the one on the way to be processed
	[nv_win act] dataset $generic$basename.nv
        set NvSpectrum([nv_win act],datasets) $generic$basename.nv
	# don't display anypeak
	nv_win dispeak none
	# display the dataset at the right contour level (5 times the estimated noise level)
	nv_win lvl [expr 5.0 * [set noise{$i}]]
	# update the spectrum window
	nv_win draw 
	
		
	
	#create a new peak list having the same properties as the first increment peak list
	set previouslist $generic[lindex $newfileindex [expr $i-1]]
	set dim1peaklist [lindex [nv_peak label $previouslist] 0]
        set dim2peaklist [lindex [nv_peak label $previouslist] 1] 
	nv_peak addlist $generic$basename $dim1peaklist $dim2peaklist
	nv_win dispeak $generic$basename

	# now associate the peak list with the new open dataset
	nv_peak dataset $generic$basename $generic$basename.nv

	#pick all the peaks according to the first peak list
	set missed 0
	foreachpeak k  $previouslist {
		#put the black cursor on the the peak
		nv_win cross1 [nv_peak elem $dim1peaklist.P $previouslist.$k] [nv_peak elem $dim2peaklist.P $previouslist.$k]

		#now pick this peak
 		set varnm [NvVarnm .peakPick]
		global $varnm
    		upvar #0 $varnm panvar

    		set panvar(list) [$win dispeaks]
    		set panvar(dataset) [$win dataset]
    		set panvar(mode) 2
    		set panvar(region) [expr 2+$rmode]
    		set xy [nv_win cross1]
    		set panvar(p,1) [lindex $xy 0]
    		set panvar(p,2) [lindex $xy 1]
    		$win pick -var $varnm -dataset $panvar(dataset) $panvar(list)

		
		set currentpeaknuminnewlist [expr $k-$missed]
		# give it the same assignment as in the first peak list if the peak was picked
		set error [catch "nv_peak elem $dim1peaklist.L $generic$basename.$currentpeaknuminnewlist [nv_peak elem $dim1peaklist.L $previouslist.$k]" class]
		if {$error} {incr missed;continue}
		set error [catch "nv_peak elem $dim2peaklist.L $generic$basename.$currentpeaknuminnewlist [nv_peak elem $dim2peaklist.L $previouslist.$k]" class]
		if {$error} {continue}
		
		
		
	}

}

	foreachpeak iPeak $generic$basename {

	#puts "$generic$basename.nv $generic$basename.$iPeak"
        eval nv_peak analysis $generic$basename.nv $generic$basename.$iPeak
        eval nv_peak elem int $generic$basename.$iPeak $Nv_Value(center)
	eval nv_peak elem vol $generic$basename.$iPeak $Nv_Value(evolume)
    	}

	# write the new xpk file if the user has chosen so
	if {$savexpk==1 && $i!=0} {
	writepks $generic$basename
	}


	## stores the intensity of each residue assigned in an array



	for {set j $firstresidue} {$j<=$lastresidue} {incr j 1} {


		set peaknumber [nv_peak match $generic$basename $j.HN]
		set peaknumber [string trimleft $peaknumber $generic$basename]
		set peaknumber [string range $peaknumber 1 end]


		if {$peaknumber!=""} {
			set peaktableint{$j,$i}  [nv_peak elem int $generic$basename.$peaknumber]
			#puts  "for data set $i  peak label $j [set peaktableint{$j,$i}]"
		} else {
			set peaktableint{$j,$i} "#"
		}

	}

	#now close the nv dataset if the user has chosen so
	if {$closenv} {
	nv_dataset close $generic$basename.nv
	}
}





nv_win draw
#end of procedure
}



proc estimatenoisefromdataset {originnv} {

    		set ndim [nv_dataset ndim $originnv]
    		set dsize [nv_dataset size $originnv 1]

    		for {set i 1} {$i<=$ndim} {incr i} {
        		set dsize [nv_dataset size $originnv $i]
        		append dims "-d$i 10p [expr round($dsize/8)]p "
    		}
    		set values [eval nv_dataset analyze $originnv $dims]
    		set sdevs [lsearch $values sdev]
    		set sdev [lindex $values [incr sdevs]]
	    	return $sdev
#end of procedure
}
