############################################################################
# Procedures to change the font for peak labeling, coordinates and the axes
# Written by Pascal Mercier, July 21 2001
# CIHR group in protein function and structure
# Brian Sykes lab
# Univeristy of Alberta
# Edmonton, Canada

# This script calls for dkf_chooseFont
# Written by Donal K. Fellows
# http://www.cs.man.ac.uk/~fellowsd/tcl/
############################################################################

proc FontSelection {} {

if {![winfo exists .fontSelectWindow]} {
       makeFontSelection
    } else {
     wm deiconify .fontSelectWindow
    }
}



proc makeFontSelection {} {
global FontforLabel FontforPeaks FontforCoordinates


#############################
#build the user interface
#############################
set base .fontSelectWindow


 toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 415x186+119+197
    wm maxsize $base 1585 1170
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "NMRView Font Selection"
    label $base.lab18 \
        -borderwidth 0  -relief raised -text {Label Font} -font {Helvetica -10}
    label $base.lab19 \
        -borderwidth 0  -relief raised -text {Peak Font} -font {Helvetica -10}
    label $base.lab20 \
        -borderwidth 0  -relief raised -text {Coordinates Font} -font {Helvetica -10} 
    entry $base.ent21 \
         -validate none -textvariable FontforLabel -font {Helvetica -10}
    entry $base.ent22 \
        -validate none -textvariable FontforPeaks -font {Helvetica -10}
    entry $base.ent23 \
        -validate none -textvariable FontforCoordinates -font {Helvetica -10}
    button $base.but24 \
          -activebackground #fe3640 -padx 9 -pady 3 -text Choose -command {ChooseFonts labelFont $FontforLabel} -font {Helvetica -10}
    button $base.but25 \
          -activebackground #fe3640 -padx 9 -pady 3 -text Choose -command {ChooseFonts peakFont $FontforPeaks} -font {Helvetica -10}
    button $base.but26 \
         -activebackground #fe3640 -padx 9 -pady 3 -text Choose -command {ChooseFonts coordFont $FontforCoordinates} -font {Helvetica -10}
    button $base.but27 \
         -activebackground #fe3640 -padx 9 -pady 3 -text Apply -command {ApplyFontChanges} -font {Helvetica -10}
    button $base.but28 \
        -activebackground #fe3640  -padx 9 -pady 3 -text Quit -command {destroy .fontSelectWindow} -font {Helvetica -10}
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.lab18 \
        -x 10 -y 25 -anchor nw -bordermode ignore 
    place $base.lab19 \
        -x 10 -y 60 -anchor nw -bordermode ignore 
    place $base.lab20 \
        -x 10 -y 100 -anchor nw -bordermode ignore 
    place $base.ent21 \
        -x 135 -y 20 -anchor nw -bordermode ignore 
    place $base.ent22 \
        -x 135 -y 55 -anchor nw -bordermode ignore 
    place $base.ent23 \
        -x 135 -y 95 -anchor nw -bordermode ignore 
    place $base.but24 \
        -x 310 -y 20 -anchor nw -bordermode ignore 
    place $base.but25 \
        -x 310 -y 55 -width 67 -height 26 -anchor nw -bordermode ignore 
    place $base.but26 \
        -x 310 -y 90 -anchor nw -bordermode ignore 
    place $base.but27 \
        -x 140 -y 140 -anchor nw -bordermode ignore 
    place $base.but28 \
        -x 215 -y 140 -anchor nw -bordermode ignore 


updatefontselectWindow
}

proc updatefontselectWindow {} {
global FontforLabel FontforPeaks FontforCoordinates

set curr_win           [nv_win act]
set FontforLabel       [lindex [$curr_win config -labelFont] 4]
set FontforPeaks       [lindex [$curr_win config -peakFont] 4]
set FontforCoordinates [lindex [$curr_win config -coordFont] 4]
}

proc ChooseFonts args {
global FontforLabel FontforPeaks FontforCoordinates
set option    [lindex $args 0]
set curr_font [lindex $args 1]
# get the current setting
set curr_win [nv_win act]
#set curr_font [lindex [$curr_win config -$option] 3]
set wintitle "Choose Font for $option"
set getnewfont [dkf_chooseFont -initialfont $curr_font -title $wintitle]

if {$getnewfont!=""} {
	switch $option {
	labelFont {set FontforLabel $getnewfont}
    	peakFont  {set FontforPeaks $getnewfont}
	coordFont {set FontforCoordinates $getnewfont}
	}
}
}

proc ApplyFontChanges {} {
global FontforLabel FontforPeaks FontforCoordinates
set curr_win [nv_win act]
$curr_win config -labelFont $FontforLabel -peakFont $FontforPeaks -coordFont $FontforCoordinates

}


