# Sample key bindings to do some useful things
# b : (nv_win crosscolor 2 blue) and (redraw)
# d : draw
# e : expand
# f : full
# F : Fontfix
# l : leveldown
# u : levelup
# m : manpick
# p : previous view
# s : showplane
# w : winlink
# x : xslicer
# y : yslicer
# W : change bakground to white
# Delete : delwinlink
# Control-s : save (save the displays)
# Control-Alt-s : savestrips
# Alt-p : showres $NvassignPanel(curres)
# Alt-s : simdraw
# Control-e : expand in several spectra at once (same window)
# Control-z : apply the z coordinates of the active spectrum
#             to several spectra at once (same window)
# Control-p : previous views in several spectra (same window)
# Control-f : fully expand several spectra at once (same window)
# Control-Up : up one plane in the third dimension in several spectra at once (same window)
# Control-Down : down one plane in the third dimension in several spectra at once (same window)
# t : Find symetric peaks (Nv_ShowSymmetryWindow)
# c : find the closest peak (ClosestPeak)
# z : Press z on a peak in an hsqc to display automatically the right 3D plane
#     corresponding to the 15N chemical shift of the peak in all spectra

#Press  button b on keyboard to switch the cursor 2 to color blue
bind Spectrum <b> {
foreach i [win open] {
$i crosscolor 2 blue
}
}

# switched to good fonts
bind Spectrum <F> {
Fontfix
}

# change to white background
bind Spectrum <W> {
#[nv_win act] configure -background whitesmoke
foreach i [win open] {
$i config -background whitesmoke
}
}

#Press  button d on keyboard to draw.
bind Spectrum <d> {
NvSpectrumDraw
}


#Press button e on keyboard to expand box.
bind Spectrum <e> {
nv_win expand 
}

#Press button f on keyboard to get full view.
bind Spectrum  <f> {
nv_win full
}

#Press button l on keyboard to go down one level.
bind Spectrum <l> {
leveldown
}

#Press button m on keyboard to pick a peak.
bind Spectrum <m> {
manpick
}

#Press button p on keyboard to get the previous view.
bind Spectrum <p> {
nv_win previous
}

#Press button s on keyboard to show a plane.
bind Spectrum <s> {
showplane
}

#Press buttons  Alt-s on keyboard simultanious to execute simdraw.
bind Spectrum <Alt-s> {
simdraw
}

#Press button Control-s simultanious on keyboard to save the displays.
bind Spectrum <Control-s> {
save
}

#Press button Controle-Alt-s simultanious on keyboard to save the strips.
bind Spectrum <Control-Alt-s> {
savestrips
}

#Press button y on keyboard to draw a spectrum perpendicular to the x-axis.
bind Spectrum <y> {
yslicer
}

#Press button u on keyboard to go up one level.
bind Spectrum <u> {
levelup
}

#Press button w on keyboard to link different displays.
bind Spectrum <w> {
winlink
}

#Press button x on keyboard to save the displays.
bind Spectrum <x> {
xslicer
}

########################################################################
#oli, July, 2004
#Press button x on keyboard to save the displays.
bind Spectrum <a> {
NvSetCursorMode peakadjust
}

#Press button x on keyboard to save the displays.
bind Spectrum <q> {
NvSetCursorMode crosshairs
}
#########################################################################

#bind Spectrum <Delete> {
#delwinlink
#}

bind Spectrum <Alt-p> {
global NvassignPanel
showres $NvassignPanel(curres)
}

# Press 'ctrl-e' to expand in several spectra at once 
# works only with spectra displayed in columns in a same window 
# Author: Vincent 
bind Spectrum <Control-e> {
     set name_win [nv_win act]
     set coord1x [nv_win cross1x]
     set coord1y [nv_win cross1y]
     set coord2y [nv_win cross2y]
     set coord2x [nv_win cross2x]
     set tl [winfo toplevel $name_win]
     set children [info commands $tl.*]
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
          nv_win cross1 $coord1x $coord1y
          nv_win cross2 $coord2x $coord2y
          nv_win expand
   }
     nv_win active $name_win }

#
# Press 'ctrl-z' to apply the z coordinates of the active spectrum
# to several spectra at once.
# works only with spectra displayed in a same window
#
bind Spectrum <Control-z> {
     set base_win [nv_win act]
     set plane_z [concat [nv_win pt z]]
     set tl [winfo toplevel $base_win]
     set children [info commands $tl.*]
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
        nv_win pt z [lindex $plane_z 0] [lindex $plane_z 1]
        NvSpectrumDraw
     }
     nv_win active $base_win
}

     
# Press 'ctrl-p' to go to previous views in several spectra at once 
# works only with spectra displayed in columns in a same window 
# Author: Vincent / David 
bind Spectrum <Control-p> {
     set name_win [nv_win act]
     set tl [winfo toplevel $name_win]
     set children [info commands $tl.*]
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
        nv_win previous
   }
     nv_win active $name_win }

# Press 'ctrl-f' to fully expand several spectra at once 
# works only with spectra displayed in columns in a same window 
# Author: Vincent 
bind Spectrum <Control-f> {
     set name_win [nv_win act]
     set tl [winfo toplevel $name_win]
     set children [info commands $tl.*]
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
        nv_win full
     }
     nv_win active $name_win } 
#
# 
# Press 'ctrl-up' to move up one plane in the third dimension 
# in several spectra at once 
# works only with spectra displayed in a same window 
# Author: Vincent
bind Spectrum <Control-Up> {
     set base_win [nv_win act]
     set tl [winfo toplevel $base_win]
     set children [info commands $tl.*]
     set d1 z
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
        Nvincr $child $d1 1
     }
     nv_win active $base_win 
} 

# 
# 
# Press 'ctrl-down' to move down one plane in the third dimension 
# in several spectra at once 
# works only with spectra displayed in a same window 
# Author: Vincent 
bind Spectrum <Control-Down> {
     set base_win [nv_win act]
     set tl [winfo toplevel $base_win]
     set children [info commands $tl.*]
     set d1 z
     foreach child $children {
        set error [catch "winfo class $child" class]
        if {$error} {continue}
        if {$class != "Spectrum" } {continue}
        nv_win active $child
        Nvincr $child $d1 -1
     }
     nv_win active $base_win 
} 


#Find symetric peaks
# Author: Pascal Mercier
bind Spectrum <t> {
Nv_ShowSymmetryWindow
}


#find the closest peak
# Author: Pascal Mercier
bind Spectrum <c> {
ClosestPeak
}


#
#
# Press z on a peak in an hsqc to display automatically the right 3D plane
# corresponding to the 15N chemical shift of the peak in all spectra
# Author: Pascal Mercier
bind Spectrum <z> {
     set name_win [nv_win act]
     set coord1x [nv_win cross1x]
     set coord1y [nv_win cross1y]
     set coord2y [nv_win cross2y]
     set coord2x [nv_win cross2x]
     set nameallwindows [nv_win open]
     set pat "\.peakId*"
     
     foreach item $nameallwindows {
     	#make sure it does not do the peakId window
	if {[string match $pat $item]==0} {
		#now select only the windows which have their 3rd dimension named 15N or N or N15
		 nv_win active $item
		 set activedataset [nv_win dataset]
		 if {[nv_dataset ndim $activedataset]>2} {
			 set namethirddim [nv_dataset label [nv_win dataset] 3]
				if {$namethirddim=="N" || $namethirddim=="15N" || $namethirddim=="N15"} {
		 		nv_win z $coord1y
		 		nv_win draw
		 		nv_win full
		 		nv_win cross1 $coord1x  $coord1x
				}
		}
     	}
     }
     nv_win active $name_win
}

#############################
#########  4D SECTION  ######
# By Olivier Julien, Nov. 2005
# Navigation in the 4th dimension
# By holding Alt + up/down
bind Spectrum <Alt-Up> {
        NvincrView win z2 1
        NvSpectrumDraw
}

bind Spectrum <Alt-Down> {
        NvincrView win z2 -1
        NvSpectrumDraw
}



