# script history: part of Merck distribution
# edited c. hastings to add DYANA interface 11/99
# edited 1/00 c hastings, to fix problems with AutoCalBin
# and CalSimbin when the NOE has negative intensity and volume
# included readAriaList from 4.1.12 base release

# edited 5/00 c hastings, to conform to namespace standards
# and avoid using global variables

# edited 6/00 c hastings, to fix update XPLOR to CNS and fix
# some problems with pseudoatoms

# Edited 6/00 c hastings, to fix bug with median calibration

# Extensively edited 12/00 c hastings - changed user interface,
# added ability to interactively edit NOE list using the panel,
# fixed many bugs, and changed output formats.
# Edited 2/01 a bonvin, to fix bugs the CNS export for the names
# of some atoms

# Edited 8/01 chastings, to fix problem with DYANA export when
# the first residue is not numbered 1.

# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved


##############################################################################################################
# fixed by Curt Hastings (privileged communication)
#  * now handles correctly stereospecific assignments of Me groups and Hs
#  known problem: will output a restraint between two protons attached to the same carbon as
#  assign ((resid x and name HB1) or (resid x and name HB2)) ((resid x and name HB1) or (resid x and name HB2)) 
#  if HB1 and HB2 are not stereospecifically assigned
#
#
# includes Write to Aria Assigned and Ambiguous (Aalim Weljie, Univeristy of Calgary)
# includes "Inactivate Redundant" by keeping the looser restraint (Pascal Mercier, University of Alberta)
# april 13 2003: now eliminates redundancy taking stereospecificy code into account
###############################################################################################################

namespace eval Nv::Noe {
    variable Noe
    variable NoeCal
}


proc Nv::Noe::ShowAnal {args} {
    global Nv_Priv
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(noeAnal)
    }
    if {$narg>1} {
        set listName [lindex $args 1]
        set listIdx [lsearch [nv_peak lists] $listName]
        if {$listIdx >= 0} {
            set curr_list $listIdx
            set curr_peak 0
        }
    } else {
        set nm $Nv_Priv(noeAnal)
    }
    if {![winfo exists $nm]} {
        Nv::Noe::MakeNoePanel $nm
    } else {
        wm deiconify $nm
        raise $nm
    }
    Nv::Noe::UpdateNoePanel $nm
}


# Begin private methods

proc Nv::Noe::MakeNoePanel {nm} {
    global Nv_Priv
    variable Noe
    variable NoeCal
    
    if {[string range $nm 0 0]!="."} {
        errmsg "Window must start with ."
        return
    }
    
    toplevel $nm
    
    frame $nm.mbar -relief raised -bd 2
    frame $nm.fr1 -border 0 -relief flat
    menubutton $nm.mbar.move -text Move -underline 0 -menu $nm.mbar.move.menu
    add_tip $nm.mbar.move peakAnal.move "Move Menu:\n\n First: Jump to first peak in list.\n\n Last: Jump to last peak in list."
    menubutton $nm.mbar.edit -text Calibrate -underline 0 -menu $nm.mbar.edit.menu
    pack $nm.mbar.move $nm.mbar.edit -side left
    menu $nm.mbar.move.menu -tearoff 0
    $nm.mbar.move.menu add command -label First -command "
    set Nv::Noe::Noe(iNoe) 0
    Nv::Noe::UpdateNoePanel $nm
    "
    $nm.mbar.move.menu add command -label Last -command "
    set Nv::Noe::Noe(iNoe) 100000
    Nv::Noe::UpdateNoePanel $nm
    "
    menu $nm.mbar.edit.menu -tearoff 0
    $nm.mbar.edit.menu add command -label "Fixed Value" -command "Nv::Noe::MakeFixedCal $nm"
    $nm.mbar.edit.menu add command -label "Bin Noes" -command "Nv::Noe::MakeBinCal $nm"
    $nm.mbar.edit.menu add command -label "Median Method" -command "Nv::Noe::MakeMedianCal $nm"
    $nm.mbar.edit.menu add command -label "Structure Method" -command "Nv::Noe::MakeStructCal $nm"
    button $nm.mbar.prev -bitmap @$Nv_Priv(image,down) -width 20 -command "
    incr Nv::Noe::Noe(iNoe) -1
    Nv::Noe::UpdateNoePanel $nm
    "
    add_tip $nm.mbar.prev peakAnal.prev "Click this button to go to the previous peak in list."
    button $nm.mbar.next -bitmap @$Nv_Priv(image,up) -width 20 -command "
    incr Nv::Noe::Noe(iNoe)
    Nv::Noe::UpdateNoePanel $nm
    "
    add_tip $nm.mbar.next peakAnal.next "Click this button to go to the next peak in list."
    entry $nm.mbar.peak -width 6 -relief sunken -textvariable Nv::Noe::Noe(iNoe)
    bind $nm.mbar.peak <Return> "Nv::Noe::UpdateNoePanel $nm"
    add_tip $nm.mbar.peak peakAnal.peak "The current peak.\n\n  Type in a value and hit the Enter key to change it."
    button $nm.mbar.delstatus -bitmap @$Nv_Priv(image,pirate) -width 20 -command "Nv::Noe::FlipStatus $nm"
    add_tip $nm.mbar.delstatus peakAnal.delstatus "Click this button to delete or undelete the current peak.  The button will be red if the status of the peak is set to DELETE."
    button $nm.mbar.locked -bitmap @$Nv_Priv(image,unlock) -width 20 -command "Nv::Noe::FlipLock $nm"
    label $nm.mbar.peaklab -width 12 -relief sunken -textvariable Nv::Noe::Noe(peak)
    pack $nm.mbar.peak $nm.mbar.prev $nm.mbar.next $nm.mbar.delstatus $nm.mbar.locked -side left
    pack $nm.mbar.peaklab -side right
    pack $nm.mbar -fill x -side top
    set fr $nm.pulldowns1
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x
    set plists [nv_peak list]
    label $fr.lab1 -text "Peaklist"
    menubutton $fr.list -text none -menu $fr.list.menu -relief raised -textvariable Nv::Noe::NoeCal(curr_list)
    #menubutton $fr.list -text none -menu $fr.list.menu -relief raised
    pack $fr.lab1 $fr.list -side left
    set menu $fr.list.menu
    menu $menu -postcommand "NvFillPeakMenu $menu Nv::Noe::NoeCal(curr_list) {} 0" -tearoff 0
    button $fr.extract -text "Extract" -command "
    Nv::Noe::Extract
    Nv::Noe::UpdateNoePanel $nm
    " -width 9
    
    button $fr.extractall -text "Extract All" -command "
    Nv::Noe::Extract -all
    Nv::Noe::UpdateNoePanel $nm
    " -width 9
    
    pack $fr.extractall $fr.extract -side right
    set fr $nm.buttons1
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x
    label $fr.lab1 -text "Scaling Factor:"
    entry $fr.ent1 -textvariable Nv::Noe::NoeCal(scalevalue) -width 4
    button $fr.but1 -text "Scale" -command "
    Nv::Noe::ScaleNoes
    Nv::Noe::UpdateNoePanel $nm
    " -width 9
    pack $fr.lab1 $fr.ent1 -side left
    pack $fr.but1 -side right
    
    set fr $nm.pulldowns2
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x
    label $fr.lab1 -text Use
    tk_optionMenu $fr.menu1 Nv::Noe::NoeCal(cal_source) intensity volume
    #    $fr.menu1 config -width 10
    label $fr.lab2 -text "to calibrate NOEs"
    button $fr.reset -text "Reset" -command "
    Nv::Noe::Reset
    Nv::Noe::UpdateNoePanel $nm
    " -width 9
    pack $fr.lab1 $fr.menu1 $fr.lab2 -side left -fill x
    pack $fr.reset -side right
    
    set fr $nm.pulldowns3
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x
    label $fr.lab2 -text "Output Format"
    tk_optionMenu $fr.menu2 Nv::Noe::NoeCal(output_type) "CNS Assigned" "CNS Ambiguous" "CNS All" "ARIA Unassigned" "DYANA"  "ARIA Assigned" "ARIA Ambiguous"
    
    #    $fr.menu2 config -width 18
    
    button $fr.write -text "Write" -command "Nv::Noe::WriteRestraints" -width 9
    pack $fr.lab2 $fr.menu2 -side left -fill x
    pack $fr.write -side right
    
    set fr $nm.buttons2
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x -expand yes
    button $fr.clear -text "Clear NOEs" -command "
    nv_noe clear
    Nv::Noe::UpdateNoePanel $nm
    "
    
    button $fr.check -text "Inactivate Meaningless" -command "
    nv_noe check
    puts {}
    Nv::Noe::ReactivateAmbiguous
    Nv::Noe::UpdateNoePanel $nm
    " -width 17
    
    
    
   
   # changed by Pascal , Oct 24 2001
   button $fr.uniq -text "Inactivate Redundant pm"  -width 17 -command {uniqpm}
  #Nv::Noe::UpdateNoePanel $nm
   pack $fr.clear $fr.check $fr.uniq -side left -fill x -expand yes
   
   # button $fr.uniq -text "Inactivate Redundant" -command "
   # nv_noe uniq
   # puts {}
   # Nv::Noe::ReactivateAmbiguous
   # Nv::Noe::UpdateNoePanel $nm
   # " -width 17
   # pack $fr.clear $fr.check $fr.uniq -side left -fill x -expand yes
    
    set fr $nm.buttons4
    frame $fr -border 2 -relief sunken
    pack $fr -side top -fill x -expand yes
    button $fr.plot -text "Plot" -command "
    Nv::Noe::Plot xyNOE
    NvGraphDisplay xyNOE .xyNOE
    "
    button $fr.close -text "Close" -command "wm withdraw $nm"
    pack $fr.plot $fr.close -side left -fill x -expand yes
    set fr $nm.fr1
    pack $fr -fill both -side top
    set rowNames {{Intensity Volume} {Scale Class} {Lower Upper }\
        {Min r^6 Max} {Pseudo0 Pseudo1} {Stereo0 Stereo1}\
    {Atom0 Atom1} }
    set rowTypes {{ival vval} {scale iclass} {lower upper} {minDistance r6Distance maxDistance}\
    {pseudo0 pseudo1} {stereo0 stereo1} {atom0 atom1} }
    set i 0
    
    set settableTypes "ival vval scale iclass lower upper pseudo0 pseudo1 stereo0 stereo1"
    
    foreach rowSet $rowNames typeSet $rowTypes {
        set j 0
        foreach row $rowSet type $typeSet {
            if {[llength $rowSet]==1} {
                set nCols 2
            } else {
                set nCols 1
            }
            label $fr.lab$row -text $row -width 8
            grid columnconfigure $fr $j -weight 100
            grid rowconfigure $fr $i -weight 100
            grid $fr.lab$row -in $fr -column $j -row $i -sticky nsew
            incr j
            entry $fr.entry$row -textvariable Nv::Noe::Noe($type) -width 8
            
            if {[lsearch $settableTypes $type] >= 0} {
                bind $fr.entry$row <KeyPress> "$nm.mbar.peaklab configure -bg yellow "
            }
            
            bind $fr.entry$row <Return> "Nv::Noe::GetNoeAnal $nm"
            
            grid columnconfigure $fr 1 -weight 100
            grid $fr.entry$row -in $fr -column $j -row $i -sticky nsew
            incr j
        }
        incr i
    }
    set NoeCal(fixCal,lower) 1.8
    set NoeCal(fixCal,upper) 6.0
    set NoeCal(medCal,R) 2.7
    set NoeCal(medCal,exp) 6
    set NoeCal(disCal,exp) 6
    set NoeCal(weakBou) 5.
    set NoeCal(mediumBou) 3.4
    set NoeCal(strongBou) 2.8
    set NoeCal(lowerInt) 30.0
    set NoeCal(upperInt) 100.0
    set NoeCal(fError) 0.125
    set NoeCal(methyl) 2.0
    set NoeCal(calAll) 0
    foreach iclass "0 1 2 3 4" {
        set NoeCal(class,$iclass) 1
    }
}

proc Nv::Noe::Extract {{mode ""}} {
    variable NoeCal
    # get number of NOE's already in the list
    # we need to convert peak ordinals to peak idnums
    nv_noe extract $NoeCal(curr_list) $mode
    
    # set the stereo fields correctly - not set by extract
    Nv::Noe::FixStereoFields
}

proc Nv::Noe::FixStereoFields {} {
    variable NoeCal
    set total [nv_noe n]
    set lists [nv_peak list]
    set listIndex [lsearch $lists $NoeCal(curr_list)]
    for {set i 0} {$i < $total} {incr i} {
        set list [nv_noe elem list $i]
        if {$list != $listIndex} {
            continue
        }
        set number [nv_noe elem number $i]
        
        set atompair [GetAtomPairs $list $number]
        set atom1 [lindex $atompair 0]
        set atom2 [lindex $atompair 2]
        
        if {![catch "nv_atom elem stereo $atom1" result]} {
            if {$result == 2 || $result == 9} {
                nv_noe elem stereo.0 $i 1
            }
        }
        if {![catch "nv_atom elem stereo $atom2" result]} {
            if {$result == 2 || $result == 9} {
                nv_noe elem stereo.1 $i 1
            }
        }
    }
}


proc Nv::Noe::ScaleNoes {} {
    variable NoeCal
    set max [nv_noe n]
    for {set i 0} {$i < $max} {incr i} {
        if {[nv_noe elem active $i] == 1} {
            nv_noe elem scale $i $NoeCal(scalevalue)
        }
    }
}

proc Nv::Noe::WriteRestraints {} {
    variable NoeCal
    switch $NoeCal(output_type) {
        "CNS Assigned" {
            Nv::Noe::CNS asn
        }
        "CNS Ambiguous" {
            Nv::Noe::CNS amb
        }
        "CNS All" {
            Nv::Noe::CNS all
        }
        
        "ARIA Unassigned" {
            Nv::Noe::Aria 
        }
         "ARIA Assigned" {
             Nv::Noe::ARIA asn
        }
         "ARIA Ambiguous" {
             Nv::Noe::ARIA amb
        }
        DYANA {
            Nv::Noe::Dyana
        }
    }
}

proc Nv::Noe::MakeBinCal {nm} {
    variable NoeCal
    
    set calnm $nm
    append calnm "-fixedCal"
    
    destroy $calnm
    toplevel $calnm
    
    set fr $calnm.fr1
    
    frame $fr -border 0 -relief raised -bd 2
    pack $fr -fill both -side top -fill x -expand yes
    frame $fr.row1
    pack $fr.row1 -side top -fill x -expand yes
    label $fr.row1.lab -text "Intensities above "
    pack $fr.row1.lab -side left
    entry $fr.row1.upperInt -width 7 -textvariable Nv::Noe::NoeCal(upperInt)
    pack $fr.row1.upperInt -side left
    label $fr.row1.lab2 -text "will have an upper bound of "
    pack $fr.row1.lab2 -side left
    entry $fr.row1.upperBou -width 5 -textvariable Nv::Noe::NoeCal(strongBou)
    pack $fr.row1.upperBou -side left
    frame $fr.row2
    pack $fr.row2 -side top -fill x -expand yes
    label $fr.row2.lab -text "while intensities between "
    pack $fr.row2.lab -side left
    entry $fr.row2.lowerInt -width 7 -textvariable Nv::Noe::NoeCal(lowerInt)
    pack $fr.row2.lowerInt -side left
    label $fr.row2.lab2 -text "and"
    pack $fr.row2.lab2 -side left
    entry $fr.row2.upperInt -width 7 -textvariable Nv::Noe::NoeCal(upperInt)
    pack $fr.row2.upperInt -side left
    label $fr.row2.lab3 -text "will have"
    pack $fr.row2.lab3 -side left
    frame $fr.row3
    pack $fr.row3 -side top -fill x -expand yes
    label $fr.row3.lab1 -text "an upper bound of"
    pack $fr.row3.lab1 -side left
    entry $fr.row3.upperBou -width 5 -textvariable Nv::Noe::NoeCal(mediumBou)
    pack $fr.row3.upperBou -side left
    label $fr.row3.lab2 -text "and intensities below "
    pack $fr.row3.lab2 -side left
    frame $fr.row4
    pack $fr.row4 -side top -fill x -expand yes
    entry $fr.row4.lowerInt -width 7 -textvariable Nv::Noe::NoeCal(lowerInt)
    pack $fr.row4.lowerInt -side left
    label $fr.row4.lab1 -text "will have an upper bound of"
    pack $fr.row4.lab1 -side left
    entry $fr.row4.upperBou -width 5 -textvariable Nv::Noe::NoeCal(weakBou)
    pack $fr.row4.upperBou -side left
    frame $fr.row5
    pack $fr.row5 -side top -fill x -expand yes
    label $fr.row5.lab1 -text "Divide methyl intensities by "
    pack $fr.row5.lab1 -side left
    entry $fr.row5.methyl -width 5 -textvariable Nv::Noe::NoeCal(methyl)
    pack $fr.row5.methyl -side left
    
    frame $fr.row6
    pack $fr.row6 -side top -fill x -expand yes
    label $fr.row6.lab -text "Set all lower bounds to"
    pack $fr.row6.lab -side left
    entry $fr.row6.fixBound -width 3 -textvariable Nv::Noe::NoeCal(fixCal,lower)
    pack $fr.row6.fixBound -side left
    
    
    set fr $calnm.fr3
    frame $fr -border 0 -relief flat
    pack $fr -fill both -side top -fill x -expand yes
    button $fr.reset -text Reset -command "
    Nv::Noe::Reset
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.guess -text "Guess Bins" -command "
    Nv::Noe::AutoCalBin
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.calibrate -text Calibrate -command "
    Nv::Noe::CalSimbin
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.close -text Close -command "
    destroy $calnm
    "
    pack $fr.reset $fr.guess $fr.calibrate $fr.close -side left -fill x -expand yes
}

proc Nv::Noe::MakeFixedCal {nm} {
    variable NoeCal
    
    set calnm $nm
    append calnm "-fixedCal"
    
    destroy $calnm
    toplevel $calnm
    
    set fr $calnm.fr1
    frame $fr -border 0 -relief raised -bd 2
    pack $fr -fill both -side top
    
    frame $fr.row1
    pack $fr.row1 -side top
    label $fr.row1.lab -text "Set all upper bounds to"
    pack $fr.row1.lab -side left
    entry $fr.row1.fixBound -width 3 -textvariable Nv::Noe::NoeCal(fixCal,upper)
    pack $fr.row1.fixBound -side left
    frame $fr.row2
    pack $fr.row2 -side top
    label $fr.row2.lab -text "Set all lower bounds to"
    pack $fr.row2.lab -side left
    entry $fr.row2.fixBound -width 3 -textvariable Nv::Noe::NoeCal(fixCal,lower)
    pack $fr.row2.fixBound -side left
    set fr $calnm.fr4
    frame $fr -border 0 -relief flat
    pack $fr -fill both -side top -expand yes
    button $fr.reset -text Reset -command "
    Nv::Noe::Reset
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.calibrate -text Calibrate -command "
    Nv::Noe::AutoCal0
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.close -text Close -command "
    destroy $calnm
    "
    pack $fr.reset $fr.calibrate $fr.close -side left -expand yes -fill x
}

proc Nv::Noe::MakeMedianCal {nm} {
    variable NoeCal
    
    set calnm $nm
    append calnm "-fixedCal"
    
    destroy $calnm
    toplevel $calnm
    
    set fr $calnm.fr1
    
    frame $fr -border 0 -relief raised -bd 2
    pack $fr -fill both -side top -fill x -expand yes
    frame $fr.row1
    pack $fr.row1 -side top -fill x -expand yes
    label $fr.row1.lab -text "Calibrate distances using an r^"
    pack $fr.row1.lab -side left
    entry $fr.row1.medN -width 2 -textvariable Nv::Noe::NoeCal(medCal,exp)
    pack $fr.row1.medN -side left
    label $fr.row1.lab2 -text " function "
    pack $fr.row1.lab2 -side left
    frame $fr.row2
    pack $fr.row2 -side top -fill x -expand yes
    label $fr.row2.lab -text "where the median Noe intensity corresponds to "
    pack $fr.row2.lab -side left
    frame $fr.row3
    pack $fr.row3 -side top -fill x -expand yes
    label $fr.row3.lab1 -text "a distance of"
    pack $fr.row3.lab1 -side left
    entry $fr.row3.medR -width 4 -textvariable Nv::Noe::NoeCal(medCal,R)
    pack $fr.row3.medR -side left
    frame $fr.row4
    pack $fr.row4 -side top -fill x -expand yes
    label $fr.row4.lab1 -text "and add a tolerance of"
    pack $fr.row4.lab1 -side left
    entry $fr.row4.upperBou -width 4 -textvariable Nv::Noe::NoeCal(fError)
    pack $fr.row4.upperBou -side left
    label $fr.row4.lab2 -text "times the bound squared"
    pack $fr.row4.lab2 -side left
    frame $fr.row5
    pack $fr.row5 -side top -fill x -expand yes
    label $fr.row5.lab1 -text "Divide methyl intensities by "
    pack $fr.row5.lab1 -side left
    entry $fr.row5.methyl -width 5 -textvariable Nv::Noe::NoeCal(methyl)
    pack $fr.row5.methyl -side left
    
    frame $fr.row6
    pack $fr.row6 -side top -fill x -expand yes
    label $fr.row6.lab -text "Set all lower bounds to"
    pack $fr.row6.lab -side left
    entry $fr.row6.fixBound -width 3 -textvariable Nv::Noe::NoeCal(fixCal,lower)
    pack $fr.row6.fixBound -side left
    
    set fr $calnm.fr3
    frame $fr -border 0 -relief flat
    pack $fr -fill both -side top
    button $fr.reset -text Reset -command "
    Nv::Noe::Reset
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.calibrate -text Calibrate -command "
    Nv::Noe::AutoCal1
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.close -text Close -command "
    destroy $calnm
    "
    pack $fr.reset $fr.calibrate $fr.close -side left -fill x -expand yes
}

proc Nv::Noe::MakeStructCal {nm} {
    variable NoeCal
    set calnm $nm
    append calnm "-fixedCal"
    
    destroy $calnm
    toplevel $calnm
    
    set fr $calnm.fr1
    
    frame $fr -border 0 -relief raised -bd 2
    pack $fr -fill both -side top -fill x -expand yes
    frame $fr.row1
    pack $fr.row1 -side top -fill x -expand yes
    label $fr.row1.lab -text "Calibrate distances using an r^"
    pack $fr.row1.lab -side left
    entry $fr.row1.disN -width 2 -textvariable Nv::Noe::NoeCal(disCal,exp)
    pack $fr.row1.disN -side left
    label $fr.row1.lab2 -text " function "
    pack $fr.row1.lab2 -side left
    frame $fr.row2
    pack $fr.row2 -side top -fill x -expand yes
    label $fr.row2.lab -text "where the function is calibrated from the "
    pack $fr.row2.lab -side left
    frame $fr.row2a
    pack $fr.row2a -side top -fill x -expand yes
    label $fr.row2a.lab -text "distance:intensity relation in the current structures"
    pack $fr.row2a.lab -side left
    frame $fr.row4
    pack $fr.row4 -side top -fill x -expand yes
    label $fr.row4.lab1 -text "and add a tolerance of"
    pack $fr.row4.lab1 -side left
    entry $fr.row4.upperBou -width 4 -textvariable Nv::Noe::NoeCal(fError)
    pack $fr.row4.upperBou -side left
    label $fr.row4.lab2 -text "times the bound squared"
    pack $fr.row4.lab2 -side left
    frame $fr.row5
    pack $fr.row5 -side top -fill x -expand yes
    label $fr.row5.lab1 -text "Divide methyl intensities by "
    pack $fr.row5.lab1 -side left
    entry $fr.row5.methyl -width 5 -textvariable Nv::Noe::NoeCal(methyl)
    pack $fr.row5.methyl -side left
    frame $fr.row6
    pack $fr.row6 -side top
    label $fr.row6.lab1 -text "Calibrate classes:"
    pack $fr.row6.lab1 -side left
    frame $fr.row7
    pack $fr.row7 -side top
    foreach iclass "0 1 2 3 4" {
        checkbutton $fr.row7.cb$iclass -text $iclass -variable Nv::Noe::NoeCal(class,$iclass) -onvalue 1 -offvalue 0
        pack $fr.row7.cb$iclass -side left
    }
    
    frame $fr.row8
    pack $fr.row8 -side top -fill x -expand yes
    label $fr.row8.lab -text "Set all lower bounds to"
    pack $fr.row8.lab -side left
    entry $fr.row8.fixBound -width 3 -textvariable Nv::Noe::NoeCal(fixCal,lower)
    pack $fr.row8.fixBound -side left
    
    set fr $calnm.fr3
    frame $fr -border 0 -relief flat
    pack $fr -fill both -side top
    button $fr.reset -text Reset -command "
    Nv::Noe::Reset
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.calibrate -text Calibrate -command "
    Nv::Noe::AutoCal2
    Nv::Noe::UpdateNoePanel $nm
    "
    button $fr.close -text Close -command "
    destroy $calnm
    "
    
    pack $fr.reset $fr.calibrate $fr.close -side left -fill x -expand yes
}

proc Nv::Noe::FlipStatus {nm} {
    variable Noe
    
    set status [nv_noe elem active $Nv::Noe::Noe(iNoe)]
    if {$status>0} {
        nv_noe elem active $Nv::Noe::Noe(iNoe) 0
        $nm.mbar.delstatus config -fg red
        $nm.mbar.delstatus config -highlightcolor red
    } else {
        nv_noe elem active $Nv::Noe::Noe(iNoe) 1
        $nm.mbar.delstatus config -fg black
        $nm.mbar.delstatus config -highlightcolor black
    }
}

proc Nv::Noe::FlipLock {nm} {
    global Nv_Priv
    variable Noe
    set status [nv_noe elem active $Nv::Noe::Noe(iNoe)]
    if {$status==2} {
        nv_noe elem active $Nv::Noe::Noe(iNoe) 1
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,unlock)
    } elseif {$status==1} {
        nv_noe elem active $Nv::Noe::Noe(iNoe) 2
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,lock)
    }
}

proc Nv::Noe::UpdateNoePanel args {
    global Nv_Priv
    variable Noe
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(noeAnal)
    }
    wm title $nm NoeAnalysis
    set nNoes [nv_noe n]
    if {$Noe(iNoe)>=$nNoes} {
        set Noe(iNoe) [expr $nNoes-1]
    }
    if {$Noe(iNoe)<0} {
        set Noe(iNoe) 0
    }
    if {$nNoes == 0} {
        set Noe(active) ""
        set Noe(ival) ""
        set Noe(scale) ""
        set Noe(iclass) ""
        set Noe(upper) ""
        set Noe(lower) ""
        set Noe(pseudo0) ""
        set Noe(pseudo1) ""
        set Noe(stereo0) ""
        set Noe(stereo1) ""
        set Noe(res0) ""
        set Noe(res1) ""
        set Noe(aname0) ""
        set Noe(aname1) ""
        set Noe(peak) ""
        set Noe(minDistance) ""
        set Noe(r6Distance) ""
        set Noe(maxDistance) ""
        set Noe(atom0) ""
        set Noe(atom1) ""
        return
    }
    set i $Noe(iNoe)
    set Noe(active) [string trim [nv_noe elem active $i]]
    set Noe(ival) [string trim [nv_noe elem int $i]]
    set Noe(vval) [string trim [nv_noe elem vol $i]]
    set Noe(scale) [string trim [nv_noe elem scale $i]]
    set Noe(iclass) [string trim [nv_noe elem class $i]]
    set Noe(upper) [string trim [nv_noe elem upper $i]]
    set Noe(lower) [string trim [nv_noe elem lower $i]]
    if {!$Nv_Priv(javaMode)} {
        set Noe(pseudo0) [string trim [nv_noe elem pseudo.0 $i]]
        set Noe(pseudo1) [string trim [nv_noe elem pseudo.1 $i]]
        set Noe(stereo0) [string trim [nv_noe elem stereo.0 $i]]
        set Noe(stereo1) [string trim [nv_noe elem stereo.1 $i]]
        set Noe(res0) [string trim [nv_noe elem res.0 $i]]
        set Noe(res1) [string trim [nv_noe elem res.1 $i]]
        set Noe(aname0) [string trim [nv_noe elem aname.0 $i]]
        set Noe(aname1) [string trim [nv_noe elem aname.1 $i]]
    }
    set Noe(peak) [string trim [lindex [nv_peak list] [nv_noe elem list $i]].[nv_noe elem number $i]]
    set Noe(peakNum) [string trim [nv_noe elem number $i]]
    set Noe(peakList) [string trim [nv_noe elem list $i]]
    set distances [GetDistance [nv_noe elem list $i] [nv_noe elem number $i]]
    set Noe(minDistance) [string trim [lindex $distances 0]]
    set Noe(r6Distance) [string trim [lindex $distances 1]]
    set Noe(maxDistance) [string trim [lindex $distances 2]]
    if {$Noe(active)>0} {
        $nm.mbar.delstatus config -fg black
        $nm.mbar.delstatus config -highlightcolor black
    } else {
        $nm.mbar.delstatus config -fg red
        $nm.mbar.delstatus config -highlightcolor red
    }
    set locked [expr 2==[nv_noe elem active $Noe(iNoe)]]
    if {$locked} {
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,lock)
    } else {
        $nm.mbar.locked config -bitmap @$Nv_Priv(image,unlock)
    }
    
    set atomPair [GetAtomPairs $Noe(peakList) $Noe(peakNum)]
    set Noe(atom0) [lindex $atomPair 0]
    set Noe(atom1) [lindex $atomPair 2]
    
    if {[llength $Noe(atom0)] > 1} {set Noe(atom0) "ambig"}
    if {[llength $Noe(atom1)] > 1} {set Noe(atom1) "ambig"}
    
    if {[winfo exists $Nv_Priv(peakAnal)]} {
        global curr_list curr_peak
        set curr_list [lindex [nv_peak list] [nv_noe elem list $i]]
        set curr_peak [nv_noe elem number $i]
        NvSetPeakAnal $Nv_Priv(peakAnal)
    }
    
    $nm.mbar.peaklab configure -bg #d9d9d9
    return 0
}


proc Nv::Noe::GetNoeAnal args {
    global Nv_Priv
    variable Noe
    set narg [llength $args]
    if {$narg>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(noeAnal)
    }
    
    set types "intensity volume scale class lower upper pseudo0 pseudo1 stereo0 stereo1"
    set idxs "ival vval scale iclass lower upper pseudo0 pseudo1 stereo0 stereo1"
    
    foreach type $types idx $idxs {
        nv_noe elem $type $Noe(iNoe) $Noe($idx)
    }
    
    $nm.mbar.peaklab configure -bg #d9d9d9
    
}

proc Nv::Noe::Dyana {{fileName {}} {first 0} {last 100000}} {
    global Nv_Prefs
    if {$fileName == {}} {
        if {$Nv_Prefs(usedirs)} {
            set idir $Nv_Prefs(rstdir)
        } else {
            set idir [pwd]
        }
        set fileName [tk_getSaveFile -initialdir $idir -title "DYANA Files"]
        if {$fileName=={}} {
            return
        }
    }
    set fileName [lindex [split $fileName .] 0]
    
    set f1 [open $fileName.upl w]
    set f2 [open $fileName.lol w]
    set n [nv_noe n]
    if {$last >= $n} {
        set last [expr $n-1]
    }
    set plists [nv_peak list]
    
    # added to handle case where first residue is not number 1
    # 8/01 c hastings
    #
    # get the first residue number
    set FirstResidueOffset [expr [nv_atom elem seq 1] - 1]

    for {set i $first} {$i <= $last} {incr i} {
        set active [nv_noe elem active $i]
        if {!$active} {
            continue
        }
        set upper [nv_noe elem upper $i]
        set lower [nv_noe elem lower $i]
        set atomPairs [GetAtomPairs [nv_noe elem list $i] [nv_noe elem number $i]]
        set protons(1) [lindex $atomPairs 0]
        set protons(2) [lindex $atomPairs 2]
        
        if {[llength $protons(1)] > 1} {set protons(1) ambig}
        if {[llength $protons(2)] > 1} {set protons(2) ambig}
        
        # Skip unassigned and ambiguously assigned peaks
        if {($protons(1) == "?") ||($protons(1) == "ambig") ||($protons(2) == "?") ||($protons(2) == "ambig")} {
            continue
        }
        
        
        set resnum(1) [lindex [split $protons(1) .] 0]
        nv_residue [expr $resnum(1) - $FirstResidueOffset] result
        set resname(1) [Nv::Noe::addcharge $result(name)]
        set aname(1) [lindex [split $protons(1) .] 1]
        set aname(1) [NmrviewToDyana $aname(1) $resname(1)]
        set resnum(2) [lindex [split $protons(2) .] 0]
        nv_residue [expr $resnum(2) - $FirstResidueOffset] result
        set resname(2) [Nv::Noe::addcharge $result(name)]
        set aname(2) [lindex [split $protons(2) .] 1]
        set aname(2) [NmrviewToDyana $aname(2) $resname(2)]
        set plistnum [nv_noe elem list $i]
        set plist [lindex $plists $plistnum]
        set pnumber [nv_noe elem number $i]
        puts $f1 "$resnum(1) $resname(1) $aname(1)\t$resnum(2) $resname(2) $aname(2)\t$upper\t# $plist.$pnumber"
        puts $f2 "$resnum(1) $resname(1) $aname(1)\t$resnum(2) $resname(2) $aname(2)\t$lower\t# $plist.$pnumber"
    }
    close $f1
    close $f2
    global Nv_Nresidues
    set f3 [open $fileName.seq w]

    # track writing of first residue number, necessary if the sequence does
    # not start at 1

    if {$FirstResidueOffset} {
        set WriteFirstResidue 1
    } else {
        set WriteFirstResidue 0
    }

    for {set i [expr 1 + $FirstResidueOffset]} {$i <= [expr $Nv_Nresidues + $FirstResidueOffset]} {incr i} {

        nv_residue [expr $i - $FirstResidueOffset] result
        set residue [Nv::Noe::addcharge $result(name)]

        # write the residue number of the first residue, for the first residue only

        if {$WriteFirstResidue} {

            puts $f3 "$residue $i"
 
            set WriteFirstResidue 0

        } else {
            puts $f3 $residue
        }
    }
    close $f3
}

proc Nv::Noe::addcharge {residue} {
    if {($residue == "ASP") ||($residue == "GLU")} {
        append residue -
    }
    if {($residue == "LYS") ||($residue == "ARG") ||($residue == "HIS")} {
        append residue +
    }
    return $residue
}

proc Nv::Noe::Aria {} {
    #get the htol and tol
    if {![info exists Noe(aria_htol)]} {
        set Nv::Noe::Noe(aria_htol) 0.5
        set Nv::Noe::Noe(aria_tol) 0.05
    }
    
    set x [expr [winfo screenwidth .]/2]
    set y [expr [winfo screenheight .]/4]
    
    set nm .tolerances
    toplevel $nm
    wm geometry $nm +$x+$y
    set fr1 $nm.fr1
    frame $fr1
    pack $fr1 -side top -fill x
    set fr2 $nm.fr2
    frame $fr2
    pack $fr2 -side top -fill x
    label $fr1.label -width 35 -text "Proton chemical shift tolerance "
    entry $fr1.entry -width 5 -textvariable Nv::Noe::Noe(aria_tol)
    pack $fr1.label $fr1.entry -side left
    
    label $fr2.label -width 35 -text "Heteronuclear chemical shift tolerance "
    entry $fr2.entry -width 5 -textvariable Nv::Noe::Noe(aria_htol)
    pack $fr2.label $fr2.entry -side left
    
    set fr3 $nm.fr3
    frame $fr3
    pack $fr3 -side top -fill x
    button $fr3.ok -text OK -command "
    destroy $nm
    Nv::Noe::ARIA unasn
    "
    pack $fr3.ok -side left
    
}

proc Nv::Noe::CNS {selection {fileName {}} {first 0} {last 100000}} {
    global Nv_Prefs
    variable Noe

    set Noe(FirstResidueOffset) [expr [nv_atom elem seq 1] - 1]
    
    if {$fileName == {}} {
        if {$Nv_Prefs(usedirs)} {
            set idir $Nv_Prefs(rstdir)
        } else {
            set idir [pwd]
        }
        set fileName [tk_getSaveFile -initialdir $idir -title "CNS Table Files"]
        if {$fileName=={}} {
            return
        }
    }
    
    set fileName [lindex [split $fileName .] 0]
    set fullname $fileName.tbl
    set f1 [open $fullname w]
    set n [nv_noe n]
    if {$last >= $n} {
        set last [expr $n-1]
    }
    set peaklists [nv_peak list]
    for {set i $first} {$i <= $last} {incr i} {
        set active [nv_noe elem active $i]
        if {!$active} {
            continue
        }
        set peak [nv_noe elem number $i]
        set list [nv_noe elem list $i]
        set listname [lindex $peaklists $list]
        set upper [string trim [nv_noe elem upper $i]]
        set lower [nv_noe elem lower $i]
        set stereo0 [nv_noe elem stereo.0 $i]
        set stereo1 [nv_noe elem stereo.1 $i]
        set atomPairs [GetAtomPairs [nv_noe elem list $i] [nv_noe elem number $i]]
        set protons(1) [lindex $atomPairs 0]
        set protons(2) [lindex $atomPairs 2]
        
        # Determine the NOE type - unassigned, ambiguous, or assigned
        
        if {($protons(1) == "?") ||($protons(2) == "?")} {
            set noeType unasn
        } elseif {([llength $protons(1)] > 1) || ([llength $protons(2)] > 1)} {
            set noeType amb
        } else {
            set noeType asn
        }
        
        # Output block for assigned peaks
        if {$noeType == "asn"} {
            
            if {($selection == "amb") || ($selection == "aria")} {continue}
            
            set output [FormatCnsAssign $protons(1) $protons(2) $stereo0 $stereo1 $lower $upper]
            if {$output == -1} {
                continue
            }
            puts -nonewline $f1 "assign "
            puts -nonewline $f1 $output
            puts $f1 " ! \t $listname.$peak"
        }
        
        if {$noeType == "amb"} {
            if {($selection == "asn") || ($selection == "aria")} {continue}
            
            if {[llength $protons(1)] != [llength $protons(2)]} {
                errmsg "Peak $listname.$peak has an uneven number of assignments in proton containing fields"
                continue
            }
            
            set nAssignments [llength $protons(1)]
            puts -nonewline $f1 "assign "
            
            for {set j 0} {$j < $nAssignments} {incr j} {
                set output [FormatCnsAssign [lindex $protons(1) $j] [lindex $protons(2) $j] $stereo0 $stereo1 $lower $upper]
                if {$j == 0} {
                    puts -nonewline $f1 $output
                    puts $f1 " ! \t $listname.$peak"
                } else {
                    puts -nonewline $f1 "or "
                    puts $f1 [lrange $output 0 9]
                }
            }
        }
        
        if {$noeType == "unasn"} {
            if {($selection != "aria")} {continue}
            
            set labels [nv_peak label $listname]
            set relations [nv_peak relation $listname]
            set nParent 0
            foreach label $labels {set ishetero($label) 0}
            
            foreach label $labels relation $relations {
                
                set labelIndex [lsearch $labels $label]
                
                if {[regexp {([Dd])([1234])} $relation all d idx]} {
                    incr idx -1
                    set hetatomlabel($label) [lindex $labels $idx]
                    set ishetero($hetatomlabel($label)) 1
                } else {
                    set hetatomlabel($label) ""
                    incr nParent
                }
            }
            
            if {$nParent != 2} {
                errmsg "Check relations for peaklist $listname.  Number of parent dimensions does not equal 2."
                continue
            }
            
            puts $f1 "assign"
            
            # htol is the heteroatom tolerance
            set tol $Noe(aria_tol)
            set htol $Noe(aria_htol)
            
            set ppms {}
            
            foreach label $labels {
                if {$ishetero($label)} {continue}
                set ppm [nv_peak elem $label.P $listname.$peak]
                lappend ppms $ppm
                set lppm [expr $ppm - $tol]
                set uppm [expr $ppm + $tol]
                
                if {$hetatomlabel($label) == ""} {
                    puts $f1 "( attr store1 < $uppm and attr store1 > $lppm )"
                } else {
                    set hetppm [nv_peak elem $hetatomlabel($label).P $listname.$peak]
                    set lhppm [expr $hetppm - $htol]
                    set uhppm [expr $hetppm + $htol]
                    puts $f1 "(( attr store1 < $uppm and attr store1 > $lppm )"
                    puts $f1 "and bondedto ( attr store1 < $uhppm and attr store1 > $lhppm))"
                }
            }
            
            set mid [expr $upper - $lower]
            #set volume [nv_noe elem volume $i]
     	    #modified by AMW to use intensities
 	    set volume [expr abs([nv_noe elem volume $i])]       
	    set ppm1 [lindex $ppms 0]
            set ppm2 [lindex $ppms 1]
            
            puts $f1 "$upper $mid 0.0 peak $peak volume $volume ppm1 $ppm1 ppm2 $ppm2"
            
        }
    }
    close $f1
}



proc Nv::Noe::ARIA {selection {fileName {}} {first 0} {last 100000}} {
    global Nv_Prefs
    variable Noe

    set Noe(FirstResidueOffset) [expr [nv_atom elem seq 1] - 1]

    if {$fileName == {}} {
        if {$Nv_Prefs(usedirs)} {
            set idir $Nv_Prefs(rstdir)
        } else {
            set idir [pwd]
        }
        set fileName [tk_getSaveFile -initialdir $idir -title "CNS Table Files"]
        if {$fileName=={}} {
            return
        }
    }
    
    set fileName [lindex [split $fileName .] 0]
    set fullname $fileName.tbl
    set f1 [open $fullname w]
    set n [nv_noe n]
    if {$last >= $n} {
        set last [expr $n-1]
    }
    set peaklists [nv_peak list]
    for {set i $first} {$i <= $last} {incr i} {
        set active [nv_noe elem active $i]
        if {!$active} {
            continue
        }
        set peak [nv_noe elem number $i]
        set list [nv_noe elem list $i]
        set listname [lindex $peaklists $list]
        set upper [string trim [nv_noe elem upper $i]]
        set lower [nv_noe elem lower $i]
        set stereo0 [nv_noe elem stereo.0 $i]
        set stereo1 [nv_noe elem stereo.1 $i]
        set atomPairs [GetAtomPairs [nv_noe elem list $i] [nv_noe elem number $i]]
        set protons(1) [lindex $atomPairs 0]
        set protons(2) [lindex $atomPairs 2]
	set label1 [lindex [nv_peak label $listname] 0]
	set label2 [lindex [nv_peak label $listname] 1]

        #added AMW     
        set volume [expr abs([nv_noe elem int $i])]
	set ppm1 [nv_peak element $label1.P $listname.[nv_noe elem number $i]]
	set ppm2 [nv_peak element $label2.P $listname.[nv_noe elem number $i]]
        # Determine the NOE type - unassigned, ambiguous, or assigned
        
        if {($protons(1) == "?") ||($protons(2) == "?")} {
            set noeType unasn
        } elseif {([llength $protons(1)] > 1) || ([llength $protons(2)] > 1)} {
            set noeType amb
        } else {
            set noeType asn
        }
        
        # Output block for assigned peaks
        if {$noeType == "asn"} {
            
            if {($selection == "amb") || ($selection == "aria")} {continue}
            
            set output [FormatCnsAssign $protons(1) $protons(2) $stereo0 $stereo1 $lower $upper]
            if {$output == -1} {
                continue
            }
            puts -nonewline $f1 "assign "
            puts -nonewline $f1 $output
	#modified AMW
            puts $f1 " peak $peak volume $volume ppm1 $ppm1 ppm2 $ppm2 ! \t $listname.$peak"
        }
        
        if {$noeType == "amb"} {
            if {($selection == "asn") || ($selection == "aria")} {continue}
            
            if {[llength $protons(1)] != [llength $protons(2)]} {
                errmsg "Peak $listname.$peak has an uneven number of assignments in proton containing fields"
                continue
            }
            
            set nAssignments [llength $protons(1)]
            puts -nonewline $f1 "assign "
            
            for {set j 0} {$j < $nAssignments} {incr j} {
                set output [FormatCnsAssign [lindex $protons(1) $j] [lindex $protons(2) $j] $stereo0 $stereo1 $lower $upper]
                if {$j == 0} {
                    puts -nonewline $f1 $output
                    puts $f1 " peak $peak volume $volume ppm1 $ppm1 ppm2 $ppm2 "
                } else {
                    puts -nonewline $f1 "or "
                    puts $f1 [lrange $output 0 9]
                }
            }
        }
        
        if {$noeType == "unasn"} {
            if {($selection != "aria")} {continue}
            
            set labels [nv_peak label $listname]
            set relations [nv_peak relation $listname]
            set nParent 0
            foreach label $labels {set ishetero($label) 0}
            
            foreach label $labels relation $relations {
                
                set labelIndex [lsearch $labels $label]
                
                if {[regexp {([Dd])([1234])} $relation all d idx]} {
                    incr idx -1
                    set hetatomlabel($label) [lindex $labels $idx]
                    set ishetero($hetatomlabel($label)) 1
                } else {
                    set hetatomlabel($label) ""
                    incr nParent
                }
            }
            
            if {$nParent != 2} {
                errmsg "Check relations for peaklist $listname.  Number of parent dimensions does not equal 2."
                continue
            }
            
            puts $f1 "assign"
            
            # htol is the heteroatom tolerance
            set tol $Noe(aria_tol)
            set htol $Noe(aria_htol)
            
            set ppms {}
            
            foreach label $labels {
                if {$ishetero($label)} {continue}
                set ppm [nv_peak elem $label.P $listname.$peak]
                lappend ppms $ppm
                set lppm [expr $ppm - $tol]
                set uppm [expr $ppm + $tol]
                
                if {$hetatomlabel($label) == ""} {
                    puts $f1 "( attr store1 < $uppm and attr store1 > $lppm )"
                } else {
                    set hetppm [nv_peak elem $hetatomlabel($label).P $listname.$peak]
                    set lhppm [expr $hetppm - $htol]
                    set uhppm [expr $hetppm + $htol]
                    puts $f1 "(( attr store1 < $uppm and attr store1 > $lppm )"
                    puts $f1 "and bondedto ( attr store1 < $uhppm and attr store1 > $lhppm))"
                }
            }
            
            set mid [expr $upper - $lower]
            set volume [expr abs{[nv_noe elem int $i]}]
            set ppm1 [lindex $ppms 0]
            set ppm2 [lindex $ppms 1]
            
            puts $f1 "$upper $mid 0.0 peak $peak volume $volume ppm1 $ppm1 ppm2 $ppm2"
            
        }
    }
    close $f1
}



proc Nv::Noe::FormatCnsAssign {atom0 atom1 stereo0 stereo1 lower upper} {

    variable Noe

    set i 0
    set result1 "("
    if {![regexp {(.*)\.([hHqQ].*)} $atom0 all res0 aname0]} {
        error "problem with $atom1"
    }
    
    if {[regsub QQ $aname0 H newname]} {
        set aname0 $newname
        append aname0 ##
    } elseif {[regsub Q $aname0 H newname]} {
        # check for stereoambiguity, which indicates
        # that the methyl group in question has intraresidue ambiguity.
        # If so, insert the wild card flag as appropriate.
        if {[regexp {Q*[0-9]} $aname0]} {
            regsub Q $atom0 M methyl0
            if {![catch "nv_atom elem stereo $methyl0" result]} {
                nv_residue [expr $res0 - $Noe(FirstResidueOffset)] resinfo
                if {(($resinfo(name) != "ILE") || ($aname0 != "QG2")) && ($result == 4)} {
                    set aname0 [string range $newname 0 [expr [string length $newname] - 2] ]
                    append aname0 ##
                } else {
                    set aname0 $newname
                    append aname0 #
                }
            }
        } else {
            set aname0 $newname
            append aname0 #
        }

    # handle case where nv_noe elem stereoX is 1 by converting
    # Hx[1/2] into Hx#

    } elseif {$stereo0  == 1} {
        # ambiguity code 1 indicates Ambiguity of geminal atoms or geminal methyl
        # proton groups
        set aname0 [string range $aname0 0 [expr [string length $aname0] - 2] ]
        append aname0 #
    }
    
    if {[string first "resid $res0 and name $aname0" $result1] == -1} {
        if {$i > 0} {
            append result1 " or "
        }
        append result1 "resid $res0 and name $aname0"
        incr i
    }
    
    append result1 ")"
    set i 0
    set result2 "("
    
    if {![regexp {(.*)\.([hHqQ].*)} $atom1 all res1 aname1]} {
        error "problem with $atom2"
    }
    
    if {[regsub QQ $aname1 H newname]} {
        set aname1 $newname
        append aname1 ##
    } elseif {[regsub Q $aname1 H newname]} {
        # check for stereoambiguity, which indicates
        # that the methyl group in question has intraresidue ambiguity.
        # If so, insert the wild card flag as appropriate.
        if {[regexp {Q*[0-9]} $aname1]} {
            regsub Q $atom1 M methyl1
            if {![catch "nv_atom elem stereo $methyl1" result]} {
                nv_residue [expr $res1 - $Noe(FirstResidueOffset)] resinfo
                if {(($resinfo(name) != "ILE") || ($aname0 != "QG2")) && ($result == 4)} {
                    set aname1 [string range $newname 0 [expr [string length $newname] - 2] ]
                    append aname1 ##
                } else {
                    set aname1 $newname
                    append aname1 #
                }
            }
        } else {

            set aname1 $newname
            append aname1 #

        }

    # handle case where nv_noe elem stereoX is 1 by converting
    # Hx[1/2] into Hx#

    } elseif {$stereo1 == 1} {
        # ambiguity code 1 indicates Ambiguity of geminal atoms or geminal methyl
        # proton groups
        set aname1 [string range $aname1 0 [expr [string length $aname1] - 2] ]
        append aname1 #
    }

    if {[string first "resid $res1 and name $aname1" $result2] == -1} {
        if {$i > 0} {
            append result2 " or "
        }
        append result2 "resid $res1 and name $aname1"
        incr i
    }
    
    append result2 ") "
    append result2 "$upper [expr {$upper-$lower}] 0.0"

    if {($aname0 == $aname1) && ($res0 == $res1)} {
        return -1
    } else {
        return "$result1 $result2"
    }
}

proc Nv::Noe::GetAtomPairs {list pk} {
    set labels [nv_peak labels $list]
    set relations [nv_peak relation $list]
    set nProtons 0
    set listname [lindex [nv_peak list] $list]
    foreach label $labels relation $relations {
        
        set ambiguous 0
        
        # replace 'methyl' specifiers with 'hx1' specifiers
        # and deal with multi-assigned peaks
        set assign($label) [string toupper [nv_peak elem $label.L $list.$pk]]
        set assign($label) [Nv::Noe::DoPseudo $assign($label)]
        
        if {$assign($label)==-1} {
            error "Unable to process $list.$pk"
        }
        
        set hasProton 0
        
        foreach atom $assign($label) {
            if {[regexp {.*\.[hHqQ].*} $atom]} {
                set hasProton 1
            }
        }
        
        if {$hasProton} {
            
            # nProtons is the number of dimensions that have been found to contain at least one proton
            
            incr nProtons
            
            
            set protons($nProtons) $assign($label)
            
            if {[regexp {([Dd])([1234])} $relation all d idx]} {
                incr idx -1
                set hlabel [lindex $labels $idx]
                set hetatoms($nProtons) [Nv::Noe::DoPseudo [string toupper [eval nv_peak elem $hlabel.L $list.$pk]]]
            } else {
                set hetatoms($nProtons) -
            }
            
        }
    }
    
    if {$nProtons > 2} {
        errmsg "list $list peak $pk : more than two dimensions contain proton assignments"
        return -1
    }
    if {$nProtons < 2} {
        set protons(2) "?"
        set hetatoms(2) "?"
    }
    
    if {$nProtons == 0} {
        set protons(1) "?"
        set hetatoms(1) "?"
    }
    
    set nProtons1 [llength $protons(1)]
    set nProtons2 [llength $protons(2)]
    
    if { ($nProtons1 == 1) && ($nProtons2 == 1) } {
        # simple case
        return [list $protons(1) $hetatoms(1) $protons(2) $hetatoms(2)]
    }
    
    if { ($nProtons1 > 1) && ($nProtons2 > 1) && ($nProtons1 != $nProtons2) } {
        # the two lists are of unequal length, and both are longer than 1
        errmsg "list $list peak $pk : number of protons don't match"
        return -1
    }
    
    # if we get here, one of the lists is of length one, and the other of length
    # greater than one.  In this case, the element of length one is assumed to be
    # paired with each element of the list of length greater than one.
    
    # case one:  first dimension is of length one
    if { $nProtons1 == 1 } {
        while { [llength $protons(1)] < $nProtons2 } {
            lappend protons(1) [lindex $protons(1) 0]
            lappend hetatoms(1) [lindex $hetatoms(1) 0]
        }
        return [list $protons(1) $hetatoms(1) $protons(2) $hetatoms(2)]
    }
    
    # case two:  second dimension is of length one
    if { $nProtons1 == 2 } {
        while { [llength $protons(2)] < $nProtons1 } {
            lappend protons(2) [lindex $protons(2) 0]
            lappend hetatoms(2) [lindex $hetatoms(2) 0]
        }
        return [list $protons(1) $hetatoms(1) $protons(2) $hetatoms(2)]
    }
    
    # should never get here
    errmsg "unknown error processing list $list peak $pk"
    return -1
    
}

proc Nv::Noe::DoPseudo {assignments} {
    # deal with ambiguous assignments (a colon-separated list)
    if {[regexp : $assignments]} {
        return $assignments
    }
    # and deal with unassigned peaks
    if {$assignments == "?"} {
        return $assignments
    }
    set nasn [llength $assignments]
    # load assignments into arrays
    for {set i 0} {$i < $nasn} {incr i} {
        set assign($i) [lindex $assignments $i]
        set res($i) [lindex [split $assign($i) .] 0]
        set aname($i) [lindex [split $assign($i) .] 1]
    }
    
    # see if we have an ambiguous assignment list from more than one residue
    set resnum $res(0)
    set same_residue 1
    for {set i 1} {$i < $nasn} {incr i} {
        if {$res($i) != $resnum} {
            set same_residue 0
        }
    }
    
    set filtered_anames ""
    set filtered_aspecs ""
    
    
    # preprocess each individual label
    # correct methyl specifiers and
    # hx2,3 specifiers (where x is a methyl c)
    # to hx1-specifiers
    # discard repeated specifiers
    
    for {set i 0} {$i < $nasn} {incr i} {
        set restype [nv_atom elem rname $assign($i)]
        
        # correct methyl specifiers
        if {[string match M* $aname($i)]} {
            set aname($i) [string trimleft $aname($i) M]
            set aname($i) Q$aname($i)
        }
        # correct methyl Hx3 specifiers trapping for TRP HE3 and HZ3
        if {([string match H*3 $aname($i)]) && ($aname($i)!="HE3") && ($aname($i)!="HZ3")} {
            set aname($i) [string trimright $aname($i) 3]
            append aname($i) 1
        }
        
        # correct methyl Hx2 specifiers (easy case)
        if {[string match H*22 $aname($i)]} {
            if {($restype=="ASN") &&($aname($i)=="HD22")} {
                # Do nothing
            } elseif {($restype=="GLN") &&($aname($i)=="HE22")} {
                # Do nothing
            } else {
                set aname($i) [string trimright $aname($i) 2]
                append aname($i) 21
            }
        }
        # correct methyl Hx2 specifiers (hard case)
        if {[string match H*12 $aname($i)]} {
            if {($restype=="ILE") &&($aname($i)=="HG12")} {
                # Do nothing
            } else {
                set aname($i) [string trimright $aname($i) 2]
                append aname($i) 1
            }
        }
        # correct Ala HB2 specifiers
        if {($restype=="ALA") &&([string match HB2 $aname($i)])} {
            set aname($i) [string trimright $aname($i) 2]
            append aname($i) 1
        }
        
        # now convert all methyl specifiers of type Hx1 to Qx
        
        if {($aname($i) == "HB1") &&($restype == "ALA")} {
            set aname($i) QB
        }
        if {$aname($i) == "HD11"} {
            set aname($i) QD1
        }
        if {$aname($i) == "HD21"} {
            set aname($i) QD2
        }
        if {($aname($i) == "HG11") &&($restype !="ILE")} {
            set aname($i) QG1
        }
        if {$aname($i) == "HG21"} {
            set aname($i) QG2
        }
        
        # Build a list of filtered atom names.
        # If there is only one residue represented, it will be used in a
        # second round of pseudoatom building
        
        if {[lsearch $filtered_anames $aname($i)] < 0} {
            lappend filtered_anames $aname($i)
        }
        
        lappend filtered_aspecs [join [list $res($i) $aname($i)] "."]
    }
    
    
    # finished preprocessing
    #
    # the aname array contains the corrected anames
    # the res array contains the residues
    #
    #
    
    # if there are 2 or more residues in the inputs, just return
    # the whole preprocessed list
    
    if { !$same_residue } {
        return $filtered_aspecs
    }
    
    # if there is only one entry in the filtered anames list, return
    # the atom_specifier corresponding to that
    # aname (this makes sure "3.HN 3.HN" is converted to "3.N"
    
    if { ([llength $filtered_anames]) == 1 } {
        return [join [list $res(0) $filtered_anames] "."]
    }
    
    # secondary processing - this section does pseudo atom conversions
    # like VAL QG1 QG2 to QQG, or methylene HB1 + HB2 -> QB
    
    set aname1 [lindex $filtered_anames 0]
    set aname2 [lindex $filtered_anames 1]
    switch $aname1 {
        HA1 {
            if {$aname2 == "HA2"} {
                return $resnum.QA
            }
        }
        HA2 {
            if {$aname2 == "HA1"} {
                return $resnum.QA
            }
        }
        HB1 {
            if {$aname2 == "HB2"} {
                return $resnum.QB
            }
        }
        HB2 {
            if {$aname2 == "HB1"} {
                return $resnum.QB
            }
        }
        HG1 {
            if {$aname2 == "HG2"} {
                return $resnum.QG
            }
        }
        HG2 {
            if {$aname2 == "HG1"} {
                return $resnum.QG
            }
        }
        HG11 {
            if {$aname2 == "HG12"} {
                return $resnum.QG1
            }
            if {$aname2 == "HG21"} {
                return $resnum.QQG
            }
        }
        HG12 {
            if {$aname2 == "HG11"} {
                return $resnum.QG1
            }
        }
        HD1 {
            if {$aname2 == "HD2"} {
                return $resnum.QD
            }
        }
        HD2 {
            if {$aname2 == "HD1"} {
                return $resnum.QD
            }
        }
        HE1 {
            if {$aname2 == "HE2"} {
                return $resnum.QE
            }
        }
        HE2 {
            if {$aname2 == "HE1"} {
                return $resnum.QE
            }
        }
        HG21 {
            if {$aname2 == "HG11"} {
                return $resnum.QQG
            }
        }
        HD11 {
            if {$aname2 == "HD21"} {
                return $resnum.QQD
            }
        }
        HD21 {
            if {$aname2 == "HD11"} {
                return $resnum.QQD
            }
        }
        HE11 {
            if {$aname2 == "HE21"} {
                return $resnum.QQE
            }
        }
        HE21 {
            if {$aname2 == "HE11"} {
                return $resnum.QQE
            }
        }
        QD1 {
            if {$aname2 == "QD2"} {
                return $resnum.QQD
            }
            if {$aname2 == "HD21"} {
                return $resnum.QQD
            }
        }
        QD2 {
            if {$aname2 == "QD1"} {
                return $resnum.QQD
            }
            if {$aname2 == "HD11"} {
                return $resnum.QQD
            }
        }
        QG1 {
            if {$aname2 == "QG2"} {
                return $resnum.QQG
            }
            if {$aname2 == "HG21"} {
                return $resnum.QQG
            }
        }
        QG2 {
            if {$aname2 == "QG1"} {
                return $resnum.QQG
            }
            if {$aname2 == "HG11"} {
                return $resnum.QQG
            }
        }
    }
    # if no match, return starting value
    return $filtered_aspecs
}

proc Nv::Noe::GetDistance {list pk} {
    set labels [nv_peak labels $list]
    set nProtons 0
    
    # Check if the peak is ambiguous or unassigned. If so, distances cannot be calculated for this peak.
    set result [GetAtomPairs $list $pk]
    set atom0 [lindex $result 0]
    set atom1 [lindex $result 2]
    if {($atom0 == "?") ||($atom1 == "?") || ([llength $atom0] > 1) || ([llength $atom1] > 1)} {
        return {"" "" ""}
    }
    
    foreach label $labels {
        
        
        set assign($label) [eval nv_peak elem $label.L $list.$pk]
        set hasProton 0
        foreach atom $assign($label) {
            if {[regexp {.*\.[hH].*} $atom]} {
                set hasProton 1
            }
        }
        if {$hasProton} {
            incr nProtons
            set protons($nProtons) $assign($label)
        }
    }
    if {$nProtons > 2} {
        errmsg "Too many dimensions of $list.$pk contain protons"
    }
    if {$nProtons < 2} {
        return {- - -}
    }
    
    if {[llength $protons(1)] != [llength $protons(2)]} {
        errmsg "Peak $list.$pk has an unequal number of assignments in the proton containing fields"
        return {- - -}
    }
    
    set maxDistance -1.0
    set minDistance 1.0e6
    set r6distance 0.0
    
    foreach atom1 $protons(1) atom2 $protons(2) {
        set distance [distance $atom1 $atom2]
        if {$distance < $minDistance} {
            set minDistance $distance
        }
        if {$distance > $maxDistance} {
            set maxDistance $distance
        }
        set r6distance [expr {$r6distance + pow($distance,-6.0)}]
    }
    
    set r6distance [format "%6.2f" [expr {pow($r6distance,-1.0/6.0)}]]
    set minDistance [format "%6.2f" $minDistance]
    set maxDistance [format "%6.2f" $maxDistance]
    
    return "$minDistance $r6distance $maxDistance"
}

proc Nv::Noe::AutoCal0 {} {
    variable NoeCal
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 1} {
            continue
        }
        nv_noe elem lower $i $NoeCal(fixCal,lower)
        nv_noe elem upper $i $NoeCal(fixCal,upper)
        nv_noe elem active $i 2
    }
}

proc Nv::Noe::AutoCal1 {} {
    variable NoeCal
    set m 0
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 1} {
            continue
        }
        set class [nv_noe elem class $i]
        set iclass [expr {($class % 10)}]
        if {$NoeCal(class,$iclass)} {
            lappend iVals [expr {[nv_noe elem $NoeCal(cal_source) $i]*[nv_noe elem scale $i]}]
            incr m
        }
    }
    if {$m > 0} {
        set iVals [lsort -real -decreasing $iVals]
        set median [lindex $iVals [expr $m/2]]
        puts "A distance of $NoeCal(medCal,R) corresponds to the median intensity of $median for $m noes"
        CalSim $median $NoeCal(medCal,R) $NoeCal(medCal,exp)
    }
}

proc Nv::Noe::AutoCalBin {} {
    variable NoeCal
    set m 0
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 1} {
            continue
        }
        set class [nv_noe elem class $i]
        set iclass [expr {($class % 10)}]
        if {$NoeCal(class,$iclass)} {
            lappend iVals [expr {abs([nv_noe elem $NoeCal(cal_source) $i]*[nv_noe elem scale $i])}]
            incr m
        }
    }
    if {$m > 4} {
        set iVals [lsort -real -decreasing $iVals]
        set intensity1 [lindex $iVals [expr $m/4]]
        set intensity2 [lindex $iVals [expr 2*$m/4]]
        set NoeCal(lowerInt) $intensity2
        set NoeCal(upperInt) $intensity1
    }
}

proc Nv::Noe::AutoCal2 {{first 0} {last 100000}} {
    variable NoeCal
    set n [nv_noe n]
    if {$last >= $n} {
        set last [expr $n-1]
    }
    set exp 6
    set sumDAll 0.0
    set sumVAll 0.0
    set n 0
    foreach jclass "0 1 2 3 4" {
        set dRev($jclass) 0.0
        set vRef($jclass) 0.0
        set sumD 0.0
        set sumV 0.0
        set m 0
        for {set i $first} {$i <= $last} {incr i} {
            set active [nv_noe elem active $i]
            if {$active != 1} {
                continue
            }
            set class [nv_noe elem class $i]
            set iclass [expr {($class % 10)}]
            if {$iclass != $jclass} {
                continue
            }
            if {$NoeCal(class,$iclass)} {
                set list [nv_noe elem list $i]
                set number [nv_noe elem number $i]
                set scale [nv_noe elem scale $i]
                set ival [nv_noe elem $NoeCal(cal_source) $i]
                
                # Added to deal with negative intensities
                set ival [expr abs($ival)]
                set distances [GetDistance $list $number]
                if {$distances == ""} {
                    continue
                }
                set d [lindex $distances 1]
                if {$d < 4.0} {
                    set sumD [expr {$sumD+pow($d,-1.0*$exp)}]
                    incr n
                    incr m
                    set sumV [expr {$sumV+$ival*$scale}]
                }
            }
        }
        if {$m > 0} {
            set dRef($jclass) [expr {pow(($sumD/$m) ,(-1.0/$exp))}]
            set vRef($jclass) [expr {$sumV/$m}]
            puts "A distance of $dRef($jclass) in class $jclass \n corresponds to an intensity of $vRef($jclass) for $m NOEs"
        }
        set sumDAll [expr {$sumDAll+$sumD}]
        set sumVAll [expr {$sumVAll+$sumV}]
    }
    set dRefAll [expr {pow(($sumDAll/$n) ,(-1.0/$exp))}]
    set vRefAll [expr {$sumVAll/$n}]
    if $NoeCal(calAll) {
        puts "A distance of $dRefAll corresponds\n to an intensity of $vRefAll for $n NOEs"
        CalSim $vRefAll $dRefAll $NoeCal(disCal,exp)
    } else {
        foreach jclass "0 1 2 3 4" {
            if {$NoeCal(class,$jclass)} {
                CalSim $vRef($jclass) $dRef($jclass) $NoeCal(disCal,exp)
            }
        }
    }
}

proc Nv::Noe::CalSimNoe {ival noeI0 noeN noeR0} {
    variable NoeCal
    set max 6.0
    set min 2.2
    set floor 0.0
    set n 6.0
    if {$ival <= $floor} {
        return $max
    }
    set C [expr $noeI0*(exp($noeN*log($noeR0)))]
    set I $C/$ival
    if {$I < 0.0} {
        return $max
    }
    set b [expr exp((1.0/$noeN)*log($I))]
    set b [expr $b + $NoeCal(fError)*$b*$b]
    if {$b>$max} {
        return $max
    }
    if {$b<$min} {
        return $min
    }
    return $b
}

proc Nv::Noe::Reset {} {
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        nv_noe elem active $i 1
    }
}

proc Nv::Noe::CalSimbin {} {
    variable NoeCal
    set methyl1 $NoeCal(methyl)
    set methyl2 [expr $NoeCal(methyl)*$NoeCal(methyl)]
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 1} {
            continue
        }
        nv_noe elem active $i 2
        set iclass [nv_noe elem class $i]
        if {$iclass > 19} {
            set methyl $methyl2
        } elseif {$iclass > 9} {
            set methyl $methyl1
        } else {
            set methyl 1.0
        }
        set ival [expr abs([nv_noe elem $NoeCal(cal_source) $i])]
        set scale [nv_noe elem scale $i]
        set ival [expr {abs($ival*$scale/$methyl)}]
        if {$ival > $NoeCal(upperInt)} {
            set bou $NoeCal(strongBou)
        } elseif {$ival < $NoeCal(lowerInt)} {
            set bou $NoeCal(weakBou)
        } else {
            set bou $NoeCal(mediumBou)
        }
        nv_noe elem upper $i $bou
        nv_noe elem lower $i $NoeCal(fixCal,lower)
    }
}
proc Nv::Noe::CalSim {noeI0 noeR0 noeN} {
    variable NoeCal
    set max 5.0
    set min 2.2
    set methyl1 $NoeCal(methyl)
    set methyl2 [expr $NoeCal(methyl)*$NoeCal(methyl)]
    set n [nv_noe n]
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 1} {
            continue
        }
        set class [nv_noe elem class $i]
        set iclass [expr {($class % 10)}]
        if {$NoeCal(class,$iclass)} {
            if {$class > 19} {
                set methyl $methyl2
            } elseif {$class > 9} {
                set methyl $methyl1
            } else {
                set methyl 1.0
            }
            set ival [nv_noe elem $NoeCal(cal_source) $i]
            set scale [nv_noe elem scale $i]
            set ival [expr {abs($ival*$scale) /$methyl}]
            set bou [CalSimNoe $ival $noeI0 $noeN $noeR0]
            nv_noe elem upper $i $bou
            nv_noe elem lower $i $NoeCal(fixCal,lower)
        }
        nv_noe elem active $i 2
    }
}
proc Nv::Noe::Plot {{gname xyNOE} {expr 1} {object xy_noe} {color red} {color2 blue}} {
    global $gname
    variable NoeCal
    upvar #0 $gname xyGraph
    if {![info exists xyGraph(canvas)]} {
        set tl .$gname
        Nv_MkXYGraph $tl $gname
    }
    if {![winfo exists $xyGraph(canvas)]} {
        set tl .$gname
        Nv_MkXYGraph $tl $gname
    }
    set methyl1 $NoeCal(methyl)
    set methyl2 [expr $NoeCal(methyl)*$NoeCal(methyl)]
    NvGraphSetWorld $gname 0 0 180 7
    set xyGraph(yAxisLeft,deltaMajor) 1
    set xyGraph(xAxisBottom,deltaMajor) 90
    set xyGraph(yAxisLeft,deltaMinor) 0.2
    set xyGraph(xAxisBottom,deltaMinor) 45
    set xyGraph(yAxisLeft,ticPrec) 1
    set xyGraph(xAxisBottom,ticPrec) 0
    set xyGraph(xAxisBottom,label) Intensity
    set xyGraph(yAxisLeft,label) Bound
    set xobj ${object}_x
    set yobj ${object}_y
    set yobj2 ${object}_y2
    catch "Nv_XYObjectSet $gname $yobj $xobj"
    catch "Nv_XYObjectAdd $gname $yobj2 $xobj"
    set xyGraph($yobj,color) $color
    set xyGraph($yobj2,color) $color2
    set n [nv_noe n]
    v_resize $xobj $n
    v_resize $yobj $n
    v_resize $yobj2 $n
    for {set i 0} {$i < $n} {incr i} {
        set active [nv_noe elem active $i]
        if {$active != 2} {
            continue
        }
        set bou [nv_noe elem upper $i]
        set iclass [nv_noe elem class $i]
        if {$iclass > 19} {
            set methyl $methyl2
        } elseif {$iclass > 9} {
            set methyl $methyl1
        } else {
            set methyl 1.0
        }
        set ival [nv_noe elem $NoeCal(cal_source) $i]
        set scale [nv_noe elem scale $i]
        set ival [expr {abs($ival*$scale) /$methyl}]
        v_setval $xobj $i $ival
        v_setval $yobj $i $bou
        set list [nv_noe elem list $i]
        set number [nv_noe elem number $i]
        set distances [GetDistance $list $number]
        if {$distances != ""} {
            set d [lindex $distances 1]
            v_setval $yobj2 $i $d
        } else {
            v_setval $yobj2 $i 0.0
        }
    }
    set ymax [lindex [v_max $yobj] 0]
    NvGraphSetWorld $gname 0 0 $ymax 7
}

proc Nv::Noe::ReadAriaList {args} {
    set nargs [llength $args]
    if {$nargs < 1} {
        set filename aria.tbl
    } else {
        set filename [lindex $args 0]
    }
    set fi [open $filename r]
    set lists [nv_peak lists]
    set elemNamesP {p spectrumName peakNumber testSet inWeight  volume volumeError intensity intensityError  hppm1 dhppm1 ppm1 dppm1  hppm2 dhppm2 ppm2 dppm2}
    set elemNamesA {a peakType fomAria curWeight  lower upper sumDistanceAve sumDistanceStd  backVolumeAria backVolumeAriaStd allAssi nta}
    set elemNamesC {c contribution fomContribution distanceAve distanceStd backVolume backVolumeStd  segid1 rnum1 rname1 hname1 hPPM1 hppmDelta1  pname1 pPPM1\
    pppmDelta1  segid2 rnum2 rname2 hname2 hPPM2 hppmDelta2  pname2 pPPM2 pppmDelta2 }
    puts read
    while {[gets $fi s] != -1} {
        if {[regexp {( *)(a)(.*)} $s all]} {
            set state A
            set m 12
        } elseif {[regexp {( *)(p)(.*)} $s all]} {
            set state P
            set m 17
        } elseif {[regexp {( *)(c)(.*)} $s all]} {
            set state C
            set m 25
        } else {
            close $fi
            error "invalid line $s"
        }
        set nelems [llength $s]
        while {$nelems < $m} {
            if {[gets $fi s1] == -1} {
                close $fi
                error "ReadAriaList:  End of file"
            }
            set s [concat $s $s1]
            set nelems [llength $s]
        }
        if {$nelems != $m} {
            error "ReadAriaList:  Invalid number of elements for aria line"
        }
        foreach elemName [set elemNames$state] value $s {
            set $elemName $value
        }
        if {$state=="P"} {
            set nContributions 0
            if {[lsearch $lists $spectrumName] == -1} {
                puts "add $spectrumName"
                lappend lists $spectrumName
                set dims p1
                if {$hppm1 != "-"} {
                    lappend dims h1
                }
                lappend dims p2
                if {$hppm2 != "-"} {
                    lappend dims h2
                }
                eval nv_peak addlist $spectrumName $dims
            }
            if {[catch "eval nv_peak status $spectrumName.$peakNumber"]} {
                set peakNum2 [nv_peak add $spectrumName]
                eval nv_peak idnum $spectrumName.$peakNum2 $peakNumber
                eval nv_peak elem int $spectrumName.$peakNumber $intensity
                eval nv_peak elem vol $spectrumName.$peakNumber $volume
                if {$ppm1 != "-"} {
                    eval nv_peak elem p1.P $spectrumName.$peakNumber $ppm1
                }
                if {$ppm2 != "-"} {
                    eval nv_peak elem p2.P $spectrumName.$peakNumber $ppm2
                }
                if {$hppm1 != "-"} {
                    eval nv_peak elem h1.P $spectrumName.$peakNumber $hppm1
                }
                if {$hppm2 != "-"} {
                    eval nv_peak elem h2.P $spectrumName.$peakNumber $hppm2
                }
            }
            set comment ""
            lappend comment "testSet $testSet inWeight $inWeight"
            nv_peak elem comment $spectrumName.$peakNumber $comment
        } elseif {$state=="A"} {
            set comment [eval nv_peak elem comment $spectrumName.$peakNumber]
            set comment [eval concat $comment "peakType $peakType fomAria $fomAria"]
            set comment [concat $comment "lower $lower upper $upper sumDistanceAve $sumDistanceAve"]
            set comment [concat $comment "sumDistanceStd $sumDistanceStd"]
            set comment [concat $comment "backVolumeAria $backVolumeAria allAssi $allAssi nta $nta"]
            nv_peak elem comment $spectrumName.$peakNumber $comment
        } elseif {$state=="C"} {
            set comment [eval nv_peak elem comment $spectrumName.$peakNumber]
            set comment [eval concat $comment "contribution $contribution fomContribution $fomContribution"]
            set comment [concat $comment "distanceAve $distanceAve distanceStd $distanceStd backVolume $backVolume backVolumeStd $backVolumeStd"]
            nv_peak elem comment $spectrumName.$peakNumber $comment
            if {$nContributions == 0} {
                nv_peak elem p1.L $spectrumName.$peakNumber $rnum1.$pname1
                nv_peak elem p2.L $spectrumName.$peakNumber $rnum2.$pname2
                if {$hppm1 != "-"} {
                    nv_peak elem h1.L $spectrumName.$peakNumber $rnum1.$hname1
                }
                if {$hppm2 != "-"} {
                    nv_peak elem h2.L $spectrumName.$peakNumber $rnum2.$hname2
                }
            }
            incr nContributions
        }
    }
}


proc Nv::Noe::NmrviewToDyana {aname resname} {
    set aname [string toupper $aname]
    
    if {($resname == "ALA") &&([string match HB* $aname])} {
        return QB
    }
    
    if {($resname == "MET") &&([string match HE* $aname])} {
        return QE
    }
    
    switch $aname {
        HB1 {
            return HB3
        }
        HG1 {
            return HG3
        }
        HG11 {
            return HG13
        }
    }
    if {($resname == "TYR") ||($resname == "HIS+") ||($resname == "TRP") ||($resname == "PHE")} {
        return $aname
    }
    switch $aname {
        HD1 {
            return HD3
        }
        HE1 {
            return HE3
        }
    }
    return $aname
}

# This procedure is called after Inactivate Redundant because that command is unaware
# of ambiguous peak handling procedures.

proc Nv::Noe::ReactivateAmbiguous {} {
    set total [nv_noe n]
    for {set i 1} {$i < $total} {incr i} {
        set list [nv_noe elem list $i]
        set peaknumber [nv_noe elem number $i]
        set result [GetAtomPairs $list $peaknumber]
        if {([llength [lindex $result 0]] > 1) ||([llength [lindex $result 2]] > 1)} {
            nv_noe elem active $i 2
        }
    }
}



################################################################################
# procedure to eliminate redundant NOE
# since April 13, 2003, the script takes into account stereospecificity
################################################################################

proc uniqpm {} {

# the variable look range specify how far ahead in the noe list 
# the script will look for a redundant assignment
# since nmrview internally sorts the noe lists, there is no need to 
# loop through the whole noe list for each restraint
# this takes a lot of time

set lookrange 25
set nbnoe [nv_noe n]
for {set i 0} {$i<$nbnoe} {incr i} {
	if {[nv_noe elem active $i]==0} {continue}
  set pair1 [uniqpmchangeofname $i]
	set atom1 [lindex $pair1 0]
	set atom2 [lindex $pair1 1]
	set intensityself [expr abs ([nv_noe elem int $i])]
	set upperbondself [nv_noe elem upper $i]
	set selfpeaklistfrom [lindex [nv_peak list] [nv_noe elem list $i]]
	
	# search for a symmetric assignment
	if {$i>[expr $nbnoe-$lookrange]} {set endat $nbnoe} else {set endat [expr $i+$lookrange]}
	for {set k [expr $i +1]} {$k<$endat} {incr k} {
		if {[nv_noe elem active $k]==0} {continue}
		set pair2 [uniqpmchangeofname $k]
		set atom3 [lindex $pair2 0]
		set atom4 [lindex $pair2 1]
		
		if {($atom1==$atom3 && $atom2==$atom4) || ($atom1==$atom4 && $atom2==$atom3)} {
			#check which one has the highest intensity if they come from the same peak list
			#othewise check which one has the highest value for the upper bond
			set peaklistfrompartner [lindex [nv_peak list] [nv_noe elem list $k]]
			
			if {$selfpeaklistfrom==$peaklistfrompartner} {
				if {$intensityself>=[expr abs([nv_noe elem int $k])]} {
					#puts "inactivating noe $i ($selfpeaklistfrom) coupled with peak $k ($peaklistfrompartner) ($atom1, $atom2) ($atom3,$atom4)"
					nv_noe elem active $i 0
				} else {
					#puts "inactivating noe $k ($peaklistfrompartner) coupled with peak $i ($selfpeaklistfrom) ($atom1, $atom2) ($atom3,$atom4)"
					nv_noe elem active $k 0
				}
			} else {
				# the peaks are from different peak list.  Judge by the upper bond
				set upperbondpartner [nv_noe elem upper $k]
				if {$upperbondself<=$upperbondpartner} {
					#puts "inactivating noe $i ($selfpeaklistfrom) coupled with peak $k ($peaklistfrompartner) (upper bound) ($atom1, $atom2) ($atom3,$atom4)"
					nv_noe elem active $i 0
				} else {
					#puts "inactivating noe $k ($peaklistfrompartner) coupled with peak $i ($selfpeaklistfrom) (upper bound) ($atom1, $atom2) ($atom3,$atom4)"
					nv_noe elem active $k 0
				}
			}
		}
	}
}

###################################
#end of procedure uniqpm
##################################
}




##############################################################################################
# the following procedure changes the name of the atoms according to stereospecificity code
# for unique noe filtering
# getting frustrated here with how nmrview does not handle stereospecific assignment right
# writting my own stuff
##############################################################################################

proc uniqpmchangeofname {noenumber} {

set numberpeaklistorigin [nv_noe elem list $noenumber]
set namepeaklist [lindex [nv_peak list] $numberpeaklistorigin]
set peaknumber [nv_noe elem number $noenumber]
set peaklabel [nv_peak label $namepeaklist]

set atoms [concat [nv_peak elem [lindex $peaklabel 0].L $namepeaklist.$peaknumber] [nv_peak elem [lindex $peaklabel 1].L $namepeaklist.$peaknumber]]

set answer ""

foreach atom $atoms {
	set atom_sstereo [nv_atom element sstereo $atom]
	set nb_of_proton [lindex $atom_sstereo 1]
	set stereospecificity [nv_atom elem stereo $atom]
	#set residuetype [nv_atom elem rname $atom]
	set dotpos [string first . $atom]
	set residuenumber [string range $atom 0 [expr $dotpos-1]]
	set protonname [string range $atom [expr $dotpos+1] end]
	set lengthname [string length $protonname]
	
	set result $atom
	
	if {$stereospecificity!=1} {
		# it's not stereospecifically assigned
		
		# if it's a methyl group and it contains four letters, delete the last two letters and replace by a pound sign
		# ex: 80.HG11 becomes 80.HG## in the case of a valine
		# if not, just delete the last letter and replace by a #
		if {$nb_of_proton==3 && $lengthname==4} {
			set result "$residuenumber.[string range $protonname 0 1]##"
		} else {
			set result "$residuenumber.[string range $protonname 0 [expr [string length $protonname] -2]]#"
		}
	} 
	
	set answer [concat $answer $result] 
}
	
return $answer
#end of procedure uniqpmchangeofname
}
