# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved
#

#############################################################
#Modifications:
#Check for folded chemical shifts
#
#
#
# written by Pascal Mercier
# MRC group on protein structure
# Department of Biochemistry
# University of Alberta
# March 21, 2003
#############################################################

proc Nv_ShowPeakId {nm} {
    if {![winfo exists $nm]} {
        Nv_MkPeakId $nm
    }
    wm deiconify $nm
    update
    update
}
proc Nv_MkPeakId {nm} {
    if {[winfo exists $nm]} {
        destroy $nm
    }
    set varnm [NvVarnm $nm]
    global $varnm Nv_Priv
    upvar #0 $varnm panvar
    
    
    toplevel $nm
    wm withdraw $nm
    frame $nm.data
    pack $nm.data -side left -fill both
    #    NvMkSpectrum2 $nm.0 left
    #    NvMkSpectrum2 $nm.1 left
    
    set fr $nm.data
    frame $fr.fr1
    frame $fr.fr2
    frame $fr.fr3
    pack $fr.fr2 -fill x
    pack $fr.fr1 -fill both -expand yes
    pack $fr.fr3 -fill x  -side bottom
    
    label $fr.fr2.climLabel -text CLimit
    entry $fr.fr2.climit -width 4
    label $fr.fr2.cutLabel -text CutOff
    entry $fr.fr2.cutoff -width 4
    label $fr.fr2.expLabel -text Exp
    entry $fr.fr2.exp -width 2
    checkbutton $fr.fr2.spec1  -text Spectrum1 -variable ${varnm}(spec1)
    checkbutton $fr.fr2.spec2  -text Spectrum2 -variable ${varnm}(spec2)
    pack $fr.fr2.climLabel $fr.fr2.climit $fr.fr2.expLabel $fr.fr2.exp $fr.fr2.cutLabel $fr.fr2.cutoff $fr.fr2.spec1 $fr.fr2.spec2 -side left
    if {!$Nv_Priv(javaMode)} {
        listbox $fr.atms -selectmode extended -font {Courier -14} -width 66
        scrollbar $fr.fr1.scroll -command "$fr.atms yview"
        pack $fr.atms -in $fr.fr1 -side left -fill both -expand yes
        #pack $fr.fr1.scroll -side right -fill y
        $fr.atms configure -selectbackground yellow
    } else {
        jscrollpane $fr.fr1.scroll
        listbox $fr.atms -selectmode extended -font {Courier -14} -width 66 -bd 2 -relief sunken
        $fr.fr1.scroll add $fr.atms
        pack $fr.fr1.scroll -side left -fill both -expand yes
        $fr.atms configure -selectionbackground yellow
    }
    #    label $fr.fr2.select  -relief sunken
    #    pack $fr.fr2.select -fill x -expand yes
    
    button $fr.fr3.config -text "Configure Folding" -command "NvConfigurePeak"
    button $fr.fr3.id -text Identify  -command "NvIdPeak"
    button $fr.fr3.assign -text Assign -command "NvAssignPeak"
    button $fr.fr3.close -text Close -command "destroy $nm"
    pack  $fr.fr3.config $fr.fr3.id $fr.fr3.assign $fr.fr3.close -side left -expand yes
    #    bind $fr.atms  <Button-1> "$fr.fr2.select configure -text \[$fr.atms get \[$fr.atms nearest %y\] \]"
    bind $fr.atms  <Double-Button-1> "NvShowIdPeak \[$fr.atms nearest %y\]"
    $fr.atms config -yscrollcommand "$fr.fr1.scroll set"
}

proc NvIdPeak {} {
    global Nv_Priv curr_list curr_peak    
    global FoldingConfiguration
    
    if {[info exists Nv_Priv(noID)] && $Nv_Priv(noID)} {
        set Nv_Priv(noID) 0
        return
    }
    set nm $Nv_Priv(peakId)
    set fr $nm.data
    set labels [nv_peak label $curr_list]
    set patterns [nv_peak pattern $curr_list]
    set relations [nv_peak relation $curr_list]
    set tolerances [nv_peak idtol $curr_list]
    set ndim [llength $labels]
    set search ""
    set ppms ""
   
    set newResult ""

	#initialize variables
	for {set i 1} {$i<=4} {incr i} {
		set chem_shifts($i) ""
	}
	
   #calculate folded offsets for each dimension
   set activedataset [nv_peak dataset $curr_list]
   for {set i 1} {$i<=$ndim} {incr i} {
    	set sf($i) [nv_dataset sf $activedataset $i]
		set sw($i) [nv_dataset sw $activedataset $i]
		set size($i) [nv_dataset size $activedataset $i]
		set folded_offset($i) [expr $sw($i)/$sf($i)]
		set ppminpeaklist($i) [nv_peak elem [lindex $labels [expr $i-1]].P $curr_list.$curr_peak]
		
		#calculate all the shifts for that dimension
		#make sure first the peak list has been configured for folding
		#if not, just define folding as off for all dimensions
		
		if {![info exists FoldingConfiguration($curr_list,foldonoff,$i)]} {
			set FoldingConfiguration($curr_list,foldonoff,$i) 0
		}
		
		if {$FoldingConfiguration($curr_list,foldonoff,$i)} {
			#folding is on for this dimension
			if {![info exists FoldingConfiguration($curr_list,order,$i)]} {
				set FoldingConfiguration($curr_list,order,$i) 0
			}
			#make sure the order is not set to an empty string
			if {[llength $FoldingConfiguration($curr_list,order,$i)]==0} {set FoldingConfiguration($curr_list,order,$i) 0}
			set numstep $FoldingConfiguration($curr_list,order,$i)
		} else {
			#there is no folding needed for this dimension
			set numstep 0	
		}
		
		for {set j -$numstep} {$j<=$numstep} {incr j} {
			lappend chem_shifts($i) [expr $ppminpeaklist($i)-$j*$folded_offset($i)]
		} 
	}
   
  
   #now need to combine all of the shift combinaisons into a list
   set chems_shifts_combinaison ""
   # for the next foreach command below, the list length must be at least of length 1 for it to work
	# will set the shifts for the non-existant dimensions to "nd"
	# this won't have any consequence later
   for {set i 2} {$i<=4} {incr i} {
   	if {$chem_shifts($i)==""} {set chem_shifts($i) "nd"}
   }
   foreach ppm1 $chem_shifts(1) {
   	 foreach ppm2 $chem_shifts(2) {
   	 	 foreach ppm3 $chem_shifts(3) {
   	 	 	 foreach ppm4 $chem_shifts(4) {
   	 	 	 	lappend chems_shifts_combinaison "$ppm1 $ppm2 $ppm3 $ppm4"
   	 	 	 }
   	 	 }
   	 }
   }
   
  

#evalutate some variables here used for later
set dataset [nv_peak dataset $curr_list]
set cutoff [string trim [$fr.fr2.cutoff get]]
if {$cutoff == ""} {set cutoff 100000.0}
for {set i 0} {$i<$ndim} {incr i} {
  set tolerance($i) [string trim [lindex $tolerances $i]]
  set label($i) [lindex $labels $i]
  set pattern($i) [string trim [lindex $patterns $i]]
  set relation($i) [string trim [lindex $relations $i]]
}


set idResult "" 
foreach set_of_shifts $chems_shifts_combinaison {
   
   set search ""
   set useDims 0
   
   # this block builds a search string	
    for {set i 0} {$i<$ndim} {incr i} {
       
        if {[expr {$tolerance($i) > 30.0}]} {continue}
        incr useDims
      
        set ppm [lindex $set_of_shifts $i]
        lappend ppms $ppm
        
        if {$pattern($i)==""} {set pattern($i) *}
        if {$pattern($i)=="."} {set pattern($i) *}
       
        if {([string length $relation($i)]>0) && ($relation($i) != ".")} {
            lappend search "$pattern($i)\($relation($i)\)@${ppm}~$tolerance($i)"
	    } else {
            lappend search "$pattern($i)@${ppm}~$tolerance($i)"
        }
    }

    set ndim $useDims
    update idletasks
    set result [eval nv_idpeak -thresh $cutoff $search]
	set idResult [concat $idResult $result]
}


    set iDis [expr {$ndim*2+3}]
    set exp [string trim [$fr.fr2.exp get]]
    if {$exp == ""} {
        set exp 6.0
    }
    set sum6 0.0
    set dis6s ""
 
 
    foreach line $idResult {
        set dis [lindex $line $iDis]
        if {$dis == 0.0} {
            set dis6  0.0;
        } else {
            set dis6 [expr {pow($dis,-$exp)}]
        }
        lappend dis6s $dis6
        set sum6 [expr {$sum6+$dis6}]
    }
    foreach dis6 $dis6s line $idResult {
        if {$sum6 > 0.0} {
            set ratio [expr {$dis6/$sum6}]
        } else {
            set ratio 0.0
        }
        lappend newResult "$line [format %6.4f $ratio]"
    }
    #   foreach atms $idResult {
        #	set satom [lindex $atms 1]
        #	set sppm [nv_atom elem ppm [nv_atom elem parent $satom]]
        #	set fppms "[lindex $ppms 1] [lindex $ppms 0] $sppm"
        #        lappend newResult [concat $atms [eval "nv_peak find noesy3dn $fppms"]]
    #   }
    #   NvFillIdPeaks $newResult
  

 #clear the existing list
$fr.atms delete 0 end

 NvFillIdPeaks $newResult    
      update idletasks

#end of proc NvIdPeak
}







proc NvFillIdPeaks {idResult} {
    global Nv_Priv  curr_list
    set nm $Nv_Priv(peakId)
    set fr $nm.data
    #    $nm.fr2.select configure -text ""
    set labels [nv_peak label $curr_list]
    set ndim [llength $labels]
    $fr.atms delete 0 end
    set nID [llength $idResult]
    set disIndex [expr {$ndim*2+1}]
    set cutoff [string trim [$fr.fr2.cutoff get]]
    if {$cutoff == ""} {
        set cutoff 500.0
    }
    set climit [string trim [$fr.fr2.climit get]]
    if {$climit == ""} {
        set climit 1.0
    }
    set atms [lindex $idResult 0]
    set width [string length $atms]
    if {$width > 0} {
        $fr.atms configure -width $width
    }
    set idResult [lsort -decreasing -index end -real $idResult]
    set csum 0.0
    set nSet 0
    foreach atms $idResult {
        set dis [lindex $atms $disIndex]
        set cval [lindex $atms end]
        if {$csum < $climit} {
            incr nSet
        }
        set csum [expr {$csum+$cval}]
        $fr.atms insert end $atms
    }
    if {!$Nv_Priv(javaMode)} {
        if {$nID > [$fr.atms cget -height]} {
            pack $fr.fr1.scroll -side right -fill y
        } else {
            pack forget $fr.fr1.scroll
        }
    }
    $fr.atms selection clear 0 end
    for {set i 0} {$i < $nSet} {incr i} {
        $fr.atms selection set $i
    }
    $fr.atms activate 0
    if {[llength $idResult]>0} {
        $fr.atms activate 0
        #			$fr.fr2.select configure -text [$fr.atms get 0]
    }
    update idletasks
}

proc NvShowIdPeak {idx} {
    global Nv_Priv  curr_list curr_peak
    set nm $Nv_Priv(peakId)
    set varnm [NvVarnm $nm]
    global $varnm
    upvar #0 $varnm panvar
    if {!$panvar(spec1) && !$panvar(spec2)} {return}
    set fr $nm.data
    set dataset [nv_peak dataset $curr_list]
    set labels [nv_peak labels $curr_list]
    set nDim [llength $labels]
    set tolerances [nv_peak idtol $curr_list]
    set patterns [nv_peak pattern $curr_list]
    set i 0
    set j 1
    foreach pattern1 $patterns {
        set atom [string range $pattern1 [expr [string first "." $pattern1]+1] end]
        set atom [string tolower $atom]
        if {![string match $atom h]} {
            set hetero $i
        }
        incr i
    }
    set i 0
    foreach pattern1 $patterns {
        set res1 [string range $pattern1 0 [expr [string first "." $pattern1]-1]]
        set ok 1
        foreach pattern2 [lrange $patterns $j end]  {
            set res2 [string range $pattern2 0 [expr [string first "." $pattern2]-1]]
            if {$res2 == $res1} {
                set ok 0
                break
            }
        }
        if {$ok} {
            set oddPat $i
            break
        }
        incr i
        incr j
    }
    set dims "x y z"
    set odims "x y z"
    foreach i "0 1 2" {
        if {($i != $oddPat) && ($i != $hetero) } {
            set xdim $i
            break;
        }
    }
    set dims [lreplace $dims $oddPat $oddPat y]
    set dims [lreplace $dims $hetero $hetero z]
    set dims [lreplace $dims $xdim $xdim x]
    set odims [lreplace $odims $oddPat $oddPat x]
    set odims [lreplace $odims $hetero $hetero z]
    set odims [lreplace $odims $xdim $xdim y]
    
    set ppms ""
    set atoms [lrange [$fr.atms get $idx] 0 [expr $nDim-1]]
    
    foreach atom $atoms label $labels dim "x y z" tol $tolerances {
        set appm$dim [nv_atom elem ppm $atom]
        set cppm [eval nv_peak elem $label.P $curr_list.$curr_peak]
        set ppm($dim,1) [expr $cppm-2.0*$tol]
        set ppm($dim,2) [expr $cppm+2.0*$tol]
    }
    
    foreach dim "x y z" {
        lappend ppms $dim
        lappend ppms "$ppm($dim,1) $ppm($dim,2)"
    }
    if {$panvar(spec1)} {
        if {![winfo exists $nm.0]} {
            NvMkSpectrum2 $nm.0 left
        }
        nv_win active $nm.0
        nv_win dataset $dataset
        nv_win xdim 1
        nv_win ydim 2
        update idletasks
        nv_win ppm x $ppm(x,1) $ppm(x,2)
        nv_win ppm y $ppm(y,1) $ppm(y,2)
        nv_win ppm z $ppm(z,1) $ppm(z,2)
        NvUpdateSpecAttr
        nv_win crosshair 1 $appmx $appmy 2
        nv_win draw
    }
    
    if {$panvar(spec2)} {
        if {![winfo exists $nm.1]} {
            NvMkSpectrum2 $nm.1 left
        }
        set ppms ""
        set atom [lindex $atoms $oddPat]
        set atom [nv_atom elem parent $atom]
        set appmz [nv_atom elem ppm $atom]
        set ppm(z,1) $appmz
        set ppm(z,2) $appmz
        foreach dim $odims {
            lappend ppms $dim
            lappend ppms "$ppm($dim,1) $ppm($dim,2)"
        }
        nv_win active $nm.1
        NvUpdateSpecAttr
        $nm.1 dataset $dataset
        nv_win active $nm.1
        $nm.1 xdim [incr oddPat]
        $nm.1 ydim [incr xdim]
        $nm.1 ppm x $ppm(x,1) $ppm(x,2)
        $nm.1 ppm y $ppm(y,1) $ppm(y,2)
        $nm.1 ppm z $ppm(z,1) $ppm(z,2)
        NvUpdateSpecAttr
        # $nm.1 cross1  $appmy $appmx
    }
    if {$panvar(spec2)} {
        nv_win crosshair 2 $appmx $appmy 2
        $nm.1 draw
    }
}

proc NvAssignPeak {{srchResults {}}} {
    global Nv_Priv curr_list curr_peak
    set Nv_Priv(noID) 1
    set nm $Nv_Priv(peakId)
    set fr $nm.data
    set selected [$fr.atms curselection]
    puts $selected
    set atomSets ""
    foreach element $selected {
        lappend atomSets [$fr.atms get $element]
    }
    puts $atomSets
    set labels [nv_peak label $curr_list]
    set ndim [llength $labels]
    set tolerances [nv_peak idtol $curr_list]
    set j 0
    for {set i 0} {$i<$ndim} {incr i} {
        puts $j
        set tolerance [string trim [lindex $tolerances $i]]
        if {[expr {$tolerance > 30.0}]} {
            continue
        }
        set label [lindex $labels $i]
        set assign ""
        foreach atomSet $atomSets {
            puts $atomSet
            puts [lindex $atomSet $j]
            lappend assign [lindex $atomSet $j]
        }
        puts $assign
        nv_peak elem $label.L $curr_list.$curr_peak $assign
        puts [nv_peak elem $label.L $curr_list.$curr_peak]
        incr j
    }
    #	nv_database sync
    NvUpdatePeakAnal $Nv_Priv(peakAnal)
    set Nv_Priv(noID) 0
}




proc NvConfigurePeak {} {
 if {![winfo exists .configurefolding]} {
        Nv_Mkconfigurefolding
    }
    wm deiconify .configurefolding
    update
}


proc Nv_Mkconfigurefolding {} {

global FoldingConfiguration
global curr_list

set peaklabel [nv_peak label $curr_list]
set ndim [llength $peaklabel]

toplevel .configurefolding
wm title .configurefolding "Configure folding for $curr_list"

frame .configurefolding.options -bd 2 -relief solid
pack .configurefolding.options -fill x -side top 

label .configurefolding.options.labeldim(0) -text "Dimensions:"
grid .configurefolding.options.labeldim(0) -row 1 -column 0 -sticky nw  -ipadx 15

label .configurefolding.options.buttondim(0) -text "Calculate Folding"
grid  .configurefolding.options.buttondim(0) -row 2 -column 0 -sticky nw  -ipadx 15

label .configurefolding.options.orderdim(0) -text "Order of Folding"
grid  .configurefolding.options.orderdim(0) -row 3 -column 0 -sticky nw  -ipadx 15


for {set i 1} {$i<=$ndim} {incr i} {
	label .configurefolding.options.labeldim($i) -text [lindex $peaklabel [expr $i-1]]  
	grid  .configurefolding.options.labeldim($i) -row 1 -column $i -sticky n -padx 5
	
	checkbutton .configurefolding.options.buttondim($i) -variable FoldingConfiguration($curr_list,foldonoff,$i)
	grid        .configurefolding.options.buttondim($i) -row 2 -column $i -sticky n
	
	entry .configurefolding.options.orderdim($i) -width 2 -textvariable FoldingConfiguration($curr_list,order,$i)
	grid  .configurefolding.options.orderdim($i) -row 3 -column $i -sticky n
}

button .configurefolding.ok -text "OK" -width 6 -command {destroy .configurefolding}
pack .configurefolding.ok -side bottom
}

